/*******************************************************************************
* Licensed Materials - Property of HCL
* (c) Copyright HCL Technologies Ltd. 2017, 2018.  All Rights Reserved.
*
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
*******************************************************************************/

/**
 * An object representing a transformation configuration.
 * @typedef {object} TCObject
 * @property {string} isAbstract True if the TC is abstract, false otherwise
 * @property {string} bodyExtension 
 * @property {string} buildLibraryArguments 
 * @property {string} buildLibraryCommand 
 * @property {string} capsulesToTest 
 * @property {string} codeSyncEnabled 
 * @property {string} commonPreface 
 * @property {string} commonPrefaceIfdef 
 * @property {string} compilationMakeArguments 
 * @property {string} compilationMakeCommand 
 * @property {string} compilationMakeInsert 
 * @property {string} compilationMakeType 
 * @property {string} compilationTopMakeArguments 
 * @property {string} compilationTopMakeCommand 
 * @property {string} compileArguments 
 * @property {string} compileCommand 
 * @property {string} compileIndividually 
 * @property {string} constantDefinitions 
 * @property {string} constantProvider 
 * @property {string} contextSensitiveLibraryBuild 
 * @property {string} copyrightText 
 * @property {string} defaultArguments 
 * @property {string} docCommentStyle 
 * @property {string} environment
 * @property {TCObject} eval Holds an object with evaluated properties taking into account inherited and default values.
 * With tc.eval.propertyName you can access the actual property value used for build,
 * while tc.propertyName would give the value specified for current tc or 'null' if the field is not set (without applying defaults or computing inheritance).
 * @property {string} executableName 
 * @property {string} externalCommand
 * @property {string} externalCleanCommand
 * @property {string} externalConfigurationName 
 * @property {string} externalContainer 
 * @property {string} fastLaneForFrequent 
 * @property {string} genUserCodeQualifiers 
 * @property {string} generateClassInclusions 
 * @property {string} generateMakefileForExternalProject 
 * @property {string} headerExtension 
 * @property {string} includeFilename 
 * @property {string} inclusionPaths 
 * @property {string} jetTransformationId 
 * @property {string} language 
 * @property {string} languagePreference 
 * @property {string} libraries 
 * @property {string} libraryName 
 * @property {string} linkArguments 
 * @property {string} linkCommand 
 * @property {string} linkOrder 
 * @property {string} createTargetProject
 * @property {string[]} parents Inherited TCs. Each parent TC is specified by means of its Eclipse platform URL that is on the form 'platform:/resource/PROJECT/TC_FILENAME'.
 * @property {string} prefix 
 * @property {string[]} prerequisites Prerequisites for the TC. Each prerequisite TC is specified by means of its Eclipse platform URL that is on the form 'platform:/resource/PROJECT/TC_FILENAME'.
 * @property {string} progesssData 
 * @property {string} shortUnitInclude
 * @property {string[]} sources List of source elements for the TC. Each source element is specified by means of its URI.
 * @property {string} sourceLanguagePrefix 
 * @property {string} sourceRestriction 
 * @property {string} sourceSubdirectory 
 * @property {string} supportForFileArtifacts 
 * @property {string} targetConfiguration 
 * @property {string} targetConfigurationName 
 * @property {string} targetLocation 
 * @property {string} targetRestriction 
 * @property {string} targetServicesLibrary 
 * @property {string} testContainerName 
 * @property {string} testsRunnerName 
 * @property {string} threads 
 * @property {string} topCapsule 
 * @property {string} translators 
 * @property {CppTransformType} type The type of TC.
 * @property {string} unitName 
 * @property {string} unitSubdirectory 
 * @property {string} useAbsolutePaths 
 * @property {string} useGenericToolChain 
 * @property {string} userLibraries 
 * @property {string} userObjectFiles 
 * @property {string} targetProject 
 */

/**
 * An object representing a control for enabling a certain build variant in the {{DOC_PRODUCT}} user interface.
 * @typedef {object} UIControl
 * @property {string} kind Specifies the kind of control. Currently two kinds of controls are supported: "checkbox" and "dropdown". 
 */

/**
 * An object representing an alternative for an enumeration build variant. It corresponds to an item in a dropdown menu in the user interface.
 * @typedef {object} Alternative
 * @property {string} name Name of this alternative. Corresponds to the menu item label in the dropdown menu.
 * @property {string} script Build variant script file to be invoked by the model compiler when this alternative has been chosen for
 * the enumeration build variant. This script file must have one or both of the global functions {@link preProcess} or {@link postProcess} defined.
 * If a relative path is used to specify the script file it will be interpreted relative to the folder with build variants declaration script.
 * @property {object[]} [args] Optional array of arguments to pass to the script file when this alternative has been chosen for the enumeration build variant.
 * @property {string} [description] Optional description of the alternative. This text will appear as a tooltip of the dropdown control in
 * the user interface when this particular alternative has been chosen.
 * @property {boolean} [defaultValue] Specifies if this alternative should be chosen by default for the enumeration build variant.
 */

/**
 * Transformation configuration types for C++
 * @enum {string}
 */
var CppTransformType = {
    /** C++ Library */    
    Library: '',
    /** C++ External Library */
    ExternalLibrary: '',
    /** C++ Executable */    
    Executable: '',
    /** C++ Test Executable */
    TestExecutable: ''
};

/**
 * This is the entry point of a Build Variants script. It must be present and gets called by the {{DOC_PRODUCT}} user interface
 * when a TC is about to be built (or transformed). The implementation of the function should construct JavaScript objects
 * that describe the controls that will be present in the user interface for selecting which variant of the application to build.
 * @param {TCObject} tc Transformation configuration that is about to be built (or transformed)
 */
function initBuildVariants(tc) { }

/** 
 * Global "Build Variant Framework" object. This object is not available in build variant scripts that run in the context of the model compiler.
 * @namespace
 */
var BVF = {};

/**
 * An object representing a variation in how an application can be built (a so called "build variant").
 * This object specifies which control to use in the {{DOC_PRODUCT}} user interface for enabling that particular build variant.
 * It also specifies which script the model compiler should invoke when the build variant is enabled (either in the user interface
 * or by means of command-line options for the model compiler).
 * @typedef {object} BuildVariant
 * @property {string} name The name of the build variant
 * @property {string} script Build variant script file to be invoked by the model compiler when this particular build variant is enabled.
 * This script file must have one or both of the global functions {@link preProcess} or {@link postProcess} defined.
 * If a relative path is used to specify the script file it will be interpreted relative to the folder with build variants declaration script.
 * This property is applicable for boolean build variants.
 * @property {object[]} [args] Optional array of arguments to pass to the script file. This property is applicable for boolean build variants.
 * @property {UIControl} [control] Optional specification of the control to use for this build variant in the {{DOC_PRODUCT}} user interface.
 * If omitted the structure of the BuildVariant object determines which control to use.
 * @property {string|boolean} [defaultValue] Optional default value for the build variant. For a boolean (checkbox) this should be either
 * true or false, while for an enumeration (dropdown) it should be the name of one of the alternatives (enumeration literals).
 * Note that for enumerations you can also specify the default value using the "defaultValue" property of an {@link Alternative}.
 * @property {string} [description] Optional description of what the build variant does. This text will appear as a tooltip of the
 * build variant control in the user interface.
 * @property {Alternative} [alternatives] Describes the alternatives for an enumeration (dropdown) build variant. Each alternative corresponds
 * to an item in a dropdown menu in the user interface.
 */

/**
 * An object representing a set of common utility functions and corresponding script file with implementations.
 * This set of utilities can be added in {@link initBuildVariants} function and later they will be automatically evaluated
 * and available in all build variants implementation files.
 * @typedef {object} CommonUtils
 * @property {string} name The name of common utils set
 * @property {string} script A script file with common utility functions implementations. If a relative path is used to specify the script file
 * it will be interpreted relative to the folder with build variants declaration script.
 * @property {string} [description] Optional description of the utilities included into the set.
 */

/**
 * A build configuration is a set of build variants that are enabled for a build. It has a textual representation which is a semicolon-
 * separated list of build variants, where each boolean build variant is identified by its name, and each enumeration build variant is
 * identified by its name followed by an equal sign and then followed by the name of one of its alternatives. For example:
 * "Debug; Target=Linux". 
 * @typedef {object} BuildConfiguration
 * @property {string} name The name of the build configuration. It corresponds to an item in the Configuration dropdown menu in the {{DOC_PRODUCT}}
 * user interface.
 * @property {string} config The textual representation of the build configuration as explained above.
 * @property {string} [description] A description of the build configuration. It will appear as a tooltip when the build configuration
 * has been selected in the {{DOC_PRODUCT}} user interface.
 */

/**
 * Prints an informational message. When executed in the context of the {{DOC_PRODUCT}} user interface the message will be printed in the Build Variants console.
 * When executed in the context of the model compiler the message will be printed as a regular model compiler message with information severity.
 * @param {string} msg Message to be printed
 * @param {...object} [args] Arguments that matches placeholders in the msg string. The type of the argument should match the type of placeholder, for example %s for string etc.
 * 
 * @memberof BVF
 */
BVF.formatInfo = function (msg, args) { }

/**
 * Prints a warning message. When executed in the context of the {{DOC_PRODUCT}} user interface the message will be printed in the Build Variants console.
 * When executed in the context of the model compiler the message will be printed as a regular model compiler message with warning severity.
 * @param {string} msg Message to be printed
 * @param {...object} [args] Arguments that matches placeholders in the msg string. The type of the argument should match the type of placeholder, for example %s for string etc.
 * 
 * @memberof BVF
 */
BVF.formatWarning = function (msg) { }

/**
 * Prints an error message. When executed in the context of the {{DOC_PRODUCT}} user interface the message will be printed in the Build Variants console.
 * When executed in the context of the model compiler the message will be printed as a regular model compiler message with error severity.
 * @param {string} msg Message to be printed
 * @param {...object} [args] Arguments that matches placeholders in the msg string. The type of the argument should match the type of placeholder, for example %s for string etc.
 * 
 * @memberof BVF
 */
BVF.formatError = function (msg) { }

/**
 * Adds one or several build variants to be available for the current build.
 * Each build variant object describes a control in the {{DOC_PRODUCT}} user interface that can be used for enabling a particular build variation.
 * The argument objects will be validated to ensure they are well-formed BuildVariant objects.
 * @param {...BuildVariant} buildVariants One or several objects describing a build variant.
 *
 * @example
 * BVF.add({
 *   name: 'Debug',
 *   script: 'debug.js',
 *   description: 'Debug build'
 * });
 *
 * @memberof BVF
 */
BVF.add = function (buildVariants) { }

/**
 * Adds one or several predefined build configurations. Each predefined build configuration will appear initially in the Configuration
 * dropdown menu in the {{DOC_PRODUCT}} user interface.
 * @param {...BuildConfiguration} buildConfigurations One or several objects describing predefined build configurations.
 *
 * @example
 * BVF.addPredefConfigs({
 *   name:'My config',
 *   config:'Debug;Target=Linux'
 * });
 * 
 * @memberof BVF
 */
BVF.addPredefConfigs = function (buildConfiguration) { }

/**
 * Sets a default build configuration. It will be initially selected in the Configuration dropdown menu in the {{DOC_PRODUCT}} user interface.
 * It is enough to specify the "name" property of the {@link BuildConfiguration} since that is what is used for identifying it.
 * @param {BuildConfiguration} buildConfiguration An object describing the default build configuration.
 *
 * @example
 * BVF.setDefaultConfig({name:'My config'});
 *
 * @memberof BVF
 */
BVF.setDefaultConfig = function (buildConfiguration) { }

/**
 * Adds one or several sets with common utility functions. Should be called from {@link initBuildVariants} function.
 *
 * @param {...CommonUtils} commonUtils One or several objects describing common utilities.
 *
 * @example
 * let myUtils = {
 *   name: 'My utils',
 *   script: 'myUtils.js'
 * }
 * function initBuildVariants(tc) {
 *   BVF.addCommonUtils(myUtils);
 *   ...
 * }
 * 
 * @memberof BVF
 */
BVF.addCommonUtils = function (commonUtils) { }

/** 
 * Global "Transformation Configuration Framework" object. This object is only available in build variant scripts that run in the context of the model compiler.
 * @namespace
 */
var TCF = {};

/**
 * Returns an object with global settings. This object can be retrieved at any time during the build process.
 * It can for example be used as a means to store information across different calls of build variant scripts.
 * @return {object} Object with global settings.
 *
 * @example
 * if (TCF.globals().myStringProperty == 'hello') {
 *   TCF.globals().myIntProperty = 1;
 * };
 *
 * @memberof TCF
 */
TCF.globals = function () { }

/**
 * The main method to create new transformation configuration. The first created transformation configuration in a file is considered as root TC for this file.
 * This method returns an object implementing Map<Object,Object> interface.
 * Each created transformation has string ID.
 * The first (root) transformation defined in a file get ID that matches URI of file (frameworks attempts to compute platform:/resource URI for file).
 * @param {string} descriptorId Predefined kind of Transformation Configuration that will be created
 * @return {TCObject} An object representing newly created Transformation Configuration
 *
 * @example
 * let tc = TCF.define(TCF.CPP_TRANSFORM);
 * tc.x = value;
 * var transformId = tc.getId();
 *
 * @memberof TCF
 */
TCF.define = function (descriptorId) { }

/**
 * Returns array of all prerequisites of given TC in depth-first order. The last element is the top TC itself.
 * This method performs loading of all prerequisite and parents TCs.
 * @param {TCObject} topTC Top Transformation Configuration
 * @return {TCObject[]} An ordered list of all prerequisites for given TC, starting from inner-most, ending with top TC.
 *
 * @example
 * var prereqs = TCF.orderedGraph(tc);
 * for (i = 0; i < prereqs.length; ++i) {
 *    var arguments = prereqs[i].compileArguments;
 * }
 *
 * @memberof TCF
 */
TCF.orderedGraph = function (topTC) { }

/**
 * Returns top TC for current transformation.
 * It can be used from prerequisite TCs to check some properties from top transformation configuration.
 * @return {TCObject} Top Transformation Configuration for current transformation
 *
 * @example
 * let tc = TCF.define(TCF.CPP_TRANSFORM);
 * let topTC = TCF.getTopTC().eval;
 * tc.targetServicesLibrary = topTC.targetServicesLibrary;
 *
 * @memberof TCF
 */
TCF.getTopTC = function () { }

/**
 * Resolves platform resource URI and returns an absolute file path
 *
 * @param {string} ref platform:/resource URI
 * @return {string} An absolute path to resource, or null if resolution failed.
 *
 * @memberof TCF
 */
TCF.resolveURI = function (ref) { }

/**
 * This function returns a path to the folder that contains JavaScript file with build variants declarations.
 * Build variants implementation JavaScript files and files with commonly used utils are usually also stored in that folder.
 *
 * @return {string} An absolute path to the folder that contains build variants declaration file
 *
 * @example
 * // Load JavaScript file with specific user-defined utility functions
 * load(TCF.buildVariantsFolder() + '/myUtils.js');
 *
 * @memberof TCF
 */
TCF.buildVariantsFolder = function () { }

/**
 * This function returns build variants declaration file specified in global preferences.
 * It can be used for information printout.
 *
 * @return {string} Build variants declaration JavaScript file
 *
 * @memberof TCF
 */
TCF.buildVariantsScript = function () { }

/**
 * This function can be defined in a build variant script in order to perform some actions just before the default processing of the built TC.
 * Note that since the TC has not yet been processed, it is not possible to read its settings. What you typically can do in this function
 * is to set-up values for global variables that are referenced inside the built TC or its prerequisites. Thereby you can affect the evaluation
 * of such TC settings which in turn will have an effect on the build.
 * @param {string} projectName Name of Eclipse project to which the built TC belongs.
 * @param {...any} args Custom arguments passed to the function by means of the {@link BuildVariant.args} property in the {@link BuildVariant} object.
 */
function preProcess(projectName, args) { }

/**
 * This function can be defined in a build variant script in order to perform some actions just after the default processing of the built TC.
 * You can manipulate all settings in the built TC, and its prerequisites, in order to accomplish the desired build.
 * @param {TCObject} topTC TC that will be built. This is the "top" TC, i.e. the TC selected to be built (as opposed to a prerequisite TC).
 * @param {TCObject[]} allTCs An array consisting of all TCs that will be built. This is the list of all prerequisite TCs followed by the "top" TC itself.
 * @param {...any} args Custom arguments passed to the function by means of the "args" property in the {@link BuildVariant} object.
 */
function postProcess(topTC, args) { }

/**
 * This function can be used to evaluate additional JavaScript files with customer-specific global utility functions in the context of build variant implementation JS file.
 * You can put a number of global helpful functions into some JS file and load it in one of the first lines of your build variants implementation files wherever you need them.
 * You can also access environment variables with standard Java system functions to get some common paths.
 *
 * @param {string} path Full path to the JavaScript file that will be evaluated in the current context.
 *
 * @example
 * let env = java.lang.System.getenv();
 * load(env.MY_BUILD_VARIANTS_LOCATION + '/myUtils.js');
 * load('/opt/common/bvutils/debugprint.js');
 * load(TCF.buildVariantsFolder() + '/myUtils.js');
 *
 */
function load(path) { }

/** 
 * Global object with utilities related to messages. This object is only available in build variant scripts that run in the context of the model compiler.
 * @namespace
 */
var MSG = {};

/**
 * Prints an informational message as a regular model compiler message with information severity when model compiler is loading TCs for build.
 * When executed in the context of the {{DOC_PRODUCT}} user interface the message will appear in the UML Development console where model compiler build log is usually printed.
 * @param {string} msg Message to be printed
 * @param {...object} [args] Arguments that matches placeholders in the msg string. The type of the argument should match the type of placeholder, for example %s for string etc.
 * 
 * @memberof MSG
 */
MSG.formatInfo = function (msg, args) { }

/**
 * Prints a warning message as a regular model compiler message with warning severity when model compiler is loading TCs for build.
 * When executed in the context of the {{DOC_PRODUCT}} user interface the message will appear in the UML Development console where model compiler build log is usually printed.
 * @param {string} msg Message to be printed
 * @param {...object} [args] Arguments that matches placeholders in the msg string. The type of the argument should match the type of placeholder, for example %s for string etc.
 * 
 * @memberof MSG
 */
MSG.formatWarning = function (msg) { }

/**
 * Prints an error message when model compiler is loading TCs for build.
 * It will add a message with error severity to model compiler message list and will hence cause termination of the model compiler with error status.
 * When model compiler is executed in the context of the {{DOC_PRODUCT}} user interface the messages will appear in the UML Development console
 * where model compiler build log is usually printed.
 * @param {string} msg Message to be printed
 * @param {...object} [args] Arguments that matches placeholders in the msg string. The type of the argument should match the type of placeholder, for example %s for string etc.
 * 
 * @memberof MSG
 */
MSG.formatError = function (msg) { }
