/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTActor_inl__
#define __RTActor_inl__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTActor_h__
#include <RTActor.h>
#endif

#ifndef __RTActorClass_h__
#include <RTActorClass.h>
#endif

#ifndef __RTActorRef_h__
#include <RTActorRef.h>
#endif

#ifndef __RTActor_class_h__
#include <RTActor_class.h>
#endif

#ifndef __RTMessage_h__
#include <RTMessage.h>
#endif

RTS_INLINE int RTActor::getCurrentState( void ) const
{
    return state->current;
}

RTS_INLINE int RTActor::getHistory( int s ) const
{
#if RTStateId_MaxSize == 1
    return state->history.s1[ s - 1 ];
#elif RTStateId_MaxSize == 2
    return ( state->idShift == 0 ) ? state->history.s1[ s - 1 ] : state->history.s2[ s - 1 ];
#else
    return ( state->idShift == 0 ) ? state->history.s1[ s - 1 ]
	 : ( state->idShift == 1 ) ? state->history.s2[ s - 1 ]
	 :                           state->history.s4[ s - 1 ];
#endif
}

RTS_INLINE RTActorRef * RTActor::getReference( void ) const
{
    return homeReference;
}

RTS_INLINE int RTActor::getId( void ) const
{
    return getReference()->getId();
}

RTS_INLINE int RTActor::getIndex( void ) const
{
    return homeIndex;
}

RTS_INLINE int RTActor::index( void ) const
{
    return getIndex() + 1;
}

RTS_INLINE void RTActor::saveHistory( void )
{
}

#if ! OBSERVABLE

RTS_INLINE void RTActor::rtgChainBegin( int, const char * )
{
}

RTS_INLINE void RTActor::rtgTransitionBegin( void )
{
}

RTS_INLINE void RTActor::rtgTransitionEnd( void )
{
}

#endif // ! OBSERVABLE

RTS_INLINE void RTActor::setCurrentState( int s )
{
    state->current = static_cast<RTStateId>(s);
}

RTS_INLINE const RTMessage * RTActor::getMsg( void )
{
    return msg;
}

RTS_INLINE int RTActor::sendCopyToMe( const RTMessage * m )
{
    return context()->receiveMsgCopy(m, context() );
}

RTS_INLINE const char * RTActor::getTypeName( void ) const
{
    return getActorData()->type->name;
}

RTS_INLINE RTVersionId RTActor::getVersion( void ) const
{
    return getActorData()->type->version;
}

RTS_INLINE int RTActor::isSameType( const RTActorClass * other ) const
{
    return getActorData()->type == other;
}

RTS_INLINE int RTActor::isActive( void ) const
{
    return msg != nullptr;
}

RTS_INLINE RTController * RTActor::context( void ) const
{
    return rts;
}

RTS_INLINE RTController * RTActorRef::context( void )
{
    return getOwner()->context();
}

RTS_INLINE RTController * RTMessage::context( void ) const
{
    return receiver()->context();
}

RTS_INLINE RTController::Error RTActor::getError( void ) const
{
    return context()->getError();
}

#if SUSPEND
RTS_INLINE int RTActor::isSuspended( void ) const
{
    return suspendMsgs != nullptr;
}
#endif

#endif // __RTActor_inl__
