/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTActorClass_h__
#define __RTActorClass_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTVersionId_h__
#include <RTVersionId.h>
#endif

class  RTActor;
class  RTActorRef;
struct RTComponentDescriptor;
class  RTController;
struct RTRelayDescriptor;

typedef RTActor * ( * RTActorCreator )( RTController *, RTActorRef * );

/** A type descriptor providing information about the external characteristics of a capsule.
 A static variable of this struct will be automatically generated for each capsule in the model. It
 has the same name as the capsule. The most common use for a capsule type descriptor is with the
 Frame service; see for example Frame::base::incarnate().
 Type descriptors for other types are implemented by means of the RTObject_class struct.
 */
struct RTActorClass
{
	/** Type descriptor for the super capsule, from which the described capsule inherits. */
    const RTActorClass      * super;

    /** Capsule name as defined in the model. */
    const char              * name;

    /** The version id of the capsule. */
    RTVersionId               version;

    /** The number of service ports (i.e. the externally visible relay ports that constitute the
     communication interface of the capsule). */
    int                       numRelays;

    /** An array of objects describing the service ports of the capsule (i.e. the externally visible
     relay ports that constitute the communication interface of the capsule). */
    const RTRelayDescriptor * relays;

    /** Constructor function for creating an instance of (i.e. incarnating) the capsule. */
    RTActorCreator            create;

    /** Determines if the described capsule is the same, or a subtype of, another capsule.
	 @param other Type descriptor for the other capsule to check the described capsule against.
	 @return 1 if yes, 0 if no.
	 */
    int isKindOf( const RTActorClass * other ) const;

    /** Determines if the described capsule is compatible with a certain capsule part.
     For an optional or plugin capsule part the actual type of a capsule instance that is added to
     the capsule part may or may not be exactly the same as the type of the capsule part. A capsule
	 part has a model property "Substitutable Type" and if it's true, then the type of a capsule
	 instance only has to be compatible with the type of the capsule part. If instead the property
	 is false, then the type of the instance has to match the type of the capsule part exactly.
	 @param cp Object describing a capsule part.
	 @return 1 if yes, 0 if no.
	 */
    int substitutableFor( const RTComponentDescriptor * cp) const;
};

extern const RTActorClass EmptyActorClass;

#endif // __RTActorClass_h__
