/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTActorFactory_h__
#define __RTActorFactory_h__

#include <functional>

class RTController;
class RTActorRef;
class RTActor;

#include "RTActorFactoryInterface.h"

/** A capsule factory where capsule instances are created by means of a provided 'create' function, and deleted
 	using the regular delete operator.
 */
class RTActorFactory final : public RTActorFactoryInterface {
	RTActorFactory() = delete;

public:
	/** Create a capsule factory which will use the provided function to implement capsule instance creation.
	 @param[in] createFunc A function which performs creation of a capsule instance
	 */
	RTActorFactory(std::function<RTActor* (RTController *, RTActorRef *, int)> createFunc);

	RTActor* create(RTController *rts, RTActorRef *ref, int index) override;

	void destroy(RTActor* actor) override;

private:
	std::function<RTActor* (RTController *, RTActorRef *, int)> creator;
};

#endif // __RTActorFactory_h__
