/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTActorFactoryInterface_h__
#define __RTActorFactoryInterface_h__

/** Interface of a capsule factory responsible for creating and destroying capsule instances in a capsule part */
class RTActorFactoryInterface
{
public:
    /** Called by the TargetRTS to create a capsule instance, which later will be inserted into a capsule part.
     @param[in] rts The controller that would run the new capsule instance by default
     @param[in] ref The capsule part where the created instance will later be inserted
     @param[in] index The index within the capsule part where the instance will later be inserted
     @return The created capsule instance, ready to be inserted into the capsule part.
     If nullptr is returned the TargetRTS will use the capsule's RTActorCreator for creating the instance.
     */
	virtual RTActor* create(RTController *rts, RTActorRef *ref, int index) = 0;

	/** Called by the TargetRTS to destroy a capsule instance.
	 @param[in] actor Capsule instance to be destroyed
	 */
	virtual void destroy(RTActor* actor) = 0;
};


#endif // __RTActorFactoryInterface_h__
