/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTActorId_h__
#define __RTActorId_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTDataObject_h__
#include <RTDataObject.h>
#endif

class RTActor;
class RTActorRef;

/** An object that identifies a capsule instance at run-time. It is a wrapper object for an
 RTActor pointer. */
class RTActorId : public RTDataObject
{
private:
    RTActor           * a;

public:
    /** Construction from an RTActor pointer */
	RTActorId( const RTActor * = nullptr ); //lint !e1931

	/** Construction from an RTActor reference */
	RTActorId( const RTActorId & );

    virtual             ~RTActorId( void );

						operator RTActor *( void ) const;
    RTActorId         & operator=( const RTActor * );
    RTActorId         & operator=( const RTActorId & );
    RTActor           * operator->( void ) const;

    virtual int         operator==( const RTDataObject & ) const override;
    int                 operator==( const RTActorRef   & ) const;
    int                 operator==( const RTActorId    & ) const;

    RTS_INLINE int      operator!=( const RTDataObject & ) const;
    RTS_INLINE int      operator!=( const RTActorRef   & ) const;
    RTS_INLINE int      operator!=( const RTActorId    & ) const;

    /** Determine if this object identifies a valid capsule instance. This function should be
     called immediately after incarnating a capsule instance to check if the incarnation was
     successful. Note that after destroying a capsule instance this function still returns 1
     even if the wrapped RTActor pointer then is invalid. Make sure to call resetActor() for an
     RTActorId object that identifies a capsule instance that is destroyed.
     @return 1 if the capsule instance is valid (incarnation was successful), 0 otherwise.
     */
    int                 isValid( void ) const;

    /** Return the wrapped RTActor pointer.
     @return The wrapped RTActor pointer.
     */
    RTActor           * getRTActor( void ) const;

    /** Reset the capsule instance identifier. This function should be called if the capsule
     instance is destroyed.
     */
    void                resetActor( void );

    /** Return the type descriptor for the RTActor object.
     @return Type descriptor which allows an RTActor object to be encoded, decoded, copied etc.
     */
    virtual const RTObject_class * getClassData( void ) const override;

    /** Type descriptor for the RTActor object. */
    static  const RTObject_class   classData;

#define RTType_RTActorId ( RTActorId::classData )
};

struct RTTypedValue_RTActorId
{
    const void           * data;
    const RTObject_class * type;

    RTS_INLINE RTTypedValue_RTActorId( const RTActorId      & ); //lint !e1931
    RTS_INLINE RTTypedValue_RTActorId( const RTActorId      &,
				       	   	   	   	   const RTObject_class * );
}; //lint !e1712

#if RTS_INLINES
#include <RTActorId.inl>
#endif

#endif // __RTActorId_h__
