/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTActorRef_inl__
#define __RTActorRef_inl__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTActorId_h__
#include <RTActorId.h>
#endif

#ifndef __RTActorRef_h__
#include <RTActorRef.h>
#endif

#ifndef __RTComponentDescriptor_h__
#include <RTComponentDescriptor.h>
#endif

#ifndef __RTFieldOffset_h__
#include <RTFieldOffset.h>
#endif

RTS_INLINE RTActorRef::RTActorRef( void )
: _info    ( nullptr )
, _contents( nullptr )
#if OBSERVABLE
, probe    ( nullptr )
#endif
{
}

RTS_INLINE const RTComponentDescriptor * RTActorRef::getInfo( void ) const
{
	return _info;
}

RTS_INLINE RTActor * RTActorRef::getOwner( void ) const
{
	return RTObjectOf( RTActor,
			- static_cast<long>(_info->offset),
			RTConstCast( RTActorRef, this ) );
}

RTS_INLINE int RTActorRef::size( void ) const
{
	return _info->replication;
}

RTS_INLINE int RTActorRef::getId( void ) const
{
	return _info->id;
}

RTS_INLINE const char * RTActorRef::getName( void ) const
{
	return _info->name;
}

RTS_INLINE int RTActorRef::isFixed( void ) const
{
	return _info->kind == RTComponentDescriptor::Fixed;
}

RTS_INLINE int RTActorRef::isImported( void ) const
{
	return _info->kind == RTComponentDescriptor::Imported;
}

RTS_INLINE int RTActorRef::isOptional( void ) const
{
	return _info->kind == RTComponentDescriptor::Optional;
}

RTS_INLINE RTActor * RTActorRef::operator[]( int i ) const
{
	RTActor ** link = access( i );

	if( link == nullptr )
		return nullptr;
	else
		return *link;
}

RTS_INLINE RTActor * RTActorRef::at( int i ) const
{
	return (*this)[ i - 1 ];
}

RTS_INLINE int RTActorRef::operator==( const RTActorId & rhs ) const
{
	return rhs == *this;
}

RTS_INLINE int RTActorRef::operator!=( const RTActorId & rhs ) const
{
	return operator==( rhs ) == 0;
}

#endif // __RTActorRef_inl__
