/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTAsciiDecoding_h__
#define __RTAsciiDecoding_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTDecoding_h__
#include <RTDecoding.h>
#endif

#if OBJECT_DECODE

class  RTDataObject;
class  RTIBuffer;
struct RTUnknownObject;

/** The default decoding implementation that is used for all default type descriptors.
 It reads a string produced by RTAsciiEncoding and creates the corresponding in-memory data
 representation.
 */
class RTAsciiDecoding : public RTDecoding
{
public:
    explicit    RTAsciiDecoding( RTIBuffer * );
    virtual     ~RTAsciiDecoding( void );

    // All the get/put functions return an int which is non-zero
    // if the operation was successful. Otherwise, the following
    // function may be called to discover the reason for the
    // failure.
    int         getError( void );

    virtual int get( void **, const RTObject_class ** ) override;

    // basic data types
    virtual int get_address ( void         * & ) override;
    virtual int get_bool    ( bool           & ) override;
    virtual int get_char    ( char           & ) override;
#if RTUseFloatingPoint
    virtual int get_double  ( double         & ) override;
    virtual int get_float   ( float          & ) override;
#endif
    virtual int get_int     ( int            & ) override;
    virtual int get_long    ( long           & ) override;

	virtual int get_unsignedint( unsigned int & ) override;
	virtual int get_unsignedlong( unsigned long & ) override;
	virtual int get_unsignedshort( unsigned short & ) override;
	virtual int get_unsignedchar( unsigned char & ) override;
#if 0
#if _MSC_VER > 1000
    virtual int get_long_long( __int64 &  );
#else
    virtual int get_long_long( long long &  );
#endif
#endif

    virtual int get_short   ( short          & ) override;
    virtual int get_uchar   ( unsigned char  & ) override;
    virtual int get_unsigned( unsigned       & ) override;
    virtual int get_ulong   ( unsigned long  & ) override;
    virtual int get_ushort  ( unsigned short & ) override;

    // a nul-terminated string
    virtual int get_string( char * & ) override;

    // zero or more raw bytes
    virtual int get_opaque( int &, char * & ) override;

    // an enumerated value
    virtual int get_enum( int &, int, const RTFieldDescriptor * ) override;

    // a homogeneous array of objects
    virtual int get_array( void *, int, const RTObject_class * ) override;

    // an indirect object
    virtual int get_indirect( void *, const RTObject_class * ) override;

    // a structure
    virtual int get_struct( void *, const RTObject_class * ) override;

protected:
    int                 getName( char * value, int length );
    RTUnknownObject   * getUnknown( const char * );

    int                 getNonWhite( char & );
    int                 getByte( char & );
    void                ungetByte( char );

    int                 getFieldListStart( void );
    int                 getFieldName( char * value, int length );
    int                 getFieldSeparator( void );
    int                 getFieldListFinish( void );

private:
    enum              { UngetMax = 3 };

    int                 unget_size;
    char                unget_buffer[ UngetMax ];
    RTIBuffer         * input;

    // unavailable methods
    RTAsciiDecoding( const RTAsciiDecoding & );
    RTAsciiDecoding & operator=( const RTAsciiDecoding & );
}; //lint !e1712

#endif // OBJECT_DECODE

#endif // __RTAsciiDecoding_h__
