/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTComponentDescriptor_h__
#define __RTComponentDescriptor_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTFieldOffset_h__
#include <RTFieldOffset.h>
#endif

class RTActorFactoryInterface;
class RTActor;
class RTActorRef;
class RTController;
struct RTActorClass;
struct RTBindingDescriptor;
struct RTInterfaceDescriptor;

/**
 This structure describes a capsule part (a.k.a. a component of a capsule).
 */
struct RTComponentDescriptor
{
    enum Kind { Fixed, Optional, Imported };

    /** Name of capsule part as defined in the model */
    const char                    * name;

    /** The capsule typing the capsule part (the default type when incarnating the capsule part) */
    const RTActorClass            * defaultClass;

    /** The byte offset of the associated RTActorRef within the containing class, relative to RTActor */
    RTFieldOffset                   offset;

    /** An integer, uniquely identifying this capsule part within the containing class */
    short                           id;

    /** Distinguishes between fixed, optional and imported capsule parts */
    char /* Kind */                 kind;

    /** For a non-fixed capsule part, can other capsules than the capsule part type be used when incarnating? */
    char /* boolean */              substitutable;

    /** Multiplicity (a.k.a. replication factor) of the capsule part */
    int                             replication;

    /** The number of interface requirements */
    int                             numInterfaces;

    /** An array of interface (replication) requirements */
    const RTInterfaceDescriptor   * interfaces;

    /** The number of bindings to this capsule part */
    int                             numBindings;

    /** An array of interface (protocol) requirements */
    const RTBindingDescriptor     * bindings;

    /** Optional capsule factory to be used for creating and destroying capsule instances in this capsule part */
    RTActorFactoryInterface		  * factory;
};

#endif // __RTComponentDescriptor_h__
