/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTCustomController_h__
#define __RTCustomController_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTActor_h__
#include <RTActor.h>
#endif

#ifndef __RTPeerController_h__
#include <RTPeerController.h>
#endif

typedef void ( RTActor:: * RTActorFunction )( void );

#define REGISTER_LAYER( wait, wakeup, process ) \
		(static_cast<RTCustomController *>(context()))->registerLayer( this,  \
				wait,   \
				wakeup, \
				process )

class RTCustomController : public RTPeerController
{
private:
	RTActor           * _customLayer;
	RTActorFunction     _waitFunc;
	RTActorFunction     _wakeupFunc;
	RTActorFunction     _processFunc;

public:
	RTCustomController( RTDebugger *, const char * );
	virtual             ~RTCustomController( void );

	virtual void        mainLoop( void ) override;

	// a custom actor object may register its address, and
	// the 'waitForEvents', 'wakeup' and 'processIPCData'
	// methods to be called by the RTCustomController:

	void                registerLayer(  RTActor         *,
			RTActorFunction   waitFunc,
			RTActorFunction   wakeupFunc,
			RTActorFunction   processFunc );

protected:
	virtual void        waitForEvents( void ) override;
	virtual void        wakeup( void ) override;

private:
	// unavailable methods
	RTCustomController( const RTCustomController & );
	RTCustomController & operator=( const RTCustomController & );
}; //lint !e1712

#endif // __RTCustomController_h__
