/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTDefaultActorFactory_h__
#define __RTDefaultActorFactory_h__

#include "RTActorFactoryInterface.h"

/** A capsule factory with a default implementation for how to create and destroy capsule instances.
 This default implementation is the same as is used for capsule parts without any capsule factory.
 In generated code this class is used as base class for a custom capsule factory for a capsule part,
 where "create" and/or "destroy" has a user-defined implementation.
 */
template <typename CapsuleClass>
class RTDefaultActorFactory : public RTActorFactoryInterface {
public:
	RTActor* create(RTController *rts, RTActorRef *ref, int index) override {
		return new CapsuleClass(rts, ref);
	}

	void destroy(RTActor* actor) override {
		delete actor;
	}
};

/**
 * A capsule factory with a default destroy implementation, but without a create implementation (the
 * create function will return nullptr). In generated code this class is used as base class for a custom
 * capsule factory for a capsule part where the type of the capsule part is a capsule without a
 * default capsule constructor.
 */
class RTDefaultActorFactory_NoCreate : public RTActorFactoryInterface {
public:
	RTActor* create(RTController *rts, RTActorRef *ref, int index) override {
		return nullptr;
	}

	void destroy(RTActor* actor) override {
		delete actor;
	}
};

#endif // __RTDefaultActorFactory_h__
