/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTException_h__
#define __RTException_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTExceptionSignal_h__
#include <RTExceptionSignal.h>
#endif

#ifndef __RTProtocol_h__
#include <RTProtocol.h>
#endif

struct RTTypedValue;

/** Represents the UML-RT "Exception" protocol. It contains a number of events which your RT
 application can use for representing common exceptional situations.
 */
struct Exception
{
	/** Represents an exception port through which exception events can be raised.
	 */
    class Base : public RTProtocol
    {
    public:
		RTS_INLINE Base( void );
		RTS_INLINE ~Base( void );

		static const RTProtocolDescriptor rt_class;

		enum
		{
			rti_arithmeticError = rtiLast_RTProtocol + 1
		  , rti_error
		  , rti_notFoundError
		  , rti_notUnderstoodError
		  , rti_serviceAccessError
		  , rti_streamError
		  , rti_subclassResponsibilityError
		  , rti_timeSliceError
		  , rti_userError
		};

		RTS_INLINE
		RTExceptionSignal arithmeticError( void );
		RTS_INLINE
		RTExceptionSignal arithmeticError( const RTTypedValue & );

		RTS_INLINE
		RTExceptionSignal error( void );
		RTS_INLINE
		RTExceptionSignal error( const RTTypedValue & );

		RTS_INLINE
		RTExceptionSignal notFoundError( void );
		RTS_INLINE
		RTExceptionSignal notFoundError( const RTTypedValue & );

		RTS_INLINE
		RTExceptionSignal notUnderstoodError( void );
		RTS_INLINE
		RTExceptionSignal notUnderstoodError( const RTTypedValue & );

		RTS_INLINE
		RTExceptionSignal serviceAccessError( void );
		RTS_INLINE
		RTExceptionSignal serviceAccessError( const RTTypedValue & );

		RTS_INLINE
		RTExceptionSignal streamError( void );
		RTS_INLINE
		RTExceptionSignal streamError( const RTTypedValue & );

		RTS_INLINE
		RTExceptionSignal subclassResponsibilityError( void );
		RTS_INLINE
		RTExceptionSignal subclassResponsibilityError( const RTTypedValue & );

		RTS_INLINE
		RTExceptionSignal timeSliceError( void );
		RTS_INLINE
		RTExceptionSignal timeSliceError( const RTTypedValue & );

		RTS_INLINE
		RTExceptionSignal userError( void );
		RTS_INLINE
		RTExceptionSignal userError( const RTTypedValue & );

    protected:
		enum
		{
			rtiLast_Exception = rti_userError
		};

    private:
		// unavailable methods
		Base( const Base & );
		Base & operator=( const Base & );
    };
};

#if RTS_INLINES
#include <RTException.inl>
#endif

#endif // __RTException_h__
