/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTFieldDescriptor_h__
#define __RTFieldDescriptor_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTFieldOffset_h__
#include <RTFieldOffset.h>
#endif

class  RTDecoding;
class  RTEncoding;
struct RTObject_class;
struct RTTypeModifier;

// This structure describes one component of an actor class.
//
// Field       Meaning
// -----       -------
// name        The name of the field.
// offset      The byte offset of the field within the containing class,
//             relative to RTActor or RTSequence, as appropriate.
// type        The basic type of the field.
// modifier    Type modifier for common patterns (eg. pointer to type).

// Applied to the first of a group, lookup finds a following
// descriptor with the given name, or returns nil.

struct RTFieldDescriptor
{
    const char                * name;
    RTFieldOffset               offset;
    const RTObject_class      * type;
    const RTTypeModifier      * modifier;

    const RTFieldDescriptor   * lookup( int, const char * _name ) const;

    void                        init( void * record ) const;
    void                        copy( void       * target_record,
				      const void * source_record ) const;
#if OBJECT_DECODE
    int                         decode( void       * record,
					RTDecoding * ) const;
#endif
#if OBJECT_ENCODE
    int                         encode( const void * record,
					RTEncoding * ) const;
#endif
    void                        destroy( void * record ) const;
};

#endif // __RTFieldDescriptor_h__
