/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTLog_h__
#define __RTLog_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTProtocol_h__
#include <RTProtocol.h>
#endif

class  RTDataObject;
struct RTObject_class;
class  RTString;
struct RTTypedValue;

/** Represents the UML-RT "Log" protocol */
struct Log
{
	/** Represents a log service port */
    class Base : public RTProtocol
    {
    private:
		// this will eventually be a file descriptor.
		int             file;

    public:
		RTS_INLINE      Base( void );
		RTS_INLINE      ~Base( void );

		static const RTProtocolDescriptor rt_class;

		/** Print a carriage return to the log. */
		RTS_INLINE void cr( void );

		void            crtab( int = 1 );

		// The below functions need not be called
		void            open( const char * );
		void            open( void );
		void            close( void );

		/** Print a string to the log, followed by a carriage return.
		 @param[in] str String to print to the log.
		 */
		void            log( const char * str);

		/** Print a string to the log, followed by a carriage return.
		 @param[in] str String to print to the log.
		 */
		void            log( const RTString & str);

		/** Print a string to the log.
		 @param[in] str String to print to the log.
		 */
		void            show( const char * str);

		/** Print a string to the log.
		 @param[in] str String to print to the log.
		 */
		void            show( const RTString & str);

#if OBJECT_ENCODE
		void            log( char );
#if RTUseFloatingPoint
		void            log( double );
		void            log( float );
#endif
		void            log( int );
		void            log( long );
		void            log( short );
		void            log( const RTDataObject & );

		/** Print the default ASCII encoding of an object to the log, followed by a carriage return.
		 @param[in] data Data object to encode to a string that will be printed to the log
		 @param[in] type Type descriptor for the data object. Must provide an encode function.
		 */
		void            log( const void * data, const RTObject_class * type);

		/** Print the default ASCII encoding of an object to the log, followed by a carriage return.
		 @param[in] info An RTTypedValue object which encapsules the data object to print and its
		 type descriptor. The type descriptor must provide an encode function.
		 */
		RTS_INLINE void log( const RTTypedValue & );

		void            log( unsigned int );
		void            log( unsigned long );
		void            log( unsigned short );
		void            log( unsigned char );

		void            show( char );
#if RTUseFloatingPoint
		void            show( double );
		void            show( float );
#endif
		void            show( int );
		void            show( long );
		void            show( short );
		void            show( const RTDataObject & );

		/** Print the default ASCII encoding of an object to the log.
		 @param[in] data Data object to encode to a string that will be printed to the log
		 @param[in] type Type descriptor for the data object. Must provide an encode function.
		 */
		void            show( const void * data, const RTObject_class * type);

		/** Print the default ASCII encoding of an object to the log.
		 @param[in] info An RTTypedValue object which encapsules the data object to print and its
		 type descriptor. The type descriptor must provide an encode function.
		 */
		RTS_INLINE void show( const RTTypedValue & );

		void            show( unsigned int );
		void            show( unsigned long );
		void            show( unsigned short );
		void            show( unsigned char );
#endif
		void            space( void );
		void            tab( void );
		void            clear( void );

		/** Flush all buffered characters to the log. It's important to call this function when you
		 want to see any output in the log, otherwise printed characters may only show up once the
		 underlying stream buffer is full.
		 */
		void            commit( void );

    private:
		// unavailable methods
		Base( const Base & );
		Base & operator=( const Base & );
    };
};

#if RTS_INLINES
#include <RTLog.inl>
#endif

#endif // __RTLog_h__
