/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTMessage_inl__
#define __RTMessage_inl__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTDataObject_h__
#include <RTDataObject.h>
#endif

#ifndef __RTMessage_h__
#include <RTMessage.h>
#endif

#ifndef __RTObject_class_h__
#include <RTObject_class.h>
#endif

#ifndef __RTProtocol_h__
#include <RTProtocol.h>
#endif

#ifndef __RTTypedValue_h__
#include <RTTypedValue.h>
#endif

RTS_INLINE RTMessage::RTMessage( void )
: next          ( nullptr                   )
#if OTRTSDEBUG != DEBUG_NONE
, _sendPortId   (                         0 )
, _sendPortIndex(                         0 )
, _sendProbeId  (                        -1 )
, _sendTime     (                           )
#endif
, signal        (                         0 )
, priority      (                         0 )
, _flags        (                         0 )
, _receiver     ( nullptr                   )
, _port         ( nullptr                   )
, _portIndex    (                         0 )
, _type         ( nullptr                   )
{ //lint !e1927
	data       = nullptr;
#ifdef _lint
	const_data = nullptr;
#endif
}

RTS_INLINE void RTMessage::clear( void )
{
	if( _type != nullptr )
	{
		if( ( _flags & MaskDataOwned ) != 0 )
		{
			RTObject_class::DestroyOption disposition;

#if RTMESSAGE_PAYLOAD_SIZE != 0
			if( data == _data_area )
			{
				disposition = RTObject_class::DestroyOnly;
			}
			else
#endif
			{
				disposition = RTObject_class::DestroyAndDeallocate;
			}

			_type->destroy( data, disposition );
		}

		_type  = nullptr;
	}

	data   = nullptr;
	signal = 0;
	_flags = 0;

#if OTRTSDEBUG != DEBUG_NONE
	_sendPortId    = 0;
	_sendPortIndex = 0;
	_sendProbeId   = -1;
#endif
}

RTS_INLINE RTMessage::~RTMessage( void )
{
	clear();
} //lint !e1740

RTS_INLINE int RTMessage::isRTData( void ) const
{
	return _type != nullptr
			&& _type->isKindOf( &RTDataObject::classData ) != 0;
}

RTS_INLINE void RTMessage::setDeferred( void )
{
	_flags |= MaskDeferred;
}

RTS_INLINE void RTMessage::resetDeferred( void )
{
	_flags &= ~MaskDeferred;
}

RTS_INLINE int RTMessage::isDeferred( void ) const
{
	return _flags & MaskDeferred;
}

RTS_INLINE int RTMessage::isValid( void ) const
{
	return signal != 0;
}

RTS_INLINE void RTMessage::setInvalid( void )
{
	signal = 0;
}

RTS_INLINE void RTMessage::setSignal( int s )
{
	signal = static_cast<short>(s);
}

RTS_INLINE int RTMessage::getSignal( void ) const
{
	return signal;
}

RTS_INLINE void RTMessage::setPriority( int p )
{
	priority = static_cast<unsigned char>(p);
}

RTS_INLINE int RTMessage::getPriority( void ) const
{
	return priority;
}

RTS_INLINE const RTObject_class * RTMessage::getType( void ) const
{
	return _type;
}

RTS_INLINE void * RTMessage::getData( void ) const
{
	return data;
}

RTS_INLINE void RTMessage::setData( const void * _data_,
		const RTObject_class * _type_ )
{
	const_data = _data_;
	_type      = _type_;
}

RTS_INLINE void RTMessage::setData( const RTDataObject & _data )
{
	setData( &_data, _data.getClassData() );
}

RTS_INLINE void RTMessage::internData( bool moveData )
{
	if( _type != nullptr && _type != &RTType_void )
		_internData(moveData);
}

RTS_INLINE void RTMessage::setDestination( RTActor * actor,
		RTProtocol * port,
		int          index )
{
	_receiver  = actor;
	_port      = port;
	_portIndex = index;
}

RTS_INLINE RTActor * RTMessage::receiver( void ) const
{
	return _receiver;
}

RTS_INLINE RTProtocol * RTMessage::sap( void ) const
{
	return _port;
}

RTS_INLINE int RTMessage::sapIndex0( void ) const
{
	return _portIndex;
}

RTS_INLINE int RTMessage::sapIndex( void ) const
{
	return _portIndex + 1;
}

RTS_INLINE int RTMessage::sapId( void ) const
{
	return _port->getId();
}

#endif // __RTMessage_inl__
