/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTOBuffer_h__
#define __RTOBuffer_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

/** Represents an output buffer. */
class RTOBuffer
{
public:
	RTOBuffer( void );
    virtual ~RTOBuffer( void );

    /** Write a string to the buffer.
     @param[in] data A string to write to the buffer.
     @param[in] size The size of the string to write to the buffer.
     @return The number of bytes written to the buffer
     */
    virtual int write( const char * data, int size ) = 0;

    /** Flushes the data written to the buffer.
     For some output buffers, the written data will not appear in the output until flush() has been
     called. For others explicit flushing is not needed.
     @return 1 if successful, 0 otherwise
     */
    virtual int flush( void ) = 0;

private:
    // unavailable methods
    RTOBuffer( const RTOBuffer & );
    RTOBuffer & operator=( const RTOBuffer & );
};

#endif // __RTOBuffer_h__
