/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTObject_class_inl__
#define __RTObject_class_inl__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTObject_class_h__
#include <RTObject_class.h>
#endif

RTS_INLINE const RTObject_class * RTObject_class::super( void ) const
{
    return _super;
}

RTS_INLINE const char * RTObject_class::name( void ) const
{
    return _name;
}

RTS_INLINE RTVersionId RTObject_class::version( void ) const
{
    return _version;
}

RTS_INLINE void RTObject_class::init( void * value ) const
{
    (*_init_func)( this, value );
}

RTS_INLINE void RTObject_class::copy( void * to, const void * from ) const
{
    (*_copy_func)( this, to, from );
}

RTS_INLINE void RTObject_class::move( void * to, void * from ) const
{
	// If there is no move function, fallback to use the copy function
	if (_move_func)
		(*_move_func)( this, to, from );
	else
		(*_copy_func)( this, to, from );
}

#if OBJECT_DECODE
RTS_INLINE int RTObject_class::decode( void       * value,
				       RTDecoding * coding ) const
{
    return (*_decode_func)( this, value, coding );
}
#endif

#if OBJECT_ENCODE
RTS_INLINE int RTObject_class::encode( const void * value,
				       RTEncoding * coding ) const
{
    return (*_encode_func)( this, value, coding );
}
#endif

template <typename TYPE>
const RTObject_class * RTObject_class::fromType()
{
   	return nullptr; // No type descriptor available for this type
}

#endif // __RTObject_class_inl__
