/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTOutSignal_inl__
#define __RTOutSignal_inl__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTOutSignal_h__
#include <RTOutSignal.h>
#endif

#ifndef __RTProtocol_h__
#include <RTProtocol.h>
#endif

#include <RTMessage.h>

RTS_INLINE RTOutSignal::RTOutSignal( RTProtocol           * port,
				     	 	 	 	 int                    signal,
									 const void           * data,
									 const RTObject_class * type,
									 bool				    rValueRef)
    : _port  ( port   )
    , _signal( signal )
    , _data  ( data   )
    , _type  ( type   )
	, _rValueRef (rValueRef)
{
}

RTS_INLINE int RTOutSignal::invoke( RTMessage * replyBuffers )
{
    return _port->invoke( replyBuffers, _signal, _data, _type, false );
}

RTS_INLINE int RTOutSignal::invokeAt( int index, RTMessage * replyBuffer )
{
    return _port->invokeAt( index, replyBuffer, _signal, _data, _type, false );
}

RTS_INLINE int RTOutSignal::invoke()
{
    RTMessage implicitReplyMessage;
    return _port->invoke( &implicitReplyMessage, _signal, _data, _type, true );
}

RTS_INLINE int RTOutSignal::invokeAt( int index )
{
    RTMessage implicitReplyMessage;
    return _port->invokeAt( index, &implicitReplyMessage, _signal, _data, _type, true );
}

RTS_INLINE int RTOutSignal::reply( void )
{
    return _port->reply( _signal, _data, _type );
}

RTS_INLINE int RTOutSignal::send( int priority )
{
    return _port->send( _signal, _data, _type, priority, _rValueRef );
}

RTS_INLINE int RTOutSignal::sendAt( int index, int priority )
{
    return _port->sendAt( index, _signal, _data, _type, priority, _rValueRef );
}

#endif // __RTOutSignal_inl__
