/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTRootProtocol_h__
#define __RTRootProtocol_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTInSignal_h__
#include <RTInSignal.h>
#endif

#ifndef __RTProtocol_h__
#include <RTProtocol.h>
#endif

/** Represents a capsule port which at run-time can be bound to another capsule port. This is true
 for ports typed by all user-defined protocols, but not for some of the special TargetRTS protocols
 such as Log, Frame etc. For this kind of capsule port notifications can be enabled so that the
 special event rtBound is sent when the port gets bound at run-time, and rtUnbound when it gets
 unbound.
 */
class RTRootProtocol : public RTProtocol
{
public:
    RTS_INLINE RTRootProtocol( void );
    RTS_INLINE ~RTRootProtocol( void );

    static const RTProtocolDescriptor rt_class;

    enum // "in" signals
    {
    	rti_rtBound = rtiLast_RTProtocol + 1,
		rti_rtUnbound
    };

    RTS_INLINE RTInSignal rtBound( void );
    RTS_INLINE RTInSignal rtUnbound( void );

protected:
    enum
    {
    	rtiLast_RTRootProtocol = rti_rtUnbound
    };

private:
    // unavailable methods
    RTRootProtocol( const RTRootProtocol & );
    RTRootProtocol & operator=( const RTRootProtocol & );
}; //lint !e1712

#if RTS_INLINES
#include <RTRootProtocol.inl>
#endif

#endif // __RTRootProtocol_h__
