/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTString_inl__
#define __RTString_inl__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTString_h__
#include <RTString.h>
#endif

RTS_INLINE RTString & RTString::operator=( const RTString & rhs )
{
	if( this != &rhs )
		*this = rhs.Contents;

	return *this;
} //lint !e1539

RTS_INLINE RTString & RTString::operator=( RTString && rhs )
{
	if( this != &rhs ) {
		Contents = rhs.Contents;
		rhs.Contents = nullptr;
	}

	return *this;
}

RTS_INLINE int RTString::operator!=( const RTDataObject & rhs ) const
{
    return RTDataObject::operator!=( rhs );
}

RTS_INLINE int RTString::operator==( const RTString & rhs ) const
{
    return operator==( rhs.Contents );
}

RTS_INLINE int RTString::operator!=( const RTString & rhs ) const
{
    return operator==( rhs ) == 0;
}

RTS_INLINE int RTString::operator!=( const char * rhs ) const
{
    return operator==( rhs ) == 0;
}

RTS_INLINE RTString::operator char * ( void )
{
    return Contents;
}

RTS_INLINE RTString::operator const char * ( void ) const
{
    return Contents;
}

RTS_INLINE RTString::operator void * ( void )
{
    return Contents;
}

RTS_INLINE int operator==( const char * s1, const RTString & s2 )
{
    return s2.operator==( s1 );
}

RTS_INLINE int operator!=( const char * s1, const RTString & s2 )
{
    return s2.operator==( s1 ) == 0;
}

RTS_INLINE
RTTypedValue_RTString::
RTTypedValue_RTString( const RTString & _value )
    : data( &_value )
    , type( _value.getClassData() )
{
}

RTS_INLINE
RTTypedValue_RTString::
RTTypedValue_RTString( const RTString && _value )
    : data( &_value )
    , type( _value.getClassData() )
	, rValueRef(true)
{
}

RTS_INLINE
RTTypedValue_RTString::
RTTypedValue_RTString( const RTString       & _value,
		       const RTObject_class * _type )
    : data( &_value )
    , type( _type   )
{
}

#endif // __RTString_inl__
