/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTTimerId_h__
#define __RTTimerId_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTFieldDescriptor_h__
#include <RTFieldDescriptor.h>
#endif

#ifndef __RTObject_class_h__
#include <RTObject_class.h>
#endif

class RTController;
class RTTimerNode;

/** An identifier for a timer request. The functions in Timing::Base which start timers
 (Timing::Base::informIn(), Timing::Base::informAt() and Timing::Base::informEvery()) return an
 object of this class, and you can use that object to interact with the started timer (for example
 to cancel it).
 */
class RTTimerId
{
public:
    RTS_INLINE      RTTimerId( void );
    RTS_INLINE      RTTimerId( RTTimerNode * ); //lint !e1931
    RTS_INLINE      RTTimerId( const RTTimerId & );
		    		~RTTimerId( void );

    RTTimerId     & operator=( const RTTimerId & );
    RTTimerId     & operator=( RTTimerNode * );

    RTS_INLINE int  operator==( const RTTimerId & ) const;
    RTS_INLINE int  operator!=( const RTTimerId & ) const;

    /** Check if this object identifies a valid timer request. This function does not return the
     exact state of the timer. For example, it does not tell you if the timer is still active, or if
     the timer has already timed out but the timeout event has not yet been delivered; or if the
	 timer has been canceled or a non-periodic timer has timed out. You should call this function
	 before performing any operations on the timer, to ensure that it still is valid.
	 @return 1 if the timer id identifies a valid timer request, 0 otherwise.
     */
    RTS_INLINE int  isValid( void ) const;

    /** Get the number of timeout intervals that passed before the timeout of a periodic timer
	 was actually processed. Calling this function is a way to measure the accuracy of a
	 periodic timer. For more information see Timing::Base::timeouts().
	 @return A measurement of the number of missed timeout intervals since the last time the
	 timer expired. If the timer never has expired 0 is returned. If no interval is missed (i.e.
	 the timer expired at the specified time) 1 is returned. The higher number returned, the
	 less accurate is the timer.
	 */
    int             timeouts( void ) const;

    /** Type descriptor for an RTTimerId. */
    static const RTObject_class classData;

#define RTType_RTTimerId ( RTTimerId::classData )

private:
    friend class RTTimerNode;

    static const RTFieldDescriptor fieldData[];

    struct Ring
    {
		RTS_INLINE  Ring( void );

		Ring      * prev;
		Ring      * next;
    };

    Ring            ring;
    RTTimerNode   * node;
};

struct RTTypedValue_RTTimerId
{
    const void           * data;
    const RTObject_class * type;

    RTS_INLINE RTTypedValue_RTTimerId( const RTTimerId & ); //lint !e1931
    RTS_INLINE RTTypedValue_RTTimerId( const RTTimerId &, const RTObject_class * );
}; //lint !e1712

#if RTS_INLINES
#include <RTTimerId.inl>
#endif

#endif // __RTTimerId_h__
