/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTTiming_inl__
#define __RTTiming_inl__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTInSignal_h__
#include <RTInSignal.h>
#endif

#ifndef __RTTiming_h__
#include <RTTiming.h>
#endif

#ifndef __RTTypedValue_h__
#include <RTTypedValue.h>
#endif

RTS_INLINE
Timing::Base::Base( void )
    : RTProtocol()
    , outstanding( nullptr )
{
}

RTS_INLINE
Timing::Base::~Base( void )
{
#ifdef _lint
    outstanding = nullptr;
#endif
}

RTS_INLINE
RTInSignal Timing::Base::timeout( void )
{
    return RTInSignal( this, rti_timeout );
}

RTS_INLINE
RTTimerNode * Timing::Base::informAt( const RTTimespec & when,
				      int                prio )
{
    return informAt( when, nullptr, nullptr, prio );
}

RTS_INLINE
RTTimerNode * Timing::Base::informAt( const RTTimespec   & when,
				      const RTDataObject & data,
				      int                  prio )
{
    return informAt( when, &data, data.getClassData(), prio );
}

RTS_INLINE
RTTimerNode * Timing::Base::informAt( const RTTimespec   & when,
				      const RTTypedValue & info,
				      int                  prio )
{
    return informAt( when, info.data, info.type, prio );
}

RTS_INLINE
RTTimerNode * Timing::Base::informIn( const RTTimespec     & delta,
				      const void           * data,
				      const RTObject_class * type,
				      int                    prio )
{
    return informIn( delta, RTTimespec::zero, data, type, prio );
}

RTS_INLINE
RTTimerNode * Timing::Base::informIn( const RTTimespec & delta,
				      int                prio )
{
    return informIn( delta, nullptr, nullptr, prio );
}

RTS_INLINE
RTTimerNode * Timing::Base::informIn( const RTTimespec   & delta,
				      const RTDataObject & data,
				      int                  prio )
{
    return informIn( delta, &data, data.getClassData(), prio );
}

RTS_INLINE
RTTimerNode * Timing::Base::informIn( const RTTimespec   & delta,
				      const RTTypedValue & info,
				      int                  prio )
{
    return informIn( delta, info.data, info.type, prio );
}

RTS_INLINE
RTTimerNode * Timing::Base::informEvery( const RTTimespec     & ival,
					 const void           * data,
					 const RTObject_class * type,
					 int                    prio )
{
    return informIn( ival, ival, data, type, prio );
}

RTS_INLINE
RTTimerNode * Timing::Base::informEvery( const RTTimespec   & ival,
					 const RTDataObject & data,
					 int                  prio )
{
    return informEvery( ival, &data, data.getClassData(), prio );
}

RTS_INLINE
RTTimerNode * Timing::Base::informEvery( const RTTimespec   & ival,
					 const RTTypedValue & info,
					 int                  prio )
{
    return informEvery( ival, info.data, info.type, prio );
}

RTS_INLINE
RTTimerNode * Timing::Base::informEvery( const RTTimespec & ival )
{
    return informEvery( ival, nullptr, nullptr, General );
}

#endif // __RTTiming_inl__
