/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTTypedValue_h__
#define __RTTypedValue_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

class  RTDataObject;
struct RTObject_class;

// It is common to make temporaries of this struct.
//lint -esym(1901,RTTypedValue)

/** Encapsulates a data object and its type descriptor. */
struct RTTypedValue
{
	/** Data object */
    const void           * data;

    /** Type descriptor for the data object */
    const RTObject_class * type;

    /** Construction from a data object and its type descriptor. */
    RTS_INLINE RTTypedValue( const void * data = nullptr,
			     	 	 	 const RTObject_class * type = nullptr ); //lint !e1931

    /** Construction from a data object with a type descriptor */
    RTS_INLINE RTTypedValue( const RTDataObject & ); //lint !e1931

    RTS_INLINE RTTypedValue( const RTTypedValue & );
};

#if RTS_INLINES
#include <RTTypedValue.inl>
#endif

// Overloaded functions yielding basic descriptors:
extern RTTypedValue new_RTTypedValue( bool         const & );
extern RTTypedValue new_RTTypedValue( char         const & );
#if RTUseFloatingPoint
extern RTTypedValue new_RTTypedValue( double       const & );
extern RTTypedValue new_RTTypedValue( float        const & );
#endif
extern RTTypedValue new_RTTypedValue( int          const & );
extern RTTypedValue new_RTTypedValue( long         const & );
extern RTTypedValue new_RTTypedValue( short        const & );
extern RTTypedValue new_RTTypedValue( const void * const & );
extern RTTypedValue new_RTTypedValue( const char * const & );

#endif // __RTTypedValue_h__
