/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTMessage.h>
#include <RTProtocol.h>

void RTActor::unexpectedMessage( void )
{
    RTDiagStream & out = RTDiag::out();

    out.lock();

    if( getReference() != nullptr )
	out << getName() << "(" << index() << ")";
    else
	out << getTypeName();

    out << "@" << getCurrentStateString() << " received unexpected message: ";

    const RTMessage * m = getMsg();

    if( m == nullptr )
	out << "No message pointer!\n";
    else
    {
	RTProtocol * port = m->sap();

	if( port != nullptr )
	{
	    const char * portName = port->getName();

	    if( portName != nullptr )
		out << portName << "[" << m->sapIndex() << "]%";
	}

	const char * name = m->getSignalName();

	if( name == nullptr )
	    out << "?(" << m->getSignal() << ")";
	else
	    out << name;

	out << " data: ";

#if OBJECT_ENCODE
	const RTObject_class * type = m->getType();

	if( type != nullptr )
	    out.write( m->data, type );
	else
#endif
	    out << m->data;

	out << "\n";
    }
    out.flush();
    out.unlock();
}
