/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTDebugger.h>
#include <RTMessage.h>
#include <RTSpecialPorts.h>
#include <RTSpecialSignals.h>

// initialize this actor in its RTS given that this thread is in sender

int RTActor::initMe( const void           * data,
		     const RTObject_class * type,
		     RTController         * sender )
{
#if USE_THREADS
    if( sender != context() )
	return sender->send( this, RTInitSignal, data, type, System );
    else
#endif
    {
	(void)sender; // it _is_ used

	RTMessage m;

	m.setPriority( System );
	m.setSignal( RTInitSignal );
	m.setDestination( this, nullptr, 0 );
	m.setData( data, type );

	msg = &m;

#if OBSERVABLE
	context()->debugger()->setTimestamp( &m, nullptr, 0 );

	if( probe != nullptr )
	    probe->evtMessageOutIn( &m, RTEventInfo::SendNormal );
#endif

#if LOG_MESSAGE
	logMsg();
#endif
	initializeAll();
	processHistory();

#if OBSERVABLE
	if( probe != nullptr )
	    probe->evtChain( 0 );
#endif

	msg = nullptr;

	return 1;
    }
}
