/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorRef.h>
#include <RTImportList.h>

int RTActor::isReferencedBy( const RTActor * other ) const
{
    const RTActor      * container = this;
    const RTImportList * imports;
    const RTActor      * owner;
    const RTActorRef   * homeRef;

    do
    {
    	if( container == other )
    		return 1;

    	for( imports  = container->importList; imports != nullptr; imports  = imports->used.next )
    	{
    		owner = imports->used.reference->getOwner();

    		if( owner == nullptr )
    		{
    			// Every reference has an owner, but better safe than sorry!
    		}
    		else if( owner->isReferencedBy( other ) != 0 )
    		{
    			return 1;
    		}
    	}

    	if( (homeRef = container->getReference()) == nullptr )
    		break;
    }
    while( (container = homeRef->getOwner()) != nullptr );

    return 0;
}
