/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTMessage.h>
#include <RTSpecialSignals.h>

#if 0 // these are not declared in the header file

void RTActor::suspend( RTController * initiator )
{
	if( initiator == context() )
		suspend();
	else if( ! isSuspended() )
		initiator->send( this, RTSuspendSignal, nullptr, System, 0 );
}

void RTActor::resume( RTController * initiator )
{
	if( initiator == context() )
		resume();
	else if( isSuspended() )
		initiator->send( this, RTResumeSignal, nullptr, System, 0 );
}

#endif

// These functions can be safely called only from the same context as that
// of the target behavior. When there is some possibility the target might
// be different, use the variants above.

static RTMessage suspendedMarkerMessage;

void RTActor::suspend( void )
{
	if( ! isSuspended() )
		suspendMsgs = &suspendedMarkerMessage;
}

void RTActor::resume( void )
{
	if( isSuspended() )
	{
		RTMessage * m;

		while( (m = suspendMsgs) != &suspendedMarkerMessage )
		{
			suspendMsgs = m->next;
			context()->requeueMsg( m, 1 /*front*/ );
		}
		suspendMsgs = nullptr;
	}
}
