/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifdef PRAGMA
#pragma implementation "RTBindingDescriptor.h"
#pragma implementation "RTRelayDescriptor.h"
#endif

#include <RTActorClass.h>
#include <RTBindingDescriptor.h>
#include <RTComponentDescriptor.h>
#include <RTInterfaceDescriptor.h>
#include <RTProtocolDescriptor.h>
#include <RTRelayDescriptor.h>

int RTActorClass::substitutableFor( const RTComponentDescriptor * ref ) const
{
    if( ref->substitutable != 0 )
    {
		if( isKindOf( ref->defaultClass ) == 0 )
			return 0;
	}
	else
	{
		if( this != ref->defaultClass )
			return 0;
	}

    // check replication factors
    {
		const RTRelayDescriptor     * relay    = relays;
		const RTInterfaceDescriptor * required = ref->interfaces;

		int relayIndex = ref->numInterfaces;

		if( relayIndex < 0 || relayIndex > numRelays )
			return 0;

		for( ; --relayIndex >= 0; ++relay, ++required )
			if( relay->replication < required->replication )
			return 0;
	}

    // check compatibility of protocols
    {
		const RTRelayDescriptor   * relay   = relays;
		const RTBindingDescriptor * binding = ref->bindings;

		for( int i = ref->numBindings; --i >= 0; ++binding )
		{
			int relayIndex = binding->relay;

			if( relayIndex < 0 || relayIndex >= numRelays )
			return 0;

			if( binding->protocol->compatibleWith( relay[ relayIndex ].protocol ) == 0 )
			return 0;
		}
    }

    return 1;
}
