/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActorId.h>
#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTFieldDescriptor.h>
#include <RTObject_class.h>
#include <RTnew.h>

static RTDataObject * _access( RTActorId * target )
{
	return target;
}

static void _init( const RTObject_class *, void * target )
{
	(void)new( target ) RTActorId;
}

static void _copy( const RTObject_class *,
		void                 * target,
		const RTActorId      * source )
{
	(void)new( target ) RTActorId( *source );
}

#if OBJECT_DECODE
static int _decode( const RTObject_class *,
		RTActorId            * target,
		RTDecoding           * coding )
{
	void * address;

	if( coding->get_address( address ) == 0 )
		return 0;

	*target = static_cast<RTActor *>(address);

	return 1;
}
#endif

#if OBJECT_ENCODE
static int _encode( const RTObject_class *,
		const RTActorId      * source,
		RTEncoding           * coding )
{
	return coding->put_address( source->getRTActor() );
}
#endif

static const RTFieldDescriptor _fields[] =
{
		{
				"RTActorId",
				0,
				&RTActorId::classData,
				nullptr
		}
};

const RTObject_class RTActorId::classData =
{
		&RTType_RTDataObject,
		reinterpret_cast<RTSuperAccessFunction>(_access),
		"RTActorId",
		0 /* RTVersionId */,
		sizeof( RTActorId ),
		_init,
		reinterpret_cast<RTCopyFunction>(_copy),
		nullptr, // Not movable
#if OBJECT_DECODE
		reinterpret_cast<RTDecodeFunction>(_decode),
#endif
#if OBJECT_ENCODE
		reinterpret_cast<RTEncodeFunction>(_encode),
#endif
		RTDataObject_destroy,
		1, _fields
};

#if OBJECT_DECODE
RTTypeInstaller RTActorId_installer( RTType_RTActorId );
#endif

const RTObject_class * RTActorId::getClassData( void ) const
{
	return &RTActorId::classData;
}
