/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTActor_class.h>
#include <RTAsciiDecoding.h>
#include <RTAsciiEncoding.h>
#include <RTDebugger.h>
#include <RTFieldDescriptor.h>
#include <RTMemoryInBuffer.h>
#include <RTMemoryUtil.h>
#include <RTObject_class.h>
#include <RTTruncatingBuffer.h>

void RTActorProbe::accessFieldValue( void )
{
	FieldEntry * elem = _fieldList;

	if( elem == nullptr )
		return;

	_fieldList = elem->next;
	elem->next = nullptr;

	const RTActor_class     * info      = _actor->getActorData();
	const RTFieldDescriptor * attribute = info->findAttribute( elem->varName );

	if( attribute !=nullptr )
	{
		const int            bufferSize = 9000;
		char               * buffer     = new char [ static_cast<unsigned>(bufferSize) ];
		RTTruncatingBuffer   target ( buffer, bufferSize );
		RTAsciiEncoding      encoder( &target );

		if( elem->varValue !=nullptr )
		{
			RTMemoryInBuffer source ( elem->varValue,
					RTMemoryUtil::strlen( elem->varValue ) );
			RTAsciiDecoding  decoder( &source );

			(void)attribute->decode( _actor, &decoder ); // write
		}

		// read
		(void)attribute->encode( _actor, &encoder );
		(void)target.write( "", 1 );

		(void)_debugger->evtVariable( _id, _actorIndex, _actor,
				elem->varName, buffer );

		delete [] buffer;
	}

	delete elem;
}
