/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTActorRef.h>
#include <RTActor_class.h>
#include <RTComponentDescriptor.h>
#include <RTMain.h>
#include <RTMemoryUtil.h>

RTActor * RTActorProbe::getActorFromStrPath( void )
{
    const char * _actorPath = getStringPath();

    if( _actorPath == nullptr )
	return nullptr;

    RTActor * theActor = RTMain::topActor();

    if( theActor == nullptr )
	return nullptr;

    int actorLen = RTMemoryUtil::strlen( _actorPath );

    if( _actorPath[ 0 ] != '/' ) // must start with "/"
	return nullptr;

    if( actorLen == 1 )
	return theActor;

    // we're not copying the first character ('/')
    char * actorPath = RTMemoryUtil::strdup( &_actorPath[ 1 ], actorLen - 1 );

    int    repIndex;
    char * slidePtr  = actorPath;
    char * actorPtr  = actorPath;
    char * replPtr   = nullptr;

    for(;;)
    {
	if( *slidePtr == ':' )
	{
	    if( replPtr != nullptr ) // it _should_ be null
	    {
		delete [] actorPath;
		return nullptr;
	    }

	    *slidePtr++ = '\0';
	    replPtr = slidePtr;
	}

	if( *slidePtr == '/' || *slidePtr == '\0' )
	{
	    char saveChar = *slidePtr;
	    *slidePtr = '\0';

	    if( replPtr != nullptr )
		repIndex = RTMemoryUtil::atoi( replPtr );
	    else if( saveChar == '\0' )
		repIndex = _actorIndex;
	    else
	    {
		delete [] actorPath;
		return nullptr;
	    }

	    const RTActor_class         * info = theActor->getActorData();
	    const RTComponentDescriptor * desc = info->components;

	    for( int iComp = 0; ; ++iComp, ++desc )
	    {
		if( iComp >= info->numComponents )
		{
		    delete [] actorPath;
		    return nullptr;
		}

		if( RTMemoryUtil::strcmp( desc->name, actorPtr ) == 0 )
		    break;
	    }

	    RTActorRef * actorRef = RTObjectOf( RTActorRef, desc->offset,
							    theActor );

	    if( repIndex >= desc->replication ||
		(theActor = (*actorRef)[ repIndex ]) == nullptr )
	    {
		delete [] actorPath;
		return nullptr;
	    }

	    if( saveChar == '\0' )
		break;

	    actorPtr = slidePtr + 1;
	    replPtr  = nullptr;
	}

	++slidePtr;
    }

    delete [] actorPath;

    return theActor;
}
