/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2019, 2020.All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActorProbe.h>

#include <RTActor.h>
#include <RTAsciiEncoding.h>
#include <RTDebugger.h>
#include <RTFieldDescriptor.h>
#include <RTMemoryInBuffer.h>
#include <RTMemoryUtil.h>
#include <RTTruncatingBuffer.h>

void RTActorProbe::reportFieldValues( void )
{
    for( FieldEntry * elem = _fieldMonList; elem; elem = elem->next )
    {
        const RTActor_class     * info      = _actor->getActorData();
        const RTFieldDescriptor * attribute = info->findAttribute( elem->varName );

        if( attribute != nullptr )
        {
            const int            bufferSize = 9000;
            char               * buffer     = new char [ bufferSize ];
            RTTruncatingBuffer   target ( buffer, bufferSize );
            RTAsciiEncoding      encoder( &target );

            // read
            (void)attribute->encode( _actor, &encoder );
            (void)target.write( "", 1 );

            if( elem->varValue && !RTMemoryUtil::strcmp( buffer, elem->varValue ) )
                // value is unchanged
                continue;

            delete elem->varValue;
            (void)_debugger->evtVariable( _id, _actorIndex, _actor,
                                          elem->varName, elem->varValue = buffer );
        }

    }
}
