/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActorRef.h>
#include <RTActorSlot.h>
#include <RTComponentDescriptor.h>
#include <RTImportList.h>

RTActor ** RTActorRef::access( int index ) const
{
	int replication = _info->replication;

	if( index < 0 || index >= replication )
		return nullptr;

	switch( _info->kind )
	{
	case RTComponentDescriptor::Imported:
	{
		RTImportList * slot;

		if( replication == 1 )
			slot = static_cast<RTImportList *>(_contents);
		else
		{
			RTImportList::Unused * head = static_cast<RTImportList::Unused *>(_contents);

			slot  = static_cast<RTImportList *>(static_cast<void *>(&head[ 1 ]));
			slot += index;
		}

		if( slot->used.reference != this )
			return nullptr;
		else
			return &slot->used.actor;
	}

	case RTComponentDescriptor::Optional:
		if( replication == 1 )
			return reinterpret_cast<RTActor **>(& const_cast<RTActorRef*>(this)->_contents);
		else if( RTSmallOptional::supports( replication ) != 0 )
		{
			RTSmallOptional * slot = static_cast<RTSmallOptional *>(_contents);

			slot += index + 1;

			if( slot->inuse() == 0 )
				return nullptr;
			else
				return &slot->_actor;
		}
		else
		{
			RTLargeOptional * slot = static_cast<RTLargeOptional *>(_contents);

			slot += index + 1;

			if( slot->next != nullptr )
				return nullptr;
			else
				return &slot->actor;
		}

	case RTComponentDescriptor::Fixed:
		if( replication == 1 )
			return reinterpret_cast<RTActor **>(& const_cast<RTActorRef*>(this)->_contents);
		else
			return &(static_cast<RTActor **>(_contents))[ index ];

	default:
		return nullptr;
	}
}
