/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorRef.h>
#include <RTComponentDescriptor.h>
#include <RTDiag.h>
#include <RTImportList.h>



int RTActorRef::deport( RTActor * actor, RTImportList * entry, RTController * initiator )
{

	RTAssert( actor != nullptr );

	if( entry->used.actor != actor || entry->used.reference != this )
	{
		initiator->setError( RTController::badIndex,
				RTController::FrameDeport,
				this, nullptr );
		return 0;
	}

	int replication = _info->replication;
	RTImportList * first;

	if( replication == 1 )
		first = static_cast<RTImportList *>(_contents);
	else
	{
		RTImportList::Unused * head = static_cast<RTImportList::Unused *>(_contents);

		first = static_cast<RTImportList *>(static_cast<void *>(&head[ 1 ]));
	}

	int index = static_cast<int>(entry - first);

	if( index < 0 || index >= replication )
	{
		initiator->setError( RTController::badIndex,
				RTController::FrameDeport,
				this, nullptr );
		return 0;
	}

	remove( index );

	return 1;
}

