/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorRef.h>
#include <RTActor_class.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>

int RTActorRef::destroy( RTActor * actor )
{
	// This is the initiator context
	RTController * controller = context();

	if( actor == nullptr )
	{
		controller->setError( RTController::badActor,
				RTController::FrameDestroy,
				this, nullptr );
		return 0;
	}

	int index = actor->getIndex();

	if( index < 0 || size() <= index || (*this)[ index ] != actor )
	{
		controller->setError( RTController::badIndex,
				RTController::FrameDestroy,
				this, nullptr );
		return 0;
	}

	// disable binding notifications: we will soon purge any rtUnbound messages
	const RTActor_class    * info  = actor->getActorData();
	const RTPortDescriptor * port  = info->ports;
	int                      count = info->numPorts;

	for( port = info->ports, count = info->numPorts; --count >= 0; ++port )
	{
		if( ( port->properties & RTPortDescriptor::MaskKind )
				!= RTPortDescriptor::KindSpecial )
		{
			RTProtocol * end = RTObjectOf( RTProtocol, port->offset, actor );

			end->bindingNotification( 0 );
		}
	}

	// deport actor from all of the places that imported it
	actor->removeAllImports(controller);

#if USE_THREADS && RTFRAME_THREAD_SAFE
	lock();
#endif
	actor->_unbind( nullptr, controller);
#if USE_THREADS && RTFRAME_THREAD_SAFE
	unlock();
#endif

	remove( index );

	controller->destroy( actor, getOwner() );

	return 1;
}
