/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorRef.h>
#include <RTComponentDescriptor.h>
#include <RTDiag.h>
#include <RTImportList.h>

int RTActorRef::indexOf( const RTActor * actor ) const
{
	int replication = _info->replication;
	int index       = -1;

	if( replication <= 0 )
	{
		// excluded reference
	}
	else if( _info->kind == RTComponentDescriptor::Imported )
	{
		RTImportList * entry;

		if( replication == 1 )
		{
			entry = static_cast<RTImportList *>(_contents);

			if( entry->used.reference == this && entry->used.actor == actor )
				index = 0;
		}
		else for( entry = actor->importList; ; entry = entry->used.next )
		{
			if( entry == nullptr )
				break;

			if( entry->used.reference == this )
			{
				RTAssert( entry->used.actor == actor );

				RTImportList::Unused * head;
				RTImportList         * first;

				head  = static_cast<RTImportList::Unused *>(_contents);
				first = static_cast<RTImportList *>(static_cast<void *>(&head[ 1 ]));

				index = static_cast<int>(entry - first);
				break;
			}
		}
	}
	else
	{
		index = actor->getIndex();

		if( index < 0 || index >= replication || (*this)[ index ] != actor )
			index = -1;
	}

	return index;
}
