/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActorRef.h>
#include <RTActorSlot.h>
#include <RTComponentDescriptor.h>
#include <RTDiag.h>
#include <RTImportList.h>

void RTActorRef::unreserve( int index )
{
    int replication = _info->replication;

    RTAssert( 0 <= index && index < replication );
    RTAssert( (*this)[ index ] == nullptr );

    switch( _info->kind )
    {
    case RTComponentDescriptor::Imported:
	if( replication == 1 )
	{
	    RTImportList * entry = static_cast<RTImportList *>(_contents);

	    entry->used.reference = nullptr;
	}
	else
	{
#if USE_THREADS
	    lock();
#endif

	    RTImportList::Unused * head  = static_cast<RTImportList::Unused *>(_contents);
	    RTImportList         * first = static_cast<RTImportList *>(static_cast<void *>(&head[ 1 ]));
	    RTImportList         * entry = &first[ index ];

	    entry->unused.prev      = reinterpret_cast<RTImportList *>(head);
	    entry->unused.next      = head->next;
	    head->next->unused.prev = entry;
	    head->next              = entry;

#if USE_THREADS
	    unlock();
#endif
	}
	break;

    case RTComponentDescriptor::Optional:
	if( replication > 1 ) // we have a free list
	{
#if USE_THREADS
	    lock();
#endif
	    if( RTSmallOptional::supports( replication ) != 0 )
	    {
		RTSmallOptional * head  = static_cast<RTSmallOptional *>(_contents);
		int               iNext = head->next();
		RTSmallOptional * next  = &head[ iNext ];
		RTSmallOptional * entry = &head[ ++index ];

		entry->insert( 0, iNext );
		next->prev( index );
		head->next( index );
	    }
	    else
	    {
		//lint -save -e661
		RTLargeOptional * head  = static_cast<RTLargeOptional *>(_contents);
		RTLargeOptional * entry = &head[ index + 1 ];

		entry->prev      = head;
		entry->next      = head->next;
		head->next->prev = entry;
		head->next       = entry;
		//lint -restore
	    }
#if USE_THREADS
	    unlock();
#endif
	}
	break;

    case RTComponentDescriptor::Fixed:
    default:
	break;
    }
}
