/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTAsciiDecoding.h>
#include <RTObject_class.h>

int RTAsciiDecoding::get_indirect( void * pointer,
				   const RTObject_class * target )
{
    int     status = 1;
    void ** link   = static_cast<void **>(pointer);
    void  * value  = nullptr;
    void  * trash  = nullptr;

    if( getFieldListStart() == 0 )
    {
		int tag;

		if( get_int( tag ) == 0 || tag != 0 ) 
		{
			char byte;
			get_char(byte);

			if(byte != '\0')
				status = 0;
		}
    }
    else if( (value = target->allocate()) != nullptr )
    {
	target->init( value );

	if( target->decode( value, this ) == 0 )
	    status = 0;

	if( getFieldListFinish() == 0 )
	    status = 0;
    }

    if( status == 0 )
	trash = value;
    else
    {
	trash = *link;
	*link = value;
    }

    if( trash != nullptr )
	target->destroy( trash, RTObject_class::DestroyAndDeallocate );

    return status;
}
