/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTAsciiDecoding.h>
#include <RTMemoryUtil.h>

//lint -save -esym(429,value)

int RTAsciiDecoding::get_string( char * & value )
{
	int    result      = 1;
	int    size        = 0;
	int    buffer_size = 4096;
	char * buffer;
	char   byte;

	if( getNonWhite( byte ) == 0 )
		result = 0;

	else if( byte != '"' )
	{
		ungetByte( byte );
		result = 0;
	}

	else if( (buffer = new char [ buffer_size ]) == nullptr )
	{
		result = 0;
	}

	else
	{
		for(;;)
		{
			if( getByte( byte ) == 0 )
			{
				result = 0;
				break;
			}

			if( byte == '\0' )
			{
				result = 0;
				break;
			}

			if( byte == '\\' )
			{
				if( getByte( byte ) == 0 )
				{
					result = 0;
					break;
				}
			}
			else if( byte == '"' )
			{
				value = RTMemoryUtil::strdup( buffer, size );

				if( value == nullptr )
				{
					// The buffer we have is bigger than we need,
					// but at least it has the right value.
					buffer[ size ] = '\0';
					value  = buffer;
					buffer = nullptr;
				}

				break;
			}

			buffer[ size++ ] = byte;

			if( size >= buffer_size )
			{
				buffer_size += 4096;

				char * new_buffer = new char [ buffer_size ];

				if( new_buffer == nullptr )
				{
					result = 0;
					break;
				}

				RTMemoryUtil::memcpy( new_buffer, buffer, size );
				delete [] buffer;
				buffer = new_buffer;
			}
		}

		delete [] buffer;
	}

	return result;
}

//lint -restore
