/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTByteBlock.h>
#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTFieldDescriptor.h>
#include <RTObject_class.h>
#include <RTnew.h>

static RTDataObject * _access( RTByteBlock * target )
{
	return target;
}

static void _init( const RTObject_class *, void * target )
{
	(void)new( target ) RTByteBlock;
}

static void _copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	(void)new( target ) RTByteBlock( * static_cast<const RTByteBlock *>(source) );
}

#if OBJECT_DECODE
static int _decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	int    length;
	char * Contents;

	if( coding->get_opaque( length, Contents ) == 0 )
		return 0;
	else
	{
		RTByteBlock * data = static_cast<RTByteBlock *>(target);

		delete [] data->Contents;

		data->length   = length;
		data->Contents = Contents;

		return 1;
	}
}
#endif

#if OBJECT_ENCODE
static int _encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	const RTByteBlock * data = static_cast<const RTByteBlock *>(source);

	return coding->put_opaque( data->length, data->Contents );
}
#endif

static const RTFieldDescriptor _fields[] =
{
		{
				"RTByteBlock",
				0,
				&RTByteBlock::classData, nullptr
		}
};

const RTObject_class RTByteBlock::classData =
{
		&RTType_RTDataObject,
		reinterpret_cast<RTSuperAccessFunction>(_access),
		"RTByteBlock",
		0 /* RTVersionId */,
		sizeof( RTByteBlock ),
		_init,
		_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		_decode,
#endif
#if OBJECT_ENCODE
		_encode,
#endif
		RTDataObject_destroy,
		1, _fields
};

#if OBJECT_DECODE
RTTypeInstaller RTByteBlock_installer( RTType_RTByteBlock );
#endif

const RTObject_class * RTByteBlock::getClassData( void ) const
{
	return &RTByteBlock::classData;
}
