/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTCachedString.h>
#include <RTMemoryUtil.h>

RTCachedString & RTCachedString::operator=( const RTCachedString & rhs )
{
    lock();

    if( this != &rhs && _entry != rhs._entry )
    {
	unlink( _entry );
	link( _entry = rhs._entry );
    }

    unlock();

    return *this;
}

RTCachedString & RTCachedString::operator=( const char * string )
{
    lock();

    if( _entry == nullptr )
    {
	if( string == nullptr )
	{
	    // do nothing: both are nil
	}
	else
	{
	    _entry = add( string );
	}
    }
    else
    {
	if( string == nullptr )
	{
	    unlink( _entry );
	    _entry = nullptr;
	}
	else if( RTMemoryUtil::strcmp( *this, string ) == 0 )
	{
	    // no change
	}
	else
	{
	    unlink( _entry );
	    _entry = add( string );
	}
    }

    unlock();

    return *this;
}
