/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTCharacter.h>
#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTFieldDescriptor.h>
#include <RTObject_class.h>
#include <RTnew.h>

static RTDataObject * _access( RTCharacter * target )
{
	return target;
}

static void _init( const RTObject_class *, void * target )
{
	(void)new( target ) RTCharacter;
}

static void _copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	(void)new( target ) RTCharacter( * static_cast<const RTCharacter *>(source) );
}

#if OBJECT_DECODE
static int _decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	RTCharacter * data = static_cast<RTCharacter *>(target);

	return coding->get_char( data->value );
}
#endif

#if OBJECT_ENCODE
static int _encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	const RTCharacter * data = static_cast<const RTCharacter *>(source);

	return coding->put_char( data->value );
}
#endif

static const RTFieldDescriptor _fields[] =
{
		{
				"RTCharacter",
				0,
				&RTCharacter::classData, nullptr
		}
};

const RTObject_class RTCharacter::classData =
{
		&RTType_RTDataObject,
		reinterpret_cast<RTSuperAccessFunction>(_access),
		"RTCharacter",
		0 /* RTVersionId */,
		sizeof( RTCharacter ),
		_init,
		_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		_decode,
#endif
#if OBJECT_ENCODE
		_encode,
#endif
		RTDataObject_destroy,
		1, _fields
};

#if OBJECT_DECODE
RTTypeInstaller RTCharacter_installer( RTType_RTCharacter );
#endif

const RTObject_class * RTCharacter::getClassData( void ) const
{
	return &RTCharacter::classData;
}
