/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTCmdLineObserver.h>
#include <RTDebugger.h>
#include <RTMemoryUtil.h>

static const char prompt[] = "\r\nRTS Debug ->";

int RTCmdLineObserver::receiveData( void )
{
    bool needPrompt = false;
    int  bufIndex   = 0;

    for(;;)
    {
	if( bufIndex >= bufUsed )
	{
	    int numRead = readData( &inBuffer[ bufUsed ], bufSize - bufUsed );

	    if( numRead <= 0 )
		return numRead;

	    bufUsed += numRead;
	}

	++bufIndex;

	char next = ( bufIndex < bufUsed ) ? inBuffer[ bufIndex ]
					   : '\0';

	if( inBuffer[ bufIndex - 1 ] == '\n' )
	{
	    needPrompt = true;

	    // Change each carriage-return that immediately precedes
	    // a new-line into a space. This avoids the problem of
	    // generally handling CRs elsewhere.

	    if( bufIndex > 1 && inBuffer[ bufIndex - 2 ] == '\r' )
		inBuffer[ bufIndex - 2 ] = ' ';
	}
	else if( inBuffer[ bufIndex - 1 ] == ';' )
	    inBuffer[ bufIndex -1 ] = '\n';
	else
	    continue;

	inBuffer[ bufIndex ] = '\0';

	debugger->debugIt( inBuffer );

	inBuffer[ bufIndex ] = next;

	if( (bufUsed -= bufIndex) > 0 )
	    RTMemoryUtil::memcpy( inBuffer, &inBuffer[ bufIndex ], bufUsed );

	if( ! needPrompt )
	{
	    // prompt not needed
	}
	else if( sendData( prompt, RTMemoryUtil::strlen( prompt ) ) != 0 )
	{
	    needPrompt = false;
	}

	bufIndex = 0;
    }
}
