/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifdef PRAGMA
#pragma implementation "RTController.h"
#pragma implementation "RTController.inl"
#endif

#include <RTController.h>
#include <RTDebugger.h>
#if USE_THREADS
#include <RTSyncObject.h>
#endif
#include <RTTimerNode.h>

RTController::~RTController( void )
{
#if OTRTSDEBUG != DEBUG_NONE
	debugger()->deregisterRTS( this );
#endif

	short       bits;
	short       priority;
	RTMessage * all;

#if USE_THREADS
	delete _sync;
	acceptIncoming();
#endif

	for( bits = internalBits; bits != 0; )
	{
		priority = topPriority[ bits ];
		bits ^= 1 << priority;
		all = internalQ[ priority ].dqAll();
		resourceMgr.msgFree( all );
	}

	resourceMgr.msgFree( freeList );

#if ! DEFER_IN_ACTOR
	all = deferredQ.dqAll();
	resourceMgr.msgFree( all );
#endif

	while( freeTimerList != nullptr )
	{
		RTTimerNode * node = freeTimerList;

		freeTimerList = node->next;

		delete node;
	}

#ifdef _lint
	_name         = nullptr;
#if OTRTSDEBUG != DEBUG_NONE
	_debugger     = nullptr;
	_prev         = nullptr;
	_next         = nullptr;
#endif
	freeList      = nullptr;
	freeTimerList = nullptr;
	replyBuffer   = nullptr;
#endif
}
