/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTController.h>
#if USE_THREADS
#include <RTMutex.h>
#endif
#include <RTQFilter.h>

void RTController::filterMessages( RTActor * receiver, RTQFilter & filter )
{
	short        bits;
	short        priority;
	short        priorityBit;
	RTMessageQ * queue;

#if USE_THREADS
	if( incomingBits != 0 )
		acceptIncoming();
#endif

	// filter messages in internal queues.
	for( bits = internalBits; bits != 0; )
	{
		priority    = topPriority[ bits ];
		priorityBit = static_cast<short>( 1 << priority );
		queue       = &internalQ[ priority ];

		if( queue->filterAll( filter ) != 0 && queue->isEmpty() != 0 )
			internalBits ^= priorityBit;

		bits ^= priorityBit;
	}

	// filter messages in the deferred queue
#if DEFER_IN_ACTOR
	(void)receiver->deferredQ.filterAll( filter );
#else
	(void)receiver;
	(void)          deferredQ.filterAll( filter );
#endif
}
