/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTController.h>
#if USE_THREADS
#include <RTMutex.h>
#endif
#include <RTTimerNode.h>

void RTController::freeTimerNode( RTTimerNode * node )
{
	node->clear();

#if PURIFY
	delete node;
#else

#if USE_THREADS
	_mutex->enter();
#endif

	enum
	{
		minFreeListSize =  20,
		maxFreeListSize = 100
	};

	node->next = freeTimerList;
	freeTimerList = node;

	if( ++freeTimerListSize > maxFreeListSize )
	{
		while( freeTimerListSize > minFreeListSize )
		{
			node = freeTimerList->next;
			delete freeTimerList;
			freeTimerList = node;
			--freeTimerListSize;
		}
	}

#if USE_THREADS
	_mutex->leave();
#endif

#endif // PURIFY
}
