#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTController.h>
#include <RTDaemonInfo.h>
#include <RTDebugger.h>
#include <RTEventInfo.h>
#include <RTMemoryUtil.h>
#include <RTMessage.h>
#include <RTQFilter.h>

class RTListMessagesFilter : public RTQFilter
{
private:
    const RTDaemonInfo & _info;
public:
    RTListMessagesFilter( const RTDaemonInfo & info ) : _info( info )
    {
    }
    virtual int select( RTMessage & m ) override
    {
        addEvent(&m);
        return 0;
    }
    virtual void accept( RTMessage & ) override
    {
    }
private:
    bool probeMatches( const RTActorProbe * probe )
    {
        if( !probe )
            return false;
        if( _info.actorPath &&
            ( _info.actorIndex != probe->getActorIndex() ||
              RTMemoryUtil::strcmp( _info.actorPath, probe->getStringPath( ) ) ) )
            return false;
        return true;
    }
    void addEvent( RTMessage * msg )
    {
        RTActor * receiver = msg->receiver();
        RTActorProbe * probe = receiver->probe;
        if( probeMatches( probe ) )
        {
            probe->evtMsgQueue(msg);
        }
    }
};

void RTController::getMessages( const RTDaemonInfo & daemonInfo )
{
    short        priority;

#if USE_THREADS
    if( incomingBits != 0 )
    	acceptIncoming();
#endif

    if( (priority = topPriority[ internalBits ]) >= OTRTS_NUMPRIO )
    {
        return;
    }

    RTListMessagesFilter filter( daemonInfo );

    for (; priority < OTRTS_NUMPRIO; priority++ )
    {
        RTMessageQ * queue = &internalQ[ priority ];
        if( queue->isEmpty() )
        {
            continue;
        }

        queue->filterAll (filter);
    }
}
