/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTController.h>
#include <RTDebugger.h>
#include <RTEventInfo.h>
#include <RTMessage.h>

int RTController::send( RTActor              * receiver,
		int                    controlSignal,
		const void           * data,
		const RTObject_class * type,
		int                    priority )
{
	RTMessage * msg = newMsg();

	if( msg == nullptr )
	{
		setError( noMem, SystemService, nullptr, nullptr );
		return 0;
	}

	msg->setData( data, type );
	msg->setSignal( controlSignal );
	msg->setPriority( priority );
	msg->setDestination( receiver, nullptr, 0 );
	msg->internData();

#if OBSERVABLE
	debugger()->setTimestamp( msg, nullptr, 0 );

	if( receiver->probe != nullptr )
		receiver->probe->evtMessageOutIn( msg, RTEventInfo::SendNormal );
#endif

	receiver->context()->receive( msg, this );

	return 1;
}
