/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTMemoryUtil.h>

int RTDaemon::posInfo( RTDaemonInfo * newInfo )
{
	lock.enter();
	int i = posInfo_(newInfo);
	lock.leave();
	return i;
}

int RTDaemon::posInfo_( RTDaemonInfo * newInfo )
{
	for( int iActor = 0; iActor < actorInfo.size(); ++iActor )
	{
		RTDaemonInfo * oldInfo = static_cast<RTDaemonInfo *>(actorInfo[ iActor ]);

		if( oldInfo == nullptr )
			continue;

		if( newInfo == nullptr ) // delete no matter what
			return iActor;

		if( oldInfo->daemonKind != newInfo->daemonKind )
			continue;

		int length = newInfo->allRecurs != 0
				? RTMemoryUtil::strlen( newInfo->actorPath )
		: -1;

		// actorPath must match
		if( RTMemoryUtil::strcmp( newInfo->actorPath,
				oldInfo->actorPath,
				length ) != 0 )
		{
			continue;
		}

		// actorIndex must match unless allRecurs is true
		if( newInfo->allRecurs  == 0 &&
				oldInfo->actorIndex != newInfo->actorIndex )
		{
			continue;
		}

		// InfoActor: nothing more to compare; return match
		if( newInfo->daemonKind == RTDaemonInfo::InfoActor )
			return iActor;

		// InfoPort: portPath must match, portIndex must match
		if( newInfo->daemonKind == RTDaemonInfo::InfoPort )
		{
			if( RTMemoryUtil::strcmp( newInfo->portName,
					oldInfo->portName ) != 0 )
			{
				continue;
			}

			if( oldInfo->portIndex == newInfo->portIndex )
				return iActor;
		}

		// InfoState:

		// InfoChain:

		// InfoVariable: variableName must match
		if( newInfo->daemonKind == RTDaemonInfo::InfoVariable )
		{
			if( RTMemoryUtil::strcmp( newInfo->variableName,
					oldInfo->variableName ) == 0 )
			{
				return iActor;
			}
		}

	}
	return -1;
}
