/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTDebugger.h>
#include <RTElasticArray.h>

void RTDaemon::refresh( int observerId, int daemonId )
{
	debugger->evtRtsStatus( observerId );

	int thisDepth = 0;

	// the actors should be refreshed in depth-order, otherwise the
	// Toolset might receive an instance before its reference.
	for( ;; )
	{
		int nextDepth = -1;

		lock.enter();

		for( int iActor = 0; iActor < actorInfo.size(); ++iActor )
		{
			RTDaemonInfo * daemonInfo = static_cast<RTDaemonInfo *>(actorInfo[ iActor ]);

			if( daemonInfo == nullptr )
				continue;

			int depth = daemonInfo->actorDepth;

			if( depth < thisDepth )
				continue;
			else if( depth > thisDepth )
			{
				if( nextDepth == -1 || nextDepth > depth )
					nextDepth = depth;
			}
			else // depth match
			{
				debugger->getPresence( daemonInfo->probeNr,
						observerId,
						daemonId );

				if( daemonInfo->daemonKind == RTDaemonInfo::InfoActor )
					debugger->getReference( daemonInfo->probeNr, observerId );

				if( daemonInfo->daemonKind == RTDaemonInfo::InfoState )
					debugger->getHistory( daemonInfo->probeNr,
							daemonInfo->stateTarget );
			}
		}

		lock.leave();

		if( nextDepth > -1 )
			thisDepth = nextDepth;
		else
			break;
	}
}
