/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDataObject.h>
#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void pRTDataObject_init( const RTObject_class *, void * target )
{
	* static_cast<RTDataObject **>(target) = nullptr;
}

static void pRTDataObject_copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	RTDataObject * object = * static_cast<RTDataObject * const *>(source);

	if( object != nullptr )
		object = object->copy();

	* static_cast<RTDataObject **>(target) = object;
}

#if OBJECT_DECODE
static int pRTDataObject_decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	void                 * object;
	const RTObject_class * type;

	if( coding->get( &object, &type ) == 0 )
		return 0;

	if( type->isKindOf( &RTType_RTDataObject ) == 0 )
	{
		type->destroy( object, RTObject_class::DestroyAndDeallocate );
		return 0;
	}

	* static_cast<RTDataObject **>(target) = static_cast<RTDataObject *>(object);

	return 1;
}
#endif

#if OBJECT_ENCODE
static int pRTDataObject_encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	const RTDataObject * object = * static_cast<RTDataObject * const *>(source);

	if( object == nullptr )
		object = &EmptyObject;

	return coding->put( object, object->getClassData() );
}
#endif

static void pRTDataObject_destroy( const RTObject_class *, void * target )
{
	RTDataObject * & object = * static_cast<RTDataObject **>(target);

	delete object;

	object = nullptr;
}

const RTObject_class RTType_RTpRTDataObject =
{
		nullptr,
		nullptr, nullptr, // "(RTDataObject *)",
		0 /* RTVersionId */,
		sizeof( RTDataObject * ),
		pRTDataObject_init,
		pRTDataObject_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		pRTDataObject_decode,
#endif
#if OBJECT_ENCODE
		pRTDataObject_encode,
#endif
		pRTDataObject_destroy,
		0, nullptr
};
