/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorRef.h>
#include <RTActorRefProbe.h>
#include <RTDebugger.h>
#if USE_THREADS
#include <RTMutex.h>
#endif
#include <RTProbe.h>

void RTDebugger::actorDestroyed( RTActor * actor )
{
	RTProbe * probe;

#if USE_THREADS
	_mutex->enter();
#endif

	for( int i = 0; i < probes.size(); ++i )
	{
		if( ( probe = static_cast<RTProbe *>(probes[ i ]) ) != nullptr &&
				probe->disconnectProbe( actor ) != 0 )
		{
			parkedProbes[ parkedProbes.unusedIndex() ] = probe;
			++nrParkedProbes;
		}
	}

	RTActorRef * homeRef = actor->getReference();

	if( homeRef != nullptr && homeRef->probe != nullptr )
		homeRef->probe->evtPresence( actor->getIndex(), -1, -1 );

#if USE_THREADS
	_mutex->leave();
#endif
}
