/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActorProbe.h>
#include <RTActorRefProbe.h>
#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTDebugger.h>
#include <RTMemoryUtil.h>
#include <RTProbe.h>

int RTDebugger::addProbe( RTDaemonInfo * probeInfo )
{
    RTProbe * probe;

    for( int iProbe = 0; iProbe < probes.size(); ++iProbe )
    {
	if( (probe = static_cast<RTProbe *>(probes[ iProbe ])) == nullptr )
	    continue;

	if( probe->getActorIndex() == probeInfo->actorIndex &&
	    RTMemoryUtil::strcmp( probe->getStringPath(),
				  probeInfo->actorPath ) == 0 )
	{
	    (void)probe->adjustRefCount( +1 );
	    return iProbe;
	}
    }

    int probeNr = probes.unusedIndex();

    probe = probeInfo->actorIndex < 0
	    ? new RTActorRefProbe( this, probeNr, probeInfo->actorPath )
	    : new RTActorProbe   ( this, probeNr, probeInfo->actorPath,
						  probeInfo->actorIndex );

    probeCreated( probeNr, probe );

    return probeNr;
}
