/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDebugger.h>
#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTIOMonitor.h>
#include <RTMain.h>
#include <RTMemoryUtil.h>

static const char * const listening_notice[] =
{
    "\n",
    "*******************************************************************\n",
    "*                Please note: STDIN is turned off.                *\n",
    "*   To use the command line, telnet to the above mentioned port.  *\n",
    "* The _output_ of any command will be displayed in _this_ window. *\n",
    "*******************************************************************\n",
    "\n",nullptr
};

void RTDebugger::checkObservability( void )
{
    RTDiagStream       & out       = RTDiag::out();
    int                  argc      = RTMain::argCount();
    const char * const * argv      = RTMain::argStrings();
    const char         * optListen =nullptr;

    while( ++argv, --argc > 0 )
    {
	// -obslisten[=port]
	if( RTMemoryUtil::strcmp( *argv, "-obslisten", 10 ) == 0 )
	    optListen = *argv + 10;
    }

    out.lock();

    if( optListen !=nullptr )
    {
	int port = ( *optListen == '=' )
				? RTMemoryUtil::atoi( &optListen[ 1 ] )
				: 0;

	ioMonitor = new RTIOMonitor;

	listener.registerWith( ioMonitor );
	if( listener.create() != 0 && listener.listen( port ) != 0 )
	{
	    out << "targetRTS: observability listening at tcp port "
		<< listener.getLocalPort() << "\n";
	}
	else
	{
	    listener.close();
	    out << "targetRTS: couldn't obslisten at tcp port "
		<< port << "\n";
	    delete ioMonitor;
	    ioMonitor = nullptr;
	}
    }

    if( ioMonitor == nullptr )
	out << "targetRTS: observability listening not enabled\n";
    else
    {
#if USE_THREADS
	startupUdp();
#endif
	out << listening_notice;
    }

    out.unlock();
}
