/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDebugger.h>
#include <RTMain.h>
#include <RTMemoryUtil.h>

void RTDebugger::debugIt( const char * cmdLine )
{
    if( phase() < RTJob::running )
    {
	enter( RTJob::running );

#if OBSERVABLE
	checkObservability();
#endif

	int                  argc = RTMain::argCount();
	const char * const * argv = RTMain::argStrings();
	const char         * init = nullptr;

	while( ++argv, --argc > 0 )
	    if( RTMemoryUtil::strcmp( *argv, "-URTS_DEBUG=", 12 ) == 0 )
		init = *argv + 12;

	if( init == nullptr )
	    init = RTMemoryUtil::_getenv( "URTS_DEBUG" );

	if( init != nullptr )
	{
	    input.pushInput( init );
	    doCmd();
	}
    }

    if( cmdLine != nullptr )
	input.pushInput( cmdLine );
#if OBSERVABLE
    else if( mainObserverLoop() > 0 )
	return;
#endif

    if( phase() != RTJob::finished )
	doCmd();
}
