/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDebugger.h>
#if USE_THREADS
#include <RTMutex.h>
#endif
#include <RTProbe.h>

int RTDebugger::deleteProbe( int probeId )
{
	int result = probeId;

#if USE_THREADS
	_mutex->enter();
#endif

	RTProbe * probe = getProbe( probeId );

	if( probe == nullptr )
		result = -1;
	else if( probe->adjustRefCount( -1 ) <= 0 ) // no one left
	{
		for( int i = 0; i < parkedProbes.size(); ++i )
		{
			if( parkedProbes[ i ] == probe )
			{
				parkedProbes[ i ] = nullptr;
				--nrParkedProbes;
				break;
			}
		}

		RTActor * actor = probe->actor();

		if( actor == nullptr )
		{
			// No actor connected to this probe; it is safe to
			// disconnect and delete the probe now.
			(void)probe->disconnectProbe( nullptr );
			delete probe;
		}
		else
		{
			// Probe is connected to an actor; mark probe for
			// destruction and ask the actor to take care of it.
			probe->_destroyMe = 1;
			sendProbeSignal( actor, System );
		}

		probes[ probeId ] = nullptr;
	}

#if USE_THREADS
	_mutex->leave();
#endif

	return result;
}
