/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTController.h>
#include <RTDebugger.h>
#include <RTDiag.h>
#if USE_THREADS
#include <RTMutex.h>
#endif

void RTDebugger::detach( RTController * task )
{
#if USE_THREADS
    _mutex->enter();
#endif

    switch( task->_state )
    {
      case RTController::execUser:
      case RTController::execSystem:
      case RTController::waitBegin:
      case RTController::waitContinue:
      case RTController::waitDone:
	task->_state = RTController::detached;
#if USE_THREADS
	wakeup( task );
#endif
	break;

      case RTController::detached:
	break;

      default:
	RTDiag::panic( "unexpected controller state" );
	//lint -unreachable
	break;
    }

#if USE_THREADS
    _mutex->leave();
#endif
}
