/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTController.h>
#include <RTDebugger.h>
#include <RTMessage.h>
#include <RTProtocol.h>

int RTDebugger::doInject( RTProtocol * port,
		int                    index,
		int                    signal,
		int                    prio,
		const void           * data,
		const RTObject_class * type )
{
	RTActor      * actor    = port->getOwner();
	RTController * receiver = actor->context();
	RTMessage    * destMsg  = newMsg();

	if( destMsg == nullptr )
	{
		receiver->setError( RTController::noMem,
				RTController::SystemService, nullptr, nullptr );
		return 0;
	}

	destMsg->setData( data, type );
	destMsg->setSignal( signal );
	destMsg->setPriority( prio );
	destMsg->setDestination( actor, port, index );
	destMsg->internData();

#if USE_THREADS
	receiver->peer_receive( destMsg );
#else
	receiver->receive( destMsg, nullptr );
#endif

	return 1;
}
