/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifdef PRAGMA
#pragma implementation "RTDebugger.h"
#pragma implementation "RTDebugger.inl"
#endif

#include <RTDebugger.h>
#include <RTDiag.h>
#include <RTEventInfo.h>
#include <RTIOMonitor.h>
#include <RTProbe.h>

RTDebugger::~RTDebugger( void )
{
	RTAssert( tasks == nullptr );

#if OBSERVABLE
	for( int i = probes.size(); i-- != 0; )
	{
		RTProbe * probe = static_cast<RTProbe *>(probes[ i ]);

		if( probe != nullptr )
		{
			delete probe;
			probes[ i ] = nullptr;
		}
	}
#endif

	RTController::resourceMgr.msgFree( freeList );

#if OBSERVABLE
	closeObservability();

	while( eventHead != nullptr )
	{
		RTEventInfo * temp = eventHead->next;

		delete eventHead;

		eventHead = temp;
	}
#endif

#ifdef _lint
	tasks     = nullptr;
	freeList  = nullptr;
#if OBSERVABLE
	ioMonitor = nullptr;
	eventHead = nullptr;
	eventTail = nullptr;
#endif // OBSERVABLE
#endif // _lint
}
