/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorRef.h>
#include <RTDebugger.h>
#include <RTMemoryUtil.h>

/*
 * Parse a actor id string of the form:
 *
 *  id          : repIndex
 *              | id / componentId
 *              | id / componentId . repIndex
 *
 *  repIndex    : INT
 *  componentId : INT
 *
 */

RTActor * RTDebugger::findActor( const char * id, RTActorRef * ref )
{
    int index = RTMemoryUtil::atoi( id );

    while( ref != nullptr && 0 < index && index <= ref->size() )
    {
	RTActor * actor = (*ref)[ index - 1 ];

	if( actor == nullptr )
	    break;

	while( '0' <= *id && *id <= '9' )
	    ++id;

	if( *id == '\0' )
	    return actor;

	if( *id++ != '/' )
	    break;

	if( ! ( '0' <= *id && *id <= '9' ) )
	    break;

	ref = actor->_component( RTMemoryUtil::atoi( id ) );

	while( '0' <= *id && *id <= '9' )
	    ++id;

	if( *id != '.' )
	    index = 1;
	else
	{
	    ++id;
	    if( ! ( '0' <= *id && *id <= '9' ) )
		break;

	    index = RTMemoryUtil::atoi( id );
	}
    }

    return nullptr;
}
