/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDebugger.h>
#include <RTIOMonitor.h>
#include <RTObserver.h>
#include <RTTcpSocket.h>
#include <RTTimespec.h>
#include <RTinet.h>
#include <RTtcp.h>

int RTDebugger::mainObserverLoop( void )
{
	if( ioMonitor == nullptr )
		return 0;

	static const RTTimespec   awhile( 10L, 0L );
	RTTcpSocket             * sock;
	RTObserver              * observer;
	int                       i;

	if( ioMonitor->wait( &awhile ) <= 0 )
		return 1;

#if USE_THREADS
	if( ( ioMonitor->status( _udp ) & RTIOMonitor::CanRead ) != 0 )
#endif
	{
#if USE_THREADS
		char           anything;
		RTinet_address source_address;
		RTinet_port    source_port;

		(void)RTinet_recvfrom( _udp,
				&anything, static_cast<int>(sizeof( anything )),
				&source_address, &source_port );
#endif

		while( useEvent() > 0 ) // > 0 as long as there are events waiting
		{
			for( i = observers.size(); --i >= 0; )
			{
				observer = static_cast<RTObserver *>(observers[ i ]);

				if( observer != nullptr )
				{
					if( observer->receiveData() < 0 )
					{
						delete observer;
						observers[ i ] = nullptr;
					}
				}
			}
			observers.resize( 0 );
		}
	}

	if( (sock = newConnection()) != nullptr )
		sockets[ sockets.unusedIndex() ] = sock;

	for( i = sockets.size(); --i >= 0; )
	{
		sock = static_cast<RTTcpSocket *>(sockets[ i ]);

		if( sock != nullptr )
		{
			int obsPlace = observers.unusedIndex();

			observer = findObserver( sock, obsPlace );

			if( observer != nullptr )
			{
				observers[ obsPlace ] = observer;
				sockets[ i ] = nullptr;
			}
		}
	}
	sockets.resize( 0 );

	for( i = observers.size(); --i >= 0; )
	{
		observer = static_cast<RTObserver *>(observers[ i ]);

		if( observer != nullptr )
		{
			if( observer->receiveData() < 0 )
			{
				delete observer;
				observers[ i ] = nullptr;
			}
			else
				(void)observer->transmitData();
		}
	}
	observers.resize( 0 );

	return 1;
}
