/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTBindingEnd.h>
#include <RTDebugger.h>
#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTMessage.h>
#include <RTProtocol.h>

void RTDebugger::printMessage( const RTMessage * message )
{
    if( message != nullptr )
    {
	RTDiagStream & out   = RTDiag::out();
	RTProtocol   * port  = message->sap();
	int            index = message->sapIndex0();

	out << "  to   ";

	if( port == nullptr )
	    printActor( message->receiver() );
	else
	    printPort( port, index );

	out << ":";

	const char * signalName = message->getSignalName();

	if( signalName == nullptr )
	    out << "?(" << message->getSignal() << ")";
	else
	    out << signalName;

	if( port != nullptr )
	{
	    RTBindingEnd from;

	    if( port->peerAt( index, from ) != 0 )
	    {
		out << "\n  from ";
		printPort( from.port, from.index );
	    }
	}

	out << "\n  data ";

#if OBJECT_ENCODE
	const RTObject_class * type = message->getType();

	if( type != nullptr )
	    (void)out.write( message->data, type );
	else
#endif
	    out << "(void *)" << message->data;

	out << "\n";
    }
}
