/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTActorRef.h>
#include <RTActorRefProbe.h>
#include <RTActor_class.h>
#include <RTComponentDescriptor.h>
#include <RTDebugger.h>
#if USE_THREADS
#include <RTMutex.h>
#endif

void RTDebugger::probeAttention( const RTActor * target )
{
#if USE_THREADS
	_mutex->enter();
#endif

	RTActorProbe * probe = target->probe;

	if( probe != nullptr )
	{
		probe->accessFieldValue();

		if( probe->_destroyMe != 0 )
		{
			(void)probe->disconnectProbe( nullptr );
			delete probe;
		}
	}

	const RTActor_class         * info = target->getActorData();
	const RTComponentDescriptor * desc = info->components;

	for( int i = info->numComponents; --i >= 0; ++desc )
	{
		const RTActorRef * actorRef = RTConstObjectOf( RTActorRef, desc->offset, target );
		RTActorRefProbe  * refProbe = actorRef->probe;

		if( refProbe != nullptr && refProbe->_destroyMe != 0 )
		{
			(void)refProbe->disconnectProbe( nullptr );
			delete refProbe;
		}
	}

#if USE_THREADS
	_mutex->leave();
#endif
}
