/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTController.h>
#include <RTDebugger.h>
#if USE_THREADS
#include <RTMutex.h>
#endif

void RTDebugger::proceed( void )
{
#if USE_THREADS
    _mutex->enter();
#endif

    mode = Running;

#if OBSERVABLE
    evtRtsStatus();
#endif

    RTController * task = tasks;

    if( task != nullptr )
    {
	do
	{
	    switch( task->_state )
	    {
	    case RTController::waitBegin:
	    case RTController::waitContinue:
		task->_state = RTController::waitDone;
#if USE_THREADS
		wakeup( task );
#endif
		break;

#ifdef _lint
	    case RTController::detached:
	    case RTController::execSystem:
	    case RTController::waitDone:
	    case RTController::execUser:
#endif
	    default:
		break;
	    }
	}
	while( (task = task->_next) != tasks );
    }

#if USE_THREADS
    _mutex->leave();
#endif
}
