/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTDebugger.h>
#include <RTMessage.h>
#if USE_THREADS
#include <RTMutex.h>
#endif
#include <RTProtocol.h>

void RTDebugger::setTimestamp( RTMessage        * msg,
			       const RTProtocol * source,
			       int                sourceIndex )
{
    if( msg == nullptr )
	return; // no message

    if( source != nullptr )
    {
	RTActor * sender = source->getOwner();

	if( sender != nullptr && sender->probe != nullptr )
	{
	    msg->_sendProbeId   = sender->probe->getProbeId();
	    msg->_sendPortId    = source->getId();
	    msg->_sendPortIndex = sourceIndex;

#if USE_THREADS
	    _mutex->enter();
#endif

	    getTimestamp( msg->_sendTime );

#if USE_THREADS
	    _mutex->leave();
#endif

	    return;
	}
    }

    RTActor * receiver = msg->receiver();

    if( receiver != nullptr && receiver->probe != nullptr )
    {

#if USE_THREADS
	_mutex->enter();
#endif

	getTimestamp( msg->_sendTime );

#if USE_THREADS
	_mutex->leave();
#endif
    }
}
