/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTController.h>
#include <RTDebugger.h>
#include <RTDiag.h>
#if USE_THREADS
#include <RTMutex.h>
#endif

void RTDebugger::step_end( RTController * task )
{
#if USE_THREADS
    _mutex->enter();
#endif

    switch( task->_state )
    {
    case RTController::execUser:
	task->_state = RTController::execSystem;
	// fall through

    case RTController::execSystem:
	if( mode == Stepping && step_count == 0U )
	{
	    mode = Halted;
#if OBSERVABLE
	    evtRtsStatus();
#endif
	}
	// fall through

    case RTController::detached:
#if USE_THREADS
	_mutex->leave();
#endif
	break; // nothing to do here

#ifdef _lint
    case RTController::waitBegin:
    case RTController::waitDone:
    case RTController::waitContinue:
#endif
    default:
#if USE_THREADS
	_mutex->leave();
#endif
	RTDiag::panic( "step_end: unexpected state" );
	//lint -unreachable
	break;
    }
}
