/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTExternal.h>
#include <RTMutex.h>
#include <list>

static RTMutex externalDataMutex;

struct RTExternalData
{
	std::list<void*> list;
};

void External::Base::dataPushBack(void* d)
{
	externalDataMutex.enter();

	if (!externalData)
		externalData = new RTExternalData();

	externalData->list.push_back(d);
	externalDataMutex.leave();
}

unsigned int External::Base::dataPopFront(void** d)
{
	externalDataMutex.enter();

	if (!externalData || externalData->list.empty())
	{
		*d = nullptr;
		externalDataMutex.leave();
		return 0;
	}

	*d = externalData->list.front();
	externalData->list.pop_front();
	unsigned int remainingItems = static_cast<unsigned int>(externalData->list.size());
	externalDataMutex.leave();

	return remainingItems;
}

unsigned int External::Base::dataPopBack(void** d)
{
	externalDataMutex.enter();

	if (!externalData || externalData->list.empty())
	{
		*d = nullptr;
		externalDataMutex.leave();
		return 0;
	}

	*d = externalData->list.back();
	externalData->list.pop_back();
	unsigned int remainingItems = static_cast<unsigned int>(externalData->list.size());
	externalDataMutex.leave();

	return remainingItems;
}

void External::Base::dataDelete(void* d) {
	externalDataMutex.enter();
	if (!externalData || externalData->list.empty())
		{
			externalDataMutex.leave();
			return;
		}
	externalData->list.remove(d);
	externalDataMutex.leave();
}
