/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTController.h>
#include <RTExternal.h>
#include <RTMessage.h>
#include <RTObject_class.h>

void External::Base::enable( void )
{
    if( msg == nullptr )
    {
	RTController * controller = context();
	RTMessage    * message    = controller->newMsg();

	if( message == nullptr )
	{
	    controller->setError( RTController::noMem,
				  RTController::SystemService, nullptr, nullptr );
	}
	else
	{
	    message->setSignal( External::Base::rti_event );
	    message->setPriority( System );
	    message->setData( nullptr, &RTType_void );
	    message->setDestination( getOwner(), this, 0 );

	    // This function may only be used by the thread on which the owner
	    // capsule executes so there is no danger of leaking messages here.
	    // However we assume that this store is atomic with respect to
	    // unload. It either fetches this message or the nil pointer that
	    // was previously stored in msg.
	    msg = message;
	}
    }
}
