/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTArray_class.h>
#include <RTFieldDescriptor.h>
#include <RTObject_class.h>
#include <RTTypeModifier.h>

static void array_destroy( const RTObject_class * element,
		void                 * target,
		int                    size,
		int                    indirect )
{
	RTArray_class   array( size, element );
	void          * target_array;

	if( indirect == 0 )
		array.destroy( target, RTObject_class::DestroyOnly );
	else if( (target_array = * static_cast<void **>(target)) != nullptr )
	{
		* static_cast<void **>(target) = nullptr;

		array.destroy( target_array, RTObject_class::DestroyAndDeallocate );
	}
}

void RTFieldDescriptor::destroy( void * target ) const
{
	void * field = RTObjectOf( void, offset, target );

	if( modifier == nullptr )
		(*type->_destroy_func)( type, field );
	else
		array_destroy( type, field,
				(*modifier->_num_elements_func)( modifier, target ),
				modifier->_indirect );
}
