/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTFloat.h>

//lint -save -e922

double RTFloat::value( void )
{
	const double scale  = word_scale;
	double       result = mantissa[ mantissa_words - 1 ];

	for( int i = mantissa_words - 1; --i >= 0; )
	{
		result /= scale;
		result += mantissa[ i ];
	}

	int exponent = exp_2 + mantissa_bits - bits_per_word;

	if( exponent < 0 )
	{
		// The magnitude of the exponent can create unwanted
		// overflow problems with very small values.
		// To avoid this situation we divide in two steps.

		unsigned magnitude = static_cast<unsigned>(- exponent);

		result /= pow2( ( magnitude + 0U ) >> 1 );
		result /= pow2( ( magnitude + 1U ) >> 1 );
	}
	else if( exponent > 0 )
		result *= pow2( static_cast<unsigned>(exponent) );

	return result;
}

//lint -restore
