/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorId.h>
#include <RTActorRef.h>
#include <RTController.h>
#include <RTFrame.h>

int Frame::Base::destroy( RTActorId & actorid )
{
    RTController * controller = context();
    RTActor      * actor      = actorid.getRTActor();

    if( actor == nullptr )
    {
	controller->trace( RTController::FrameDestroy, nullptr, nullptr );

	controller->setError( RTController::badActor,
			      RTController::FrameDestroy, nullptr, nullptr );
    }
    else
    {
	RTActorRef * reference = actor->getReference();

	controller->trace( RTController::FrameDestroy, reference, nullptr );

	if( reference->isOptional() == 0 )
	{
	    controller->setError( RTController::notOptional,
				  RTController::FrameDestroy, reference, nullptr );
	}
#if RTFRAME_CHECKING != RTFRAME_CHECK_NONE
	else if( permissionCheck( RTController::FrameDestroy,
				  reference, 0 ) == 0 )
	{
	    // not allowed
	}
#endif
	else
	{
	    actorid.resetActor();
	    return reference->destroy( actor );
	}
    }

    return 0;
}
