/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTIOMonitor.h>
#include <RTMemoryUtil.h>
#include <RTTimespec.h>
#include <RTtcp.h>

int RTIOMonitor::wait( const RTTimespec * timeout )
{
	fd_set rSet;
	fd_set wSet;
	fd_set eSet;

	RTMemoryUtil::memset( &rSet, 0, sizeof( rSet ) );
	RTMemoryUtil::memset( &wSet, 0, sizeof( wSet ) );
	RTMemoryUtil::memset( &eSet, 0, sizeof( eSet ) );

	int        max_fd = -1;
	Entry    * entry;
	unsigned   index;

	for( entry = _entries, index = 0U; index < _used; ++entry, ++index )
	{
		unsigned fd   = entry->fd;
		int      bits = entry->bits &= RequestMask;

		if( max_fd < static_cast<int>(fd) )
			max_fd = static_cast<int>(fd);

		//lint -save -e647 -e661 -e662 -e909 -e912
		if( ( bits & CanRead   ) != 0 ) FD_SET( fd, &rSet );
		if( ( bits & CanWrite  ) != 0 ) FD_SET( fd, &wSet );
		if( ( bits & Exception ) != 0 ) FD_SET( fd, &eSet );
		//lint -restore
	}

	struct timeval * ptv;
	struct timeval   tv;

	if( timeout == nullptr )
		ptv = nullptr;
	else
	{
		tv.tv_sec  = timeout->tv_sec;
		tv.tv_usec = timeout->tv_nsec / 1000L;
		ptv = &tv;
	}

	if( select( max_fd + 1, &rSet, &wSet, &eSet, ptv ) <= 0L )
		return 0;

	for( entry = _entries, index = 0U; index < _used; ++entry, ++index )
	{
		unsigned fd   = entry->fd;
		int      bits = 0;

		//lint -save -e647 -e909 -e912
		if( FD_ISSET( fd, &rSet ) ) bits |= ResultCanRead;
		if( FD_ISSET( fd, &wSet ) ) bits |= ResultCanWrite;
		if( FD_ISSET( fd, &eSet ) ) bits |= ResultException;
		//lint -restore

		entry->bits |= bits;
	}

	return 1;
}
