/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTInjector.h>
#include <RTActor.h>
#include <RTActorRef.h>
#include <RTMain.h>

RTActor* RTInjector::create(RTController *rts, RTActorRef *ref, int index) {
	std::string id;
	RTActorRef* cp = ref;
	int i = -1;
	while (cp != nullptr) {
		if (i != -1) {
			id.insert(0, std::to_string(i));
			id.insert(0, ":");
		}
		id.insert(0, cp->getName());
		id.insert(0, "/");
		RTActor* c = cp->getOwner();
		if (c) {
			cp = c->getReference();
			i = c->getIndex();
			if (cp == RTMain::topReference())
				cp = nullptr; // Reached the top of the composite structure
		}
		else
			cp = nullptr;
	}
	std::function<RTActor* (RTController *, RTActorRef *, int)> createFunc = getCreateFunction(id);
	if (createFunc)
		return createFunc(rts, ref, index);

	RTActor* owner = ref->getOwner();
	if (!owner)
		return nullptr;

	const RTActorClass* actorClass = owner->defaultClass(*ref, index);
	if (!actorClass->create)
		return nullptr; // For example an abstract capsule

	return (*actorClass->create)(rts, ref);
}
