/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTFieldDescriptor.h>
#include <RTInteger.h>
#include <RTObject_class.h>
#include <RTnew.h>

static RTDataObject * _access( RTInteger * target )
{
	return target;
}

static void _init( const RTObject_class *, void * target )
{
	(void)new( target ) RTInteger;
}

static void _copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	(void)new( target ) RTInteger( * static_cast<const RTInteger *>(source) );
}

#if OBJECT_DECODE
static int _decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	RTInteger * data = static_cast<RTInteger *>(target);

	return coding->get_int( data->value );
}
#endif

#if OBJECT_ENCODE
static int _encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	const RTInteger * data = static_cast<const RTInteger *>(source);

	return coding->put_int( data->value );
}
#endif

static const RTFieldDescriptor _fields[] =
{
		{
				"RTInteger",
				0,
				&RTInteger::classData, nullptr
		}
};

const RTObject_class RTInteger::classData =
{
		&RTType_RTDataObject,
		reinterpret_cast<RTSuperAccessFunction>(_access),
		"RTInteger",
		0 /* RTVersionId */,
		sizeof( RTInteger ),
		_init,
		_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		_decode,
#endif
#if OBJECT_ENCODE
		_encode,
#endif
		RTDataObject_destroy,
		1, _fields
};

#if OBJECT_DECODE
RTTypeInstaller RTInteger_installer( RTType_RTInteger );
#endif

const RTObject_class * RTInteger::getClassData( void ) const
{
	return &RTInteger::classData;
}
