/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTLayerConnector.h>
#include <RTProtocol.h>
#include <RTQueue.h>

int RTLayerConnector::install( RTController * context, const char * name )
{
	lock();

	int status = connectors.atPut( name, this );

	if( status != 0 )
	{
		RTQueue    * queue;
		void       * entry;
		RTProtocol * port;
		Pending    * pending;

		queue = static_cast<RTQueue *>(pendingSPPs[ name ]);

		if( queue != nullptr )
		{
			// register pending SPPs
			(void)pendingSPPs.remove( name );

			RTQueue::Iterator iterator( *queue );

			while( (entry = iterator.next()) != nullptr )
			{
				pending      = static_cast<Pending *>(entry);
				port         = pending->port;
				port->_layer = nullptr;
				port->_layer = _registerSPP( context, port, pending->name );

				delete pending;
			}

			delete queue;
		}

		queue = static_cast<RTQueue *>(pendingSAPs[ name ]);

		if( queue != nullptr )
		{
			// register pending SAPs
			(void)pendingSAPs.remove( name );

			RTQueue::Iterator iterator( *queue );

			while( (entry = iterator.next()) != nullptr )
			{
				pending      = static_cast<Pending *>(entry);
				port         = pending->port;
				port->_layer = nullptr;
				port->_layer = _registerSAP( context, port, pending->name );

				delete pending;
			}

			delete queue;
		}
	}

	unlock();

	return status;
}
