/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTLayerConnector.h>

void RTLayerConnector::showSAPs( void )
{
	RTDiagStream           & out  = RTDiag::out();
	int                      regd = 0;
	int                      pend = 0;
	RTDictionary::Iterator   i;
	const RTAssociation    * assoc;
	const RTLayerConnector * connector;
	const RTQueue          * sapQueue;
	const RTQueue          * sppQueue;

	lock();

	while( (assoc = connectors.iterate( i )) != nullptr )
	{
		connector = static_cast<const RTLayerConnector *>(assoc->value);

		if( regd != 0 )
			out << "\n";

		out << "Service: '" << assoc->key << ":'\n";

		connector->_enumerate();

		regd = 1;
	}

	i.reset();
	while( (assoc = pendingSPPs.iterate( i )) != nullptr )
	{
		sapQueue = static_cast<const RTQueue *>(pendingSAPs[ assoc->key ]);
		sppQueue = static_cast<const RTQueue *>(assoc->value);

		if( pend == 0 )
			out << "\nPending (SAPs,SPPs):\n";

		out << "  '" << assoc->key
				<< ":' ( "
				<< total_replication( sapQueue )
				<< ", "
				<< total_replication( sppQueue )
				<< " )\n";

		pend = 1;
	}

	i.reset();
	while( (assoc = pendingSAPs.iterate( i )) != nullptr )
	{
		if( pendingSPPs[ assoc->key ] != nullptr )
			continue; // we already showed these SAPs

		sapQueue = static_cast<const RTQueue *>(assoc->value);

		if( pend == 0 )
			out << "\nPending (SAPs,SPPs):\n";

		out << "  '" << assoc->key
				<< ":' ( "
				<< total_replication( sapQueue )
				<< ", 0 )\n";

		pend = 1;
	}

	if( regd == 0 && pend == 0 )
		out << "  Nothing registered.\n";

	unlock();
}
