/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTLocalConnector.h>
#include <RTProtocol.h>

void RTLocalConnector::SapNode::resize( RTController * context,
					int            old_size )
{
    int new_size = initial.sap->size();

    if( new_size > old_size )
    {
	// create a new request for the extension
	node->append( context, new Request( initial.sap, old_size ) );
    }
    else // if( new_size < old_size )
    {
	// update/remove my requests

	Request * request = &initial;

	// scan past the ones we want to keep in their entirety
	while( request->limit <= new_size )
	{
	    if( (request = request->chain) == nullptr )
	    {
		// something strange happened: the size was reduced but
		// we can't find a request which includes the removed part
		return;
	    }
	}

	// adjust the limit of the last request
	request->limit = new_size;

	Request * junk;

	for( junk = request->chain; junk != nullptr; junk = junk->chain )
	{
	    junk->start = 0;
	    junk->limit = 0;
	}

	// remove any following requests
	for( request = request->chain; request != nullptr; )
	{
	    junk = request;

	    request = request->chain;

	    node->remove( context, junk );

	    delete junk;
	}
    }
}
