/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTMain.h>
#if ! USE_THREADS
#include <RTSoleController.h>
#endif

#include <signal.h>

#if ! USE_THREADS

#if defined( SIGALRM )
static void handle_sigalrm( void )
{
    (static_cast<RTSoleController *>(RTMain::controller()))->timer_interrupt();
}
#endif

#if defined( SIGIO ) || defined( SIGPOLL )
static void handle_sigio( void )
{
}
#endif

#endif // ! USE_THREADS

#if defined( SIGPIPE )
static void handle_sigpipe( void )
{
    // do nothing
}
#endif

void RTMain::installHandlers( void )
{
#if ! USE_THREADS
#if defined( SIGALRM )
    installOneHandler( SIGALRM, &handle_sigalrm );
#endif
#if defined( SIGIO )
    installOneHandler( SIGIO,   &handle_sigio   );
#endif
#if defined( SIGPOLL )
//lint -save -e778
#if defined( SIGIO ) && 0 == SIGIO - SIGPOLL
//lint -restore
    // SIGIO and SIGPOLL are both defined as the same signal;
    // we already installed the handler above.
#else
    installOneHandler( SIGPOLL, &handle_sigio   );
#endif
#endif // ! USE_THREADS
#endif
#if defined( SIGPIPE )
    installOneHandler( SIGPIPE, &handle_sigpipe );
#endif
}
