/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void bool_init( const RTObject_class *, void * target )
{
    * static_cast<bool *>(target) = false;
}

static void bool_copy( const RTObject_class *,
		       void                 * target,
		       const void           * source )
{
    * static_cast<bool *>(target) = * static_cast<const bool *>(source);
}

#if OBJECT_DECODE
static int bool_decode( const RTObject_class *,
			void                 * target,
			RTDecoding           * coding )
{
    return coding->get_bool( * static_cast<bool *>(target) );
}
#endif

#if OBJECT_ENCODE
static int bool_encode( const RTObject_class *,
			const void           * source,
			RTEncoding           * coding )
{
    return coding->put_bool( * static_cast<const bool *>(source) );
}
#endif

const RTObject_class RTType_bool =
{
     nullptr,
    nullptr,
    "bool",
    0 /* RTVersionId */,
    sizeof( bool ),
    bool_init,
    bool_copy,
	nullptr, // Not movable
#if OBJECT_DECODE
    bool_decode,
#endif
#if OBJECT_ENCODE
    bool_encode,
#endif
    RTnop_destroy,
    0, nullptr
};

#if OBJECT_DECODE
RTTypeInstaller bool_installer( RTType_bool );
#endif
