/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void char_init( const RTObject_class *, void * target )
{
    * static_cast<char *>(target) = '\0';
}

static void char_copy( const RTObject_class *,
		       void                 * target,
		       const void           * source )
{
    * static_cast<char *>(target) = * static_cast<const char *>(source);
}

#if OBJECT_DECODE
static int char_decode( const RTObject_class *,
			void                 * target,
			RTDecoding           * coding )
{
    return coding->get_char( * static_cast<char *>(target) );
}
#endif

#if OBJECT_ENCODE
static int char_encode( const RTObject_class *,
			const void           * source,
			RTEncoding           * coding )
{
    return coding->put_char( * static_cast<const char *>(source) );
}
#endif

const RTObject_class RTType_char =
{
    nullptr,
    nullptr,
    "char",
    0 /* RTVersionId */,
    sizeof( char ),
    char_init,
    char_copy,
	nullptr, // Not movable
#if OBJECT_DECODE
    char_decode,
#endif
#if OBJECT_ENCODE
    char_encode,
#endif
    RTnop_destroy,
    0,nullptr
};

#if OBJECT_DECODE
RTTypeInstaller char_installer( RTType_char );
#endif
