/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void double_init( const RTObject_class *, void * target )
{
	* static_cast<double *>(target) = 0.0;
}

static void double_copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	* static_cast<double *>(target) = * static_cast<const double *>(source);
}

#if OBJECT_DECODE
static int double_decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	return coding->get_double( * static_cast<double *>(target) );
}
#endif

#if OBJECT_ENCODE
static int double_encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	return coding->put_double( * static_cast<const double *>(source) );
}
#endif

const RTObject_class RTType_double =
{
		nullptr,
		nullptr,
		"double",
		0 /* RTVersionId */,
		sizeof( double ),
		double_init,
		double_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		double_decode,
#endif
#if OBJECT_ENCODE
		double_encode,
#endif
		RTnop_destroy,
		0, nullptr
};

#if OBJECT_DECODE
RTTypeInstaller double_installer( RTType_double );
#endif
