/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void float_init( const RTObject_class *, void * target )
{
    * static_cast<float *>(target) = 0.0f;
}

static void float_copy( const RTObject_class *,
			void                 * target,
			const void           * source )
{
    * static_cast<float *>(target) = * static_cast<const float *>(source);
}

#if OBJECT_DECODE
static int float_decode( const RTObject_class *,
			 void                 * target,
			 RTDecoding           * coding )
{
    return coding->get_float( * static_cast<float *>(target) );
}
#endif

#if OBJECT_ENCODE
static int float_encode( const RTObject_class *,
			 const void           * source,
			 RTEncoding           * coding )
{
    return coding->put_float( * static_cast<const float *>(source) );
}
#endif

const RTObject_class RTType_float =
{
    nullptr,
    nullptr,
    "float",
    0 /* RTVersionId */,
    sizeof( float ),
    float_init,
    float_copy,
	nullptr, // Not movable
#if OBJECT_DECODE
    float_decode,
#endif
#if OBJECT_ENCODE
    float_encode,
#endif
    RTnop_destroy,
    0, nullptr
};

#if OBJECT_DECODE
RTTypeInstaller float_installer( RTType_float );
#endif
