/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void int_init( const RTObject_class *, void * target )
{
	* static_cast<int *>(target) = 0;
}

static void int_copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	* static_cast<int *>(target) = * static_cast<const int *>(source);
}

#if OBJECT_DECODE
static int int_decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	return coding->get_int( * static_cast<int *>(target) );
}
#endif

#if OBJECT_ENCODE
static int int_encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	return coding->put_int( * static_cast<const int *>(source) );
}
#endif

const RTObject_class RTType_int =
{
		nullptr,
		nullptr,
		"int",
		0 /* RTVersionId */,
		sizeof( int ),
		int_init,
		int_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		int_decode,
#endif
#if OBJECT_ENCODE
		int_encode,
#endif
		RTnop_destroy,
		0, nullptr
};

#if OBJECT_DECODE
RTTypeInstaller int_installer( RTType_int );
#endif
