/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void long_init( const RTObject_class *, void * target )
{
	* static_cast<long *>(target) = 0L;
}

static void long_copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	* static_cast<long *>(target) = * static_cast<const long *>(source);
}

#if OBJECT_DECODE
static int long_decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	return coding->get_long( * static_cast<long *>(target) );
}
#endif

#if OBJECT_ENCODE
static int long_encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	return coding->put_long( * static_cast<const long *>(source) );
}
#endif

const RTObject_class RTType_long =
{
		nullptr,
		nullptr,
		"long",
		0 /* RTVersionId */,
		sizeof( long ),
		long_init,
		long_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		long_decode,
#endif
#if OBJECT_ENCODE
		long_encode,
#endif
		RTnop_destroy,
		0, nullptr
};

#if OBJECT_DECODE
RTTypeInstaller long_installer( RTType_long );
#endif
