/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void pvoid_init( const RTObject_class *, void * target )
{
	* static_cast<void **>(target) = nullptr;
}

static void pvoid_copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	* static_cast<void **>(target) = * static_cast<void * const *>(source);
}

#if OBJECT_DECODE
static int pvoid_decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	return coding->get_address( * static_cast<void **>(target) );
}
#endif

#if OBJECT_ENCODE
static int pvoid_encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	return coding->put_address( * static_cast<void * const *>(source) );
}
#endif

const RTObject_class RTType_RTpvoid =
{
		nullptr,
		nullptr,
		"RTpvoid",
		0 /* RTVersionId */,
		sizeof( void * ),
		pvoid_init,
		pvoid_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		pvoid_decode,
#endif
#if OBJECT_ENCODE
		pvoid_encode,
#endif
		RTnop_destroy,
		0, nullptr
};

#if OBJECT_DECODE
RTTypeInstaller RTpvoid_installer( RTType_RTpvoid );
#endif
