/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void short_init( const RTObject_class *, void * target )
{
	* static_cast<short *>(target) = 0;
}

static void short_copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	* static_cast<short *>(target) = * static_cast<const short *>(source);
}

#if OBJECT_DECODE
static int short_decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	return coding->get_short( * static_cast<short *>(target) );
}
#endif

#if OBJECT_ENCODE
static int short_encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	return coding->put_short( * static_cast<const short *>(source) );
}
#endif

const RTObject_class RTType_short =
{
		nullptr,
		nullptr,
		"short",
		0 /* RTVersionId */,
		sizeof( short ),
		short_init,
		short_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		short_decode,
#endif
#if OBJECT_ENCODE
		short_encode,
#endif
		RTnop_destroy,
		0, nullptr
};

#if OBJECT_DECODE
RTTypeInstaller short_installer( RTType_short );
#endif
