/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTMemoryUtil.h>
#include <RTObject_class.h>

static void string_init( const RTObject_class *, void * target )
{
	* static_cast<char **>(target) = nullptr;
}

static void string_copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	* static_cast<char **>(target) = RTMemoryUtil::strdup( * static_cast<const char * const *>(source) );
}

#if OBJECT_DECODE
static int string_decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	char * string;

	if( coding->get_string( string ) == 0 )
		return 0;

	delete [] * static_cast<char **>(target);

	* static_cast<char **>(target) = string;

	return 1;
}
#endif

#if OBJECT_ENCODE
static int string_encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	return coding->put_string( * static_cast<char * const *>(source) );
}
#endif

static void string_destroy( const RTObject_class *, void * target )
{
	char ** value = static_cast<char **>(target);

	delete [] *value;

	*value = nullptr;
}

const RTObject_class RTType_RTpchar =
{
		nullptr,
		nullptr,
		"RTpchar",
		0 /* RTVersionId */,
		sizeof( char * ),
		string_init,
		string_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		string_decode,
#endif
#if OBJECT_ENCODE
		string_encode,
#endif
		string_destroy,
		0, nullptr
};

#if OBJECT_DECODE
RTTypeInstaller RTpchar_installer( RTType_RTpchar );
#endif
