/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void uchar_init( const RTObject_class *, void * target )
{
    * static_cast<unsigned char *>(target) = '\0';
}

static void uchar_copy( const RTObject_class *,
			void                 * target,
			const void           * source )
{
    * static_cast<unsigned char *>(target) = * static_cast<const unsigned char *>(source);
}

#if OBJECT_DECODE
static int uchar_decode( const RTObject_class *,
			 void                 * target,
			 RTDecoding           * coding )
{
    return coding->get_uchar( * static_cast<unsigned char *>(target) );
}
#endif

#if OBJECT_ENCODE
static int uchar_encode( const RTObject_class *,
			 const void           * source,
			 RTEncoding           * coding )
{
    return coding->put_uchar( * static_cast<const unsigned char *>(source) );
}
#endif

const RTObject_class RTType_RTuchar =
{
    nullptr,
    nullptr,
    "RTuchar",
    0 /* RTVersionId */,
    sizeof( unsigned char ),
    uchar_init,
    uchar_copy,
	nullptr, // Not movable
#if OBJECT_DECODE
    uchar_decode,
#endif
#if OBJECT_ENCODE
    uchar_encode,
#endif
    RTnop_destroy,
    0,
	nullptr
};

#if OBJECT_DECODE
RTTypeInstaller RTuchar_installer( RTType_RTuchar );
#endif
