/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void ulong_init( const RTObject_class *, void * target )
{
    * static_cast<unsigned long *>(target) = 0UL;
}

static void ulong_copy( const RTObject_class *,
			void                 * target,
			const void           * source )
{
    * static_cast<unsigned long *>(target) = * static_cast<const unsigned long *>(source);
}

#if OBJECT_DECODE
static int ulong_decode( const RTObject_class *,
			 void                 * target,
			 RTDecoding           * coding )
{
    return coding->get_ulong( * static_cast<unsigned long *>(target) );
}
#endif

#if OBJECT_ENCODE
static int ulong_encode( const RTObject_class *,
			 const void           * source,
			 RTEncoding           * coding )
{
    return coding->put_ulong( * static_cast<const unsigned long *>(source) );
}
#endif

const RTObject_class RTType_RTulong =
{
    nullptr,
    nullptr,
    "RTulong",
    0 /* RTVersionId */,
    sizeof( unsigned long ),
    ulong_init,
    ulong_copy,
	nullptr, // Not movable
#if OBJECT_DECODE
    ulong_decode,
#endif
#if OBJECT_ENCODE
    ulong_encode,
#endif
    RTnop_destroy,
    0,
	nullptr
};

#if OBJECT_DECODE
RTTypeInstaller RTulong_installer( RTType_RTulong );
#endif
