/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void uint_init( const RTObject_class *, void * target )
{
    * static_cast<unsigned *>(target) = 0U;
}

static void uint_copy( const RTObject_class *,
		       void                 * target,
		       const void           * source )
{
    * static_cast<unsigned *>(target) = * static_cast<const unsigned *>(source);

}
#if OBJECT_DECODE
static int uint_decode( const RTObject_class *,
			void                 * target,
			RTDecoding           * coding )
{
    return coding->get_unsigned( * static_cast<unsigned *>(target) );
}
#endif

#if OBJECT_ENCODE
static int uint_encode( const RTObject_class *,
			const void           * source,
			RTEncoding           * coding )
{
    return coding->put_unsigned( * static_cast<const unsigned *>(source) );
}
#endif

const RTObject_class RTType_unsigned =
{
    nullptr,
    nullptr,
    "unsigned",
    0 /* RTVersionId */,
    sizeof( unsigned ),
    uint_init,
    uint_copy,
	nullptr, // Not movable
#if OBJECT_DECODE
    uint_decode,
#endif
#if OBJECT_ENCODE
    uint_encode,
#endif
    RTnop_destroy,
    0,
	nullptr
};

#if OBJECT_DECODE
RTTypeInstaller unsigned_installer( RTType_unsigned );
#endif
