/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void unsignedchar_init( const RTObject_class *, void * target )
{
	* static_cast<unsigned char *>(target) = '\0';
}

static void unsignedchar_copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	* static_cast<unsigned char *>(target) = * static_cast<const unsigned char *>(source);
}

#if OBJECT_DECODE
static int unsignedchar_decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	return coding->get_unsignedchar( * static_cast<unsigned char *>(target) );
}
#endif

#if OBJECT_ENCODE
static int unsignedchar_encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	return coding->put_unsignedchar( * static_cast<const unsigned char *>(source) );
}
#endif

const RTObject_class RTType_unsignedchar =
{
		nullptr,
		nullptr,
		"unsigned char",
		0 /* RTVersionId */,
		sizeof( unsigned char ),
		unsignedchar_init,
		unsignedchar_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		unsignedchar_decode,
#endif
#if OBJECT_ENCODE
		unsignedchar_encode,
#endif
		RTnop_destroy,
		0,
		nullptr
};

#if OBJECT_DECODE
RTTypeInstaller unsignedchar_installer( RTType_unsignedchar );
#endif
