/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void unsignedint_init( const RTObject_class *, void * target )
{
	* static_cast<unsigned int *>(target) = 0;
}

static void unsignedint_copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	* static_cast<unsigned int *>(target) = * static_cast<const unsigned int *>(source);
}

#if OBJECT_DECODE
static int unsignedint_decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	return coding->get_unsignedint( * static_cast<unsigned int *>(target) );
}
#endif

#if OBJECT_ENCODE
static int unsignedint_encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	return coding->put_unsignedint( * static_cast<const unsigned int *>(source) );
}
#endif

const RTObject_class RTType_unsignedint =
{
		nullptr,
		nullptr,
		"unsigned int",
		0 /* RTVersionId */,
		sizeof( unsigned int ),
		unsignedint_init,
		unsignedint_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		unsignedint_decode,
#endif
#if OBJECT_ENCODE
		unsignedint_encode,
#endif
		RTnop_destroy,
		0,
		nullptr
};

#if OBJECT_DECODE
RTTypeInstaller unsignedint_installer( RTType_unsignedint );
#endif
