/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void unsignedlong_init( const RTObject_class *, void * target )
{
	* static_cast<unsigned long *>(target) = 0UL;
}

static void unsignedlong_copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	* static_cast<unsigned long *>(target) = * static_cast<const unsigned long *>(source);
}

#if OBJECT_DECODE
static int unsignedlong_decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	return coding->get_unsignedlong( * static_cast<unsigned long *>(target) );
}
#endif

#if OBJECT_ENCODE
static int unsignedlong_encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	return coding->put_unsignedlong( * static_cast<const unsigned long *>(source) );
}
#endif

const RTObject_class RTType_unsignedlong =
{
		nullptr,
		nullptr,
		"unsigned long",
		0 /* RTVersionId */,
		sizeof( unsigned long ),
		unsignedlong_init,
		unsignedlong_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		unsignedlong_decode,
#endif
#if OBJECT_ENCODE
		unsignedlong_encode,
#endif
		RTnop_destroy,
		0,
		nullptr
};

#if OBJECT_DECODE
RTTypeInstaller unsignedlong_installer( RTType_unsignedlong );
#endif
