/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void unsignedshort_init( const RTObject_class *, void * target )
{
	* static_cast<unsigned short *>(target) = 0U;
}

static void unsignedshort_copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	* static_cast<unsigned short *>(target) = * static_cast<const unsigned short *>(source);
}

#if OBJECT_DECODE
static int unsignedshort_decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	return coding->get_unsignedshort( * static_cast<unsigned short *>(target) );
}
#endif

#if OBJECT_ENCODE
static int unsignedshort_encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	return coding->put_unsignedshort( * static_cast<const unsigned short *>(source) );
}
#endif

const RTObject_class RTType_unsignedshort =
{
		nullptr,
		nullptr,
		"unsigned short",
		0 /* RTVersionId */,
		sizeof( unsigned short ),
		unsignedshort_init,
		unsignedshort_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		unsignedshort_decode,
#endif
#if OBJECT_ENCODE
		unsignedshort_encode,
#endif
		RTnop_destroy,
		0, nullptr
};

#if OBJECT_DECODE
RTTypeInstaller unsignedshort_installer( RTType_unsignedshort );
#endif
