/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTObject_class.h>

static void ushort_init( const RTObject_class *, void * target )
{
    * static_cast<unsigned short *>(target) = 0U;
}

static void ushort_copy( const RTObject_class *,
			 void                 * target,
			 const void           * source )
{
    * static_cast<unsigned short *>(target) = * static_cast<const unsigned short *>(source);
}

#if OBJECT_DECODE
static int ushort_decode( const RTObject_class *,
			  void                 * target,
			  RTDecoding           * coding )
{
    return coding->get_ushort( * static_cast<unsigned short *>(target) );
}
#endif

#if OBJECT_ENCODE
static int ushort_encode( const RTObject_class *,
			  const void           * source,
			  RTEncoding           * coding )
{
    return coding->put_ushort( * static_cast<const unsigned short *>(source) );
}
#endif

const RTObject_class RTType_RTushort =
{
    nullptr,
    nullptr,
    "RTushort",
    0 /* RTVersionId */,
    sizeof( unsigned short ),
    ushort_init,
    ushort_copy,
	nullptr, // Not movable
#if OBJECT_DECODE
    ushort_decode,
#endif
#if OBJECT_ENCODE
    ushort_encode,
#endif
    RTnop_destroy,
    0, nullptr
};

#if OBJECT_DECODE
RTTypeInstaller RTushort_installer( RTType_RTushort );
#endif
