/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTEventInfo.h>
#include <RTMessage.h>
#include <RTObserver.h>

// return -1 if no daemon was interested
//         0 if at least one daemon was interested
//       > 0 if at least one daemon wanted to halt

int RTObserver::prepareEvent( RTEventInfo & eventInfo )
{
	int shouldHalt   = 0;
	int shouldReport = 0;

	for( int daemonId = 0; daemonId < daemons.size(); ++daemonId )
	{
		RTDaemon * theDaemon = static_cast<RTDaemon *>(daemons[ daemonId ]);

		if( theDaemon == nullptr )
			continue;

		// update the daemon's settings
		if( eventInfo.eventKind == RTEventInfo::EventPresence )
			theDaemon->setInfo( eventInfo );

		if( theDaemon->setting( eventInfo.eventKind ) == 0 )
			continue;

		if( eventInfo.eventKind != RTEventInfo::EventControl &&
				theDaemon->eventMatches( eventInfo ) == nullptr )
		{
			continue;
		}

		shouldReport  = 1;
		shouldHalt   |= theDaemon->shouldHalt();
	}

	if( shouldReport != 0 && superDaemon->shouldReport() != 0 )
	{
		if( shouldHalt != 0 )
			haltByProbe = 1;

		return shouldHalt;
	}

	return -1;
}
