/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTFieldDescriptor.h>
#include <RTMemoryUtil.h>
#include <RTObject_class.h>
#include <RTPointer.h>
#include <RTnew.h>

static RTByteBlock * _access( RTPointer * target )
{
	return target;
}

static void _init( const RTObject_class *, void * target )
{
	(void)new( target ) RTPointer;
}

static void _copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	(void)new( target ) RTPointer( * static_cast<const RTPointer *>(source) );
}

#if OBJECT_DECODE
static int _decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	RTPointer * data = static_cast<RTPointer *>(target);
	void      * block;

	if( coding->get_address( block ) == 0 )
		return 0;

	data->length   = 0;
	data->Contents = static_cast<char *>(block);

	return 1;
}
#endif

#if OBJECT_ENCODE
static int _encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	return coding->put_address( * static_cast<const RTPointer *>(source) );
}
#endif

static const RTFieldDescriptor _fields[] =
{
		{
				"RTPointer",
				0,
				&RTPointer::classData, nullptr
		}
};

const RTObject_class RTPointer::classData =
{
		&RTType_RTByteBlock,
		reinterpret_cast<RTSuperAccessFunction>(_access),
		"RTPointer",
		0 /* RTVersionId */,
		sizeof( RTPointer ),
		_init,
		_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		_decode,
#endif
#if OBJECT_ENCODE
		_encode,
#endif
		RTDataObject_destroy,
		1, _fields
};

#if OBJECT_DECODE
RTTypeInstaller RTPointer_installer( RTType_RTPointer );
#endif

const RTObject_class * RTPointer::getClassData( void ) const
{
	return &RTPointer::classData;
}
