/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTController.h>
#include <RTLayerConnector.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>

int RTProtocol::deregisterSPP( CallStyle style )
{
    RTController * controller = context();
    int            result     = 0;

#if OTRTSDEBUG >= DEBUG_VERBOSE
    controller->trace( RTController::LayerDeregisterSPP, this,
							 getRegisteredName() );
#endif

    if( ( _info->properties & RTPortDescriptor::MaskKind )
			   != RTPortDescriptor::KindUnwired ||
	( style == Programmatic &&
	( _info->properties & RTPortDescriptor::MaskRegisterLocked ) != 0 ) )
    {
	controller->setError( RTController::badOperation,
			      RTController::LayerDeregisterSPP,
			      this,
			      getRegisteredName() );
    }
    else if( isRegistered() == 0 || ( getFlags() & RegisteredAsSAP ) != 0 )
    {
	controller->setError( RTController::dereg,
			      RTController::LayerDeregisterSPP,
			      this,
			      getRegisteredName() );
    }
    else
    {
	RTLayerConnector::lock();

	_layer->deregister( controller );
	_layer = nullptr;

	RTLayerConnector::unlock();

	result = 1;
    }

    return result;
}
