/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTController.h>
#include <RTPortDescriptor.h>
#include <RTPriority.h>
#include <RTProtocol.h>
#include <RTProtocolDescriptor.h>

int RTProtocol::send( int signal,
		const void           * data,
		const RTObject_class * type,
		int                    priority,
		bool                   moveData )
{
	RTController * sender = context();

	sender->trace( RTController::CommSend, this, nullptr );

#if RTTYPECHECK_SEND != RTTYPECHECK_DONT
	switch( sendCheck( signal, type ) )
	{
	case RTProtocolDescriptor::badSignal:
		sender->setError( RTController::badSignal,
				RTController::CommSend, this, nullptr );
#if RTTYPECHECK_SEND == RTTYPECHECK_FAIL
		return 0;
#else
		break;
#endif

	case RTProtocolDescriptor::badType:
		sender->setError( RTController::badClass,
				RTController::CommSend, this, nullptr );
#if RTTYPECHECK_SEND == RTTYPECHECK_FAIL
		return 0;
#else
		break;
#endif

	case RTProtocolDescriptor::ok:
	default:
		break;
	}
#endif // RTTYPECHECK_SEND != RTTYPECHECK_DONT

	if( RTPriority::isValid( priority ) == 0 )
	{
		sender->setError( RTController::prio,
				RTController::CommSend, this, nullptr );
		return 0;
	}

	int   result      = 0;
	int   replication = _info->replication;
	End * end         = _ends;

	for( int index = 0; index < replication; ++index, ++end )
		result += end->send( sender, this, index,
				signal, data, type, priority, moveData );

	return result;
}
