/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTController.h>
#include <RTDebugger.h>
#include <RTLayerConnector.h>
#include <RTMessage.h>
#include <RTProtocol.h>

int RTProtocol::End::send( RTController         * sender,
		RTProtocol           * port,
		int                    index,
		int                    signal,
		const void           * data,
		const RTObject_class * type,
		int                    prio,
		bool					  moveData )
{
	RTMessage * msg = sender->newMsg();

	if( msg == nullptr )
	{
		sender->setError( RTController::noMem,
				RTController::CommSend, port, &index );
		return 0;
	}

	msg->setData( data, type );
	msg->setPriority( prio );

	if( signalMap != nullptr )
		msg->setSignal( signalMap[ signal - 1 ] );
	else
		msg->setSignal( signal );

	RTActor * remoteActor;

#if OBSERVABLE
	remoteActor = state != Unbound ? remote->getOwner()
			: nullptr;

	msg->setDestination( remoteActor, remote, remoteIndex );

	sender->debugger()->setTimestamp( msg, port, index );

	RTActor * owner = port->getOwner();

	if( owner->probe != nullptr )
		owner->probe->evtMessageOut( msg, RTEventInfo::SendNormal );
#endif

	if( state == Unbound )
	{
		sender->setError( RTController::noConnect,
				RTController::CommSend, port, &index );

		sender->freeMsg( msg );

		return 0;
	}

	msg->internData(moveData);

#if USE_THREADS
	// The binding may have been severed while we were busy interning the data.
	int need_lock = 0;

	switch( state )
	{
	case Unbound:
		sender->setError( RTController::noConnect,
				RTController::CommSend, port, &index );
		sender->freeMsg( msg );
		return 0;

	case BoundXThread:
		need_lock = 1;
		RTLayerConnector::lock();
		break;

	default:
		break;
	}
#endif

	remoteActor = remote->getOwner();
	msg->setDestination( remoteActor, remote, remoteIndex );

#if USE_THREADS
	remoteActor->context()->receive( msg, sender );
#else
	sender->receive( msg, sender );
#endif

#if USE_THREADS
	if( need_lock != 0 )
		RTLayerConnector::unlock();
#endif

	return 1;
}
