/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTBindingEnd.h>
#include <RTController.h>
#include <RTDiag.h>
#include <RTMessage.h>
#include <RTPortDescriptor.h>
#include <RTRootProtocol.h>

void RTProtocol::unbindAt( int index, RTController * initiator )
{
    RTAssert( 0 <= index && index < _info->replication );

    End * localEnd = &_ends[ index ];

    if( localEnd->state == End::Unbound )
	return;

    RTProtocol * remote      = localEnd->remote;
    int          remoteIndex = localEnd->remoteIndex;
    End        * remoteEnd   = &remote->_ends[ remoteIndex ];

    RTAssert( remoteEnd->state       == localEnd->state );
    RTAssert( remoteEnd->remote      == this );
    RTAssert( remoteEnd->remoteIndex == index );

    localEnd ->state = End::Unbound;
    remoteEnd->state = End::Unbound;

#if OTRTSDEBUG != DEBUG_NONE
    // Only the debugger might use these fields before the ends are bound again.
    localEnd ->remote      = nullptr;
    localEnd ->remoteIndex = 0;
    remoteEnd->remote      = nullptr;
    remoteEnd->remoteIndex = 0;
#endif

    if( initiator != nullptr )
    {
	RTProtocol * port = this;
	bool         done = false;

	for(;;)
	{
	    if( port->bindingNotificationRequested() != 0 )
	    {
		RTMessage * msg = initiator->newMsg();

		if( msg == nullptr )
		    initiator->setError( RTController::noMem,
					 RTController::SystemService, nullptr, nullptr );
		else
		{
		    RTActor      * owner      = port->getOwner();
		    RTController * controller = owner->context();

		    msg->setSignal( RTRootProtocol::rti_rtUnbound );
		    msg->setPriority( Background );
		    msg->setDestination( owner, port, index );

		    controller->receive( msg, initiator );
		}
	    }

	    if( done )
		break;

	    port  = remote;
	    index = remoteIndex;
	    done  = true;
	}
    }
}
