/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTObject_class.h>
#include <RTProtocolDescriptor.h>
#include <RTSignalDescriptor.h>

RTProtocolDescriptor::Status
RTProtocolDescriptor::canReceive( int                    signal,
				  const RTObject_class * type ) const
{
    const RTSignalDescriptor * rxSignal = inSignals;
    int                        rxCount  = numInSignals;

    for( ; --rxCount >= 0; ++rxSignal )
    {
	if( rxSignal->value == signal )
	{
	    if( rxSignal->type == nullptr )
		return ok; // any type is acceptable
	    else if( type == nullptr )
		return badType; // need a specific type
	    else if( type->isKindOf( rxSignal->type ) != 0 )
		return ok; // this type is acceptable
	    else
		return badType; // this type is not acceptable
	}
    }

    return badSignal; // no such signal
}
