/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTFieldDescriptor.h>
#include <RTObject_class.h>
#include <RTReal.h>
#include <RTnew.h>

static RTDataObject * _access( RTReal * target )
{
	return target;
}

static void _init( const RTObject_class *, void * target )
{
	(void)new( target ) RTReal;
}

static void _copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	(void)new( target ) RTReal( * static_cast<const RTReal *>(source) );
}

#if OBJECT_DECODE
static int _decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	RTReal * data = static_cast<RTReal *>(target);

	return coding->get_double( data->value );
}
#endif

#if OBJECT_ENCODE
static int _encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	const RTReal * data = static_cast<const RTReal *>(source);

	return coding->put_double( data->value );
}
#endif

static const RTFieldDescriptor _fields[] =
{
		{
				"RTReal",
				0,
				&RTReal::classData, nullptr
		}
};

const RTObject_class RTReal::classData =
{
		&RTType_RTDataObject,
		reinterpret_cast<RTSuperAccessFunction>(_access),
		"RTReal",
		0 /* RTVersionId */,
		sizeof( RTReal ),
		_init,
		_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		_decode,
#endif
#if OBJECT_ENCODE
		_encode,
#endif
		RTDataObject_destroy,
		1, _fields
};

#if OBJECT_DECODE
RTTypeInstaller RTReal_installer( RTType_RTReal );
#endif

const RTObject_class * RTReal::getClassData( void ) const
{
	return &RTReal::classData;
}
