/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDiag.h>
#include <RTRelayPort.h>

int RTRelayPort::reserve( const RTActorRef * home,
		int                id,
		int                count,
		int                size,
		RTActorRef       * importer )
{
	RTAssert( count > 0 );

	if( importer == nullptr )
	{
		// This is an incarnate operation: we already
		// verified that the replication is sufficient.
		return 1;
	}
	else
	{
		int indirect = size - directUse( home, id );

		if( indirect < count )
		{
			// There is no point looking to see if the list of _importers
			// has enough empty slots: the list is not big enough.
			return 0;
		}

		RTAssert( _importers != nullptr );

		for( int i = 0; i < indirect; ++i )
		{
			if( _importers[ i ] == nullptr )
			{
				_importers[ i ] = importer;
				if( --count == 0 )
					return 1; // we found enough empty slots
			}
			else if( _importers[ i ] == importer )
			{
				// Some ports already reserved by this importer.
				// Erase all reservations made for importer and complain.
				while( --i >= 0 )
				{
					if( _importers[ i ] == importer )
						_importers[ i ] = nullptr;
				}
				break;
			}
		}

		return 0;
	}
}
