/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor.h>
#include <RTBindingEnd.h>
#include <RTActorRef.h>
#include <RTLayerConnector.h>
#include <RTRelayPort.h>

void RTRelayPort::unreserve( RTActor    * owner,
		int          id,
		int          size,
		RTActorRef * importer, RTController * initiator)
{
	int          direct = directUse( owner->getReference(), id );
	RTBindingEnd end;

	if( importer != nullptr )
	{
		if( _importers != nullptr )
		{
			for( int repIndex = size - direct; --repIndex >= 0; )
			{
				if( _importers[ repIndex ] == importer )
				{
					_importers[ repIndex ] = nullptr;
					if( owner->_followInV( end, id, direct + repIndex ) != 0 )
					{
						RTLayerConnector::lock();
						RTLayerConnector::unbind( initiator, end );
						RTLayerConnector::unlock();



					}
				}
			}
		}
	}
	else
	{
		// Primary container should be the last actor to release reservations.

		for( int repIndex = direct; --repIndex >= 0; )
		{
			if( owner->_followInV( end, id, repIndex ) != 0 )
			{
				RTLayerConnector::lock();
				RTLayerConnector::unbind( initiator, end );
				RTLayerConnector::unlock();
			}
		}
	}
}
