/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTSoleController.h>
#include <RTTimerActor.h>
#include <RTTimespec.h>

void RTSoleController::printStats( void ) const
{
	RTController::printStats();

	RTDiagStream & out = RTDiag::out();

#if RTS_COUNT
	out << "\n  Timer[informIn]       : " << counts.timersStarted
			<< "\n  Timer[timeout]        : " << counts.timersExpired
			<< "\n  Timer[cancel]         : " << counts.timersCancelled;
#endif

	RTTimerActor * timerActor = RTTimerActor::soleInstance();

	if( timerActor != nullptr )
	{
		out << "\n  Timer[unexpired]      : " << timerActor->numPending();

#if OBJECT_ENCODE
		RTTimespec when;

		if( timerActor->nextTimeout( when ) )
		{
			RTTimespec now;
			RTTimespec::getclock( now );

			when -= now;

			out << "\n  Next timeout in ";
			out.write( &when, &RTType_RTTimespec );
		}
#endif
	}

	out << "\n";
}
