/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDiag.h>
#include <RTSoleController.h>
#include <RTSpecialPorts.h>
#include <RTSpecialSignals.h>
#include <RTTimerActor.h>

#include <errno.h>
#include <signal.h>

void RTSoleController::waitForEvents( void )
{
	RTActor * timerAgent;

	if( timer_interrupts != 0 )
	{
		timer_interrupts = 0;
		if( (timerAgent = RTTimerActor::soleInstance()) != nullptr )
			timerAgent->rtsBehavior( RTInterruptSignal, RTControlPort );
		return;
	}

	sigset_t mask_all;
	sigset_t mask_none;
	sigset_t mask_old;

	// mask all signals before checking signalCounts
	if( sigfillset( &mask_all ) != 0 )
		RTDiag::panic( "sigfillset" );
	if( sigprocmask( SIG_BLOCK, &mask_all, &mask_old ) != 0 )
		RTDiag::panic( "sigprocmask" );

	// if no signals occurred since the last call to sigsuspend we will wait
	if( timer_interrupts == 0 ) //lint !e774
	{
		// wait for SIGALRM which might trigger a timeout
		if( sigemptyset( &mask_none ) != 0 )
			RTDiag::panic( "sigemptyset" );
		if( ( sigsuspend( &mask_none ) == -1 ) && ( errno != EINTR ) )
			RTDiag::panic( "sigsuspend" );
	}

	// restore the signal mask
	if( sigprocmask( SIG_SETMASK, &mask_old, nullptr ) != 0 )
		RTDiag::panic( "sigprocmask" );
}
