/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTFieldDescriptor.h>
#include <RTObject_class.h>
#include <RTString.h>
#include <RTnew.h>
#include <utility>	// std::move

static RTDataObject * _access( RTString * target )
{
	return target;
}

static void _init( const RTObject_class *, void * target )
{
	(void)new( target ) RTString;
}

static void _copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	(void)new( target ) RTString( * static_cast<const RTString *>(source) );
}

static void _move( const RTObject_class *,
		void                * target,
		void           		* source )
{
	(void)new( target ) RTString( std::move(* static_cast<RTString *>(source)) );
}

#if OBJECT_DECODE
static int _decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	char * Contents;

	if( coding->get_string( Contents ) == 0 )
		return 0;
	else
	{
		RTString * string = static_cast<RTString *>(target);

		delete [] string->Contents;

		string->Contents = Contents;

		return 1;
	}
}
#endif

#if OBJECT_ENCODE
static int _encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	const RTString * data = static_cast<const RTString *>(source);

	return coding->put_string( *data );
}
#endif

static const RTFieldDescriptor _fields[] =
{
		{
				"RTString",
				0,
				&RTString::classData, nullptr
		}
};

const RTObject_class RTString::classData =
{
		&RTType_RTDataObject,
		reinterpret_cast<RTSuperAccessFunction>(_access),
		"RTString",
		0 /* RTVersionId */,
		sizeof( RTString ),
		_init,
		_copy,
		_move,
#if OBJECT_DECODE
		_decode,
#endif
#if OBJECT_ENCODE
		_encode,
#endif
		RTDataObject_destroy,
		1, _fields
};

#if OBJECT_DECODE
RTTypeInstaller RTString_installer( RTType_RTString );
#endif

const RTObject_class * RTString::getClassData( void ) const
{
	return &RTString::classData;
}
