/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActorClass.h>
#include <RTActorId.h>
#include <RTDiag.h>
#include <RTMain.h>
#include <RTSuperActor.h>
#include <RTSystemDescriptor.h>
#include <RTTimerActor.h>

#if USE_THREADS
#include <RTThread.h>
#include <RTTimerController.h>
#endif

void RTSuperActor::t1_initialize( void )
{
    RTActorId id;

    // create threads and special actors

#if USE_THREADS
    const RTSystemDescriptor * system = RTMain::systemInfo();

    timerController = new RTTimerController( context()->debugger() );
    timerThread     = new RTThread( timerController, system->timerStackSize,
						     system->timerPriority );
#else
    RTController * timerController = context();
#endif

    id = frame.incarnate( specials, RTTimerActor::classData, nullptr, nullptr,
			  timerController, 0 ); //lint !e1901
    RTAssert( id.isValid() != 0 );

    RTMain::_topReference = &application;

    if( timer.informIn( RTTimespec::zero ) == nullptr )
	RTDiag::panic( "super timer" );
}
