/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTIOMonitor.h>
#include <RTTcpSocket.h>
#include <RTinet.h>

int RTTcpSocket::write( const char * data, int size )
{
    switch( state() )
    {
	case SynSent:
	    return 0;

#ifdef _lint
	case Closed:
	case Listen:
#endif
	default:
	    return -1;

	case Established:
	    break;
    }

    if( size == 0 )
    {
	// we wrote all we wanted to; monitor write status no longer required
	if( _monitor != nullptr )
	    _monitor->remove( _sd, RTIOMonitor::CanWrite );
	return 0;
    }

    int count = RTinet_write( _sd, data, size );

    if( count < 0 )
    {
	close();
	return -1;
    }

    if( count == 0 && _monitor != nullptr )
	// we want to be notified when we can write
	_monitor->add( _sd, RTIOMonitor::CanWrite );

    return count;
}
