/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTTime.h>
#include <RTTimespec.h>

RTTime::RTTime( int hrs, int min, int sec )
: RTSequence()
, hours  ( hrs )
, minutes( min )
, seconds( sec )
{
}

RTTime::RTTime( const RTTime & other )
: RTSequence( other )
, hours  ( other.hours   )
, minutes( other.minutes )
, seconds( other.seconds )
{
}

RTTime::RTTime( const RTTimespec & ts )
: RTSequence()
, hours  ( 0 )
, minutes( 0 )
, seconds( 0 )
{
	const long secsPerDay = 24L * 60L * 60L;
	long       secOfDay   = ts.tv_sec % secsPerDay;

	if     ( ts.tv_nsec > 0L ) ++secOfDay;
	else if( ts.tv_nsec < 0L ) --secOfDay;

	if( secOfDay < 0L )
		secOfDay += secsPerDay;

	seconds = static_cast<int>( secOfDay % 60L ); secOfDay /= 60L;
	minutes = static_cast<int>( secOfDay % 60L ); secOfDay /= 60L;
	hours   = static_cast<int>( secOfDay );
}
