/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTTimerId.h>
#include <RTTimerNode.h>

RTTimerId & RTTimerId::operator=( const RTTimerId & other )
{
    if( this == &other )
    {
	// do nothing: assignment from self
    }
    else if( node == nullptr && other.node == nullptr )
    {
	// do nothing: both invalid
    }
    else
    {
#if USE_THREADS
	RTTimerNode::critical.enter();
#endif

	if( node != other.node )
	{
	    if( node != nullptr )
		RTTimerNode::_disconnect( this );

	    if( other.node != nullptr )
		other.node->_connect( this );
	}

#if USE_THREADS
	RTTimerNode::critical.leave();
#endif
    }

    return *this;
}

RTTimerId & RTTimerId::operator=( RTTimerNode * n )
{
    if( node != nullptr || n != nullptr )
    {
#if USE_THREADS
	RTTimerNode::critical.enter();
#endif

	if( node != n )
	{
	    if( node != nullptr )
		RTTimerNode::_disconnect( this );

	    if( n != nullptr )
		n->_connect( this );
	}

#if USE_THREADS
	RTTimerNode::critical.leave();
#endif
    }

    return *this;
}
