/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTController.h>
#include <RTMessage.h>
#include <RTSpecialSignals.h>
#include <RTTimerActor.h>
#include <RTTimerId.h>
#include <RTTimerNode.h>

void RTTimerNode::_cancel( RTController * controller )
{
	RTMessage * message = msg;

	if( message != nullptr )
	{
		msg = nullptr;

		if( message->signal == RTTimeoutSignal )
			// we must wait for the message to be delivered to finish up
			message = nullptr;

		RTTimerActor * timerActor = RTTimerActor::soleInstance();

		if( timerActor != nullptr )
			timerActor->cancel( this );
	}

	// invalidate all RTTimerId objects that refer to this node
	_disconnectAll();

	RTController * target = owner;

	if( active == 0 )
		target->freeTimerNode( this );

	if( message != nullptr )
	{
#if RTS_COUNT && USE_THREADS
		if( controller != target )
		{
			message->clear();
			message->setDestination( nullptr, nullptr, 0 );
			message->setPriority( Synchronous );
			message->setSignal( RTWakeupSignal );
			target->peer_receive( message );
		}
		else
#endif
		{
			controller->freeMsg( message );
		}
	}
}

int RTTimerNode::cancel( RTController * controller, RTTimerId & id )
{
	if( id.node == nullptr )
		return 0;

#if USE_THREADS
	RTTimerNode::critical.enter();
#endif

	RTTimerNode * node = id.node;

	if( node == nullptr )
	{
#if USE_THREADS
		RTTimerNode::critical.leave();
#endif
		return 0;
	}

	node->_cancel( controller );

#if USE_THREADS
	RTTimerNode::critical.leave();
#endif

	return 1;
}
