/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTMessage.h>
#include <RTTimerNode.h>
#include <RTTiming.h>

void RTTimerNode::setValue( RTMessage            * _msg,
			    Timing::Base         * _sap,
			    const RTTimespec     & _expireTime,
			    const RTTimespec     & _repeatInterval,
			    const void           * _data,
			    const RTObject_class * _type,
			    int                    _priority )
{
    RTTimerNode * outstanding = _sap->outstanding;

    if( outstanding == nullptr )
    {
	prev              = this;
	outstanding       = this;
	_sap->outstanding = this;
    }

    prev = outstanding->prev;
    next = outstanding;

    outstanding->prev->next = this;
    outstanding->prev       = this;

    msg            = _msg;
    sap            = _sap;
    expireTime     = _expireTime;
    repeatInterval = _repeatInterval;

    if( _type == nullptr || _type == &RTType_void )
	data = RTConstCast( void, _data );
    else
    {
	data = _type->allocate();
	_type->copy( data, _data );
    }

    type = _type;

    buildMsg( msg, _sap->getOwner(), _priority );
}
