
/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDiag.h>
#include <RTTimespec.h>

long RTTimespec::ticksFromNow( long nanos_per_tick ) const
{
	RTAssert( nanos_per_tick > 0L );

	RTTimespec now;

	getclock( now );

	if( *this <= now ) //lint !e909
		return 0L;

	RTTimespec delta( *this );

	delta -= now;

	long tick_rate = 1000000000L / nanos_per_tick;
	long result    = tick_rate * delta.tv_sec + delta.tv_nsec / nanos_per_tick;

	// avoid very long timeouts by limiting results
	if( result < 0L )
		result = static_cast<long>( (static_cast<unsigned long>(~0)) >> 1 );
	else if( result == 0L )
		// apply lower bound when this is in the future
		result = 1L;

	return result;
}
