/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTController.h>
#include <RTTimerActor.h>
#include <RTTimerNode.h>
#include <RTTiming.h>

#if OBSERVABLE
#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTDebugger.h>
#endif

RTTimerNode * Timing::Base::informIn( const RTTimespec     & when,
				      const RTTimespec     & repeatInterval,
				      const void           * data,
				      const RTObject_class * type,
				      int                    prio )
{
    RTTimerNode  * node       = nullptr;
    RTController * controller = context();
    RTTimerActor * timerActor;
    RTMessage    * msg;

    controller->trace( RTController::TimerInform, this, nullptr );

    if( RTPriority::isValid( prio ) == 0 )
    {
	controller->setError( RTController::prio,
			      RTController::TimerInform,
			      this, nullptr );
    }
    else if( (timerActor = RTTimerActor::soleInstance()) == nullptr )
    {
	controller->setError( RTController::noConnect,
			      RTController::TimerInform,
			      this, nullptr );
    }
    else if( (msg = controller->newMsg()) == nullptr )
    {
	controller->setError( RTController::noMem,
			      RTController::TimerInform,
			      this, nullptr );
    }
    else if( (node = controller->newTimerNode()) == nullptr )
    {
	controller->setError( RTController::noMem,
			      RTController::TimerInform,
			      this, nullptr );
	controller->freeMsg( msg );
    }
    else
    {
	node->setValue( msg, this, when, repeatInterval,
			data, type, prio );

	timerActor->informIn( node );

#if OBSERVABLE
	RTActor      * owner = getOwner();
	RTActorProbe * probe = owner->probe;

	if( probe != nullptr )
	{
	    RTMessage obsMsg;

	    obsMsg.setData( &when, &RTType_RTTimespec );

	    controller->debugger()->setTimestamp( &obsMsg, this, 0 );

	    if( repeatInterval == RTTimespec::zero ) //lint !e909
		probe->evtMessageOutIn( &obsMsg, RTEventInfo::SendInformIn );
	    else
		probe->evtMessageOutIn( &obsMsg, RTEventInfo::SendInformEvery );
	}
#endif
    }

    return node;
}
