/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTFormat.h>
#include <RTMemoryUtil.h>
#include <RTToolsetObserver.h>

char * RTToolsetObserver::buildString( const char * actor_path,
		int          actor_index,
		const char * component_name )
{
	char * result;
	int    path_length = RTMemoryUtil::strlen( actor_path );
	int    name_length = component_name != nullptr
			? RTMemoryUtil::strlen( component_name )
	: 0;
	int    num_length;
	char   num[ 20 ];

	if( path_length == 1 ) // 1: "/" - the top actor
	{
		if( component_name == nullptr )
			result = RTMemoryUtil::strdup( actor_path, path_length );
		else
		{
			result = new char [ static_cast<unsigned>( name_length + 2 ) ];

			result[ 0 ] = '/';
			RTMemoryUtil::strcpy( &result[ 1 ], component_name, name_length );
		}
	}
	else if( (num_length = RTFormat::_int( actor_index, num,
			static_cast<int>(sizeof( num )) )) == 0 )
		result = nullptr;
	else
	{
		int total_length = path_length + num_length + 2;

		if( component_name != nullptr )
			total_length += name_length + 1;

		result = new char [ static_cast<unsigned>(total_length) ];

		RTMemoryUtil::memcpy( result, actor_path, path_length ); //lint !e668
		result[ path_length ] = ':';
		RTMemoryUtil::strcpy( &result[ path_length + 1 ], num, num_length );
		if( component_name != nullptr )
		{
			int actor_length = path_length + num_length + 1;

			result[ actor_length ] = '/';
			RTMemoryUtil::strcpy( &result[ actor_length + 1 ],
					component_name, name_length );
		}
	}

	return result;
}
