/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDebugger.h>
#include <RTToolsetObserver.h>

void RTToolsetObserver::encoDeco( char * buffer, int bufLen )
{
    rxBuffer.data = buffer;
    rxBuffer.size = bufLen;

    (void)TO_decodeBuffer( &rxBuffer, &structInfo );

    if( rxBuffer.status != DEC_SUCCESS )
	return;

    int daemonExists = setDaemon( structInfo.daemon.number );

    if( ( structInfo.func == CMD_CREATE_DAEMON && daemonExists != 0 ) ||
	( structInfo.func >  CMD_CREATE_DAEMON && daemonExists == 0 ) )
    {
	sendResultCode( RES_BAD_DAEMON );
	return;
    }

    switch( structInfo.func )
    {
	case CMD_CREATE_DAEMON:
	case CMD_UPDATE_DAEMON:
	case CMD_COMMAND_DAEMON:
	case CMD_REFRESH_DAEMON:
	case CMD_DESTROY_DAEMON:
	    handleDaemon();
	    break;

	case CMD_ADD_ACTOR:
	case CMD_DEL_ACTOR:
	    handleActor();
	    break;

	case CMD_ADD_PORT:
	case CMD_DEL_PORT:
	    handlePort();
	    break;

	case CMD_ADD_STATE:
	case CMD_DEL_STATE:
	    handleState();
	    break;

	case CMD_ADD_CHAIN:
	case CMD_DEL_CHAIN:
	    handleChain();
	    break;

	case CMD_ADD_VAR:
	case CMD_DEL_VAR:
	case CMD_START_VAR_MON:
	case CMD_STOP_VAR_MON:
	    handleVariable();
	    break;

	case CMD_SET_VAR:
	case CMD_GET_VAR:
	    accessVariable();
	    break;

	case CMD_INJECT_MSG:
	    handleInject();
	    break;

        case CMD_GET_MESSAGES:
            handleGetMessages( );
            break;

	case CMD_CLOSE_SESSION:
	    sendResultCode( RES_OK );
	    break;

	default: // I can't handle this kind of function.
	    sendResultCode( RES_BAD_FUNC );
	    break;
    }

    structInfo.daemon.number = -1;
    TO_freeStructInfo( &structInfo );
    structInfo.daemon.invokeID = nullptr;

    (void)setDaemon( -1 );
}
