/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTEventInfo.h>
#include <RTMemoryUtil.h>
#include <RTToolsetObserver.h>

int RTToolsetObserver::evtChain( int daemonId,
		const RTDaemonInfo & daemonInfo,
		const RTEventInfo  & eventInfo )
{
	const RTDaemon * theDaemon = static_cast<const RTDaemon *>(daemons[ daemonId ]);

	if( theDaemon == nullptr )
		return 0;

	int shouldHalt = theDaemon->shouldHalt();

	if( theDaemon->shouldReport() == 0 )
		return shouldHalt;

	TO_evtChain   toSendData;
	TO_daemonInfo toDaemonInfo;

	toDaemonInfo.number   = daemonId;
	toDaemonInfo.invokeID = nullptr;

	char * tempActor   = buildString( daemonInfo.actorPath,
			eventInfo.actorIndex );

	toSendData.time.sec  = eventInfo.timestamp.tv_sec;
	toSendData.time.nsec = eventInfo.timestamp.tv_nsec;

	toSendData.actor = tempActor;
	toSendData.state = eventInfo.chainStrOrigin ? RTMemoryUtil::strdup( eventInfo.chainStrOrigin ) : nullptr;
	toSendData.chain = eventInfo.chainStrTrans ? RTMemoryUtil::strdup( eventInfo.chainStrTrans  ) : nullptr;

	if( TO_encodeEvtChain( &txBuffer, &toDaemonInfo, &toSendData )
			== ENC_SUCCESS )
	{
		sendTxBuffer();
	}

	delete [] toSendData.chain;
	delete [] toSendData.state;
	delete [] toSendData.actor;

	return shouldHalt;
}
