/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTEventInfo.h>
#include <RTToolsetObserver.h>

int RTToolsetObserver::evtHistory( int daemonId,
		const RTDaemonInfo & daemonInfo,
		const RTEventInfo  & eventInfo )
{
	const RTDaemon * theDaemon = static_cast<const RTDaemon *>(daemons[ daemonId ]);

	if( theDaemon == nullptr )
		return 0;

	int shouldHalt = theDaemon->shouldHalt();

	if( theDaemon->shouldReport() == 0 )
		return shouldHalt;

	TO_evtHistory toSendData;
	TO_daemonInfo toDaemonInfo;

	toDaemonInfo.number   = daemonId;
	toDaemonInfo.invokeID = nullptr;

	if( eventInfo.observerId     == observerId &&
			structInfo.daemon.number == daemonId ) // it's a request
	{
		toDaemonInfo.invokeID = structInfo.daemon.invokeID;
	}

	char * tempActor = buildString( daemonInfo.actorPath,
			eventInfo.actorIndex );

	// set the state/history/current strings
	toSendData.state   = daemonInfo.stateStrTarget;
	toSendData.history = eventInfo.histStrHist;
	toSendData.current = eventInfo.histStrCurr;

	toSendData.actor   = tempActor;

	if( TO_encodeEvtHistory( &txBuffer, &toDaemonInfo, &toSendData )
			== ENC_SUCCESS )
	{
		sendTxBuffer();
	}

	delete [] tempActor;

	return shouldHalt;
}
