/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTAsciiEncoding.h>
#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTEventInfo.h>
#include <RTMessage.h>
#include <RTPriority.h>
#include <RTToolsetObserver.h>
#include <RTTruncatingBuffer.h>

int RTToolsetObserver::evtMessageIn( int daemonId,
		const RTDaemonInfo & daemonInfo,
		const RTEventInfo  & eventInfo )
{
	const RTDaemon * theDaemon = static_cast<const RTDaemon *>(daemons[ daemonId ]);

	if( theDaemon == nullptr )
		return 0;

	int shouldHalt = theDaemon->shouldHalt();

	if( theDaemon->shouldReport() == 0 )
		return shouldHalt;

	TO_evtMsgIn   toSendData;
	TO_daemonInfo toDaemonInfo;

	toDaemonInfo.number   = daemonId;
	toDaemonInfo.invokeID = nullptr;

	toSendData.recTime.sec  = eventInfo.timestamp.tv_sec;
	toSendData.recTime.nsec = eventInfo.timestamp.tv_nsec;

	toSendData.message.signal = eventInfo.signalName;
	toSendData.message.prio   = eventInfo.message->getPriority();

	char                 * msgDataBuf = new char [ msgDataBufSize ];
	const RTObject_class * type;

	if( eventInfo.message->data == nullptr )
		type = nullptr;
	else
		type = eventInfo.message->getType();

	if( type == nullptr )
		msgDataBuf[ 0 ] = '\0';
	else
	{
		RTTruncatingBuffer buffer( msgDataBuf, msgDataBufSize );
		RTAsciiEncoding    coding( &buffer );

		(void)coding.put( eventInfo.message->data, type );
		(void)buffer.write( "", 1 );
	}

	toSendData.message.data = msgDataBuf;

	char * tempActor = buildString( daemonInfo.actorPath,
			eventInfo.actorIndex );

	toSendData.recActor = tempActor;
	toSendData.recPort  = eventInfo.portName;

	if( TO_encodeEvtMsgIn( &txBuffer, &toDaemonInfo, &toSendData )
			== ENC_SUCCESS )
	{
		sendTxBuffer();
	}

	delete [] tempActor;
	delete [] msgDataBuf;

	return shouldHalt;
}
