#include <RTAsciiEncoding.h>
#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTEventInfo.h>
#include <RTMemoryUtil.h>
#include <RTMessage.h>
#include <RTToolsetObserver.h>
#include <RTTruncatingBuffer.h>

using namespace std;

int RTToolsetObserver::evtMessageQueue( int daemonId,
		const RTDaemonInfo & daemonInfo,
		const RTEventInfo & eventInfo)
{
	const RTDaemon * theDaemon = static_cast<const RTDaemon *>(daemons[ daemonId ]);

	if( theDaemon == nullptr )
		return 0;

	if( theDaemon->shouldReport() == 0 )
		return -1;

	TO_evtMsgQueue toSendData;
	TO_daemonInfo toDaemonInfo;

	toDaemonInfo.number   = daemonId;
	toDaemonInfo.invokeID = nullptr;

	char * tempActor = buildString( daemonInfo.actorPath,
			eventInfo.actorIndex );

	toSendData.actor = tempActor;

	if( eventInfo.message )
	{
		toSendData.port = eventInfo.portName;

		// create the signal/prio/data part
		char                 * msgDataBuf = new char [ msgDataBufSize ];
		const RTObject_class * type;

		if( eventInfo.message->data == nullptr )
			type = nullptr;
		else
			type = eventInfo.message->getType();

		if( type == nullptr )
			msgDataBuf[ 0 ] = '\0';
		else
		{
			RTTruncatingBuffer buffer( msgDataBuf, msgDataBufSize );
			RTAsciiEncoding    coding( &buffer );

			coding.put( eventInfo.message->data, type );
			buffer.write( "", 1 );
		}

		toSendData.message.signal = eventInfo.message->getSignalName();
		toSendData.message.prio = eventInfo.message->getPriority();
		toSendData.message.data = msgDataBuf;

		if( TO_encodeEvtMsgQueue( &txBuffer, &toDaemonInfo, &toSendData ) == ENC_SUCCESS )
			sendTxBuffer();

		delete msgDataBuf;
		delete tempActor;
	}
	else
	{
		toSendData.port = nullptr;
		RTMemoryUtil::memset( &toSendData.message, 0, sizeof( toSendData.message ) );

		if( TO_encodeEvtMsgQueue( &txBuffer, &toDaemonInfo, &toSendData ) == ENC_SUCCESS )
			sendTxBuffer();
	}

	return -1;
}
