/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActorClass.h>
#include <RTActor_class.h>
#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTEventInfo.h>
#include <RTFormat.h>
#include <RTToolsetObserver.h>

static const char * getTypeName( const RTActor_class * info )
{
	if( info == nullptr )
		return nullptr;

	return info->type->name;
}

int RTToolsetObserver::evtPresence( int                  daemonId,
		const RTDaemonInfo & daemonInfo,
		const RTEventInfo  & eventInfo )
{
	const RTDaemon * theDaemon = static_cast<RTDaemon *>(daemons[ daemonId ]);

	if( theDaemon == nullptr )
		return 0;

	int shouldHalt = theDaemon->shouldHalt();

	if( theDaemon->shouldReport() == 0 )
		return shouldHalt;

	TO_evtPresence toSendData;
	TO_daemonInfo  toDaemonInfo;

	toDaemonInfo.number   = daemonId;
	toDaemonInfo.invokeID = nullptr;

	if( eventInfo.observerId     == observerId &&
			structInfo.daemon.number == daemonId ) // it's a request
					{
		toDaemonInfo.invokeID = structInfo.daemon.invokeID;
					}

	toSendData.actor = buildString( daemonInfo.actorPath,
			eventInfo.actorIndex );

	toSendData.time.sec  = eventInfo.timestamp.tv_sec;
	toSendData.time.nsec = eventInfo.timestamp.tv_nsec;

	char actorID[ 20 ];
	actorID[ 0 ] = '\0';

	if( eventInfo.actor != nullptr )
		(void)RTFormat::_void( eventInfo.actor, actorID, static_cast<int>(sizeof( actorID )) );

	toSendData.actorID = actorID;
	toSendData.classID = getTypeName( eventInfo.info );

	if( TO_encodeEvtPresence( &txBuffer, &toDaemonInfo, &toSendData )
			== ENC_SUCCESS )
	{
		sendTxBuffer();
	}

	delete [] toSendData.actor;

	return shouldHalt;
}
