/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTActor_class.h>
#include <RTComponentDescriptor.h>
#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTEventInfo.h>
#include <RTToolsetObserver.h>

int RTToolsetObserver::evtReference( int                  daemonId,
		const RTDaemonInfo & daemonInfo,
		const RTEventInfo  & eventInfo )
{
	const RTDaemon * theDaemon = static_cast<const RTDaemon *>(daemons[ daemonId ]);

	if( theDaemon == nullptr )
		return 0;

	int shouldHalt = theDaemon->shouldHalt();

	if( theDaemon->shouldReport() == 0 )
		return shouldHalt;

	const RTActor_class * info = daemonInfo.info;

	if( info == nullptr )
		return 0;

	TO_evtReference toSendData;
	TO_daemonInfo   toDaemonInfo;

	toDaemonInfo.number   = daemonId;
	toDaemonInfo.invokeID = nullptr;

	if( eventInfo.observerId     == observerId &&
			structInfo.daemon.number == daemonId ) // it's a request
					{
		toDaemonInfo.invokeID = structInfo.daemon.invokeID;
					}

	toSendData.time.sec  = eventInfo.timestamp.tv_sec;
	toSendData.time.nsec = eventInfo.timestamp.tv_nsec;
	toSendData.actorRef  = buildString( daemonInfo.actorPath,
			daemonInfo.actorIndex,
			info->components[ eventInfo.compIndex ]
							  .name );
	toSendData.incarns   = eventInfo.nrIncarns;

	if( TO_encodeEvtReference( &txBuffer, &toDaemonInfo, &toSendData )
			== ENC_SUCCESS )
	{
		sendTxBuffer();
	}

	delete [] toSendData.actorRef;

	return shouldHalt;
}
