/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDaemon.h>
#include <RTEventInfo.h>
#include <RTToolsetObserver.h>

int RTToolsetObserver::evtRtsStatus( int daemonId, const RTEventInfo & eventInfo )
{
	const RTDaemon * theDaemon = static_cast<const RTDaemon *>(daemons[ daemonId ]);

	if( theDaemon == nullptr )
		return 0;

	// check for flow commands, if so, do not send anything
	if( eventInfo.targetMode == RTEventInfo::Halted )
	{
		if( lastFlowCmd == FlowPause && haltByProbe == 0 )
			return 0;
	}
	else if( lastFlowCmd == FlowResume )
		return 0;

	TO_evtRtsStatus toSendData;
	TO_daemonInfo   toDaemonInfo;

	toDaemonInfo.number   = daemonId;
	toDaemonInfo.invokeID = nullptr;

	if( eventInfo.observerId     == observerId &&
			structInfo.daemon.number == daemonId ) // it's a request
	{
		toDaemonInfo.invokeID = structInfo.daemon.invokeID;
	}

	toSendData.time.sec  = eventInfo.timestamp.tv_sec;
	toSendData.time.nsec = eventInfo.timestamp.tv_nsec;
	toSendData.status    = eventInfo.targetMode == RTEventInfo::Halted
			? ( haltByProbe != 0 ? RTS_STATUS_HALTED : RTS_STATUS_STOPPED )
					: RTS_STATUS_RUNNING;

	toSendData.byYou     = 0;

	if( TO_encodeEvtRtsStatus( &txBuffer, &toDaemonInfo, &toSendData )
			== ENC_SUCCESS )
	{
		sendTxBuffer();
	}

	return 0;
}
