/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTEventInfo.h>
#include <RTMemoryUtil.h>
#include <RTToolsetObserver.h>

int RTToolsetObserver::evtVariable( int                  daemonId,
		const RTDaemonInfo & daemonInfo,
		const RTEventInfo  & eventInfo )
{
	const RTDaemon * theDaemon = static_cast<const RTDaemon *>(daemons[ daemonId ]);

	if( theDaemon == nullptr )
		return 0;

	int shouldHalt = theDaemon->shouldHalt();

	if( theDaemon->shouldReport() == 0 )
		return shouldHalt;

	TO_evtVariable toSendData;
	TO_daemonInfo  toDaemonInfo;

	toDaemonInfo.number   = daemonId;
	toDaemonInfo.invokeID = nullptr;

	toSendData.time.sec   = eventInfo.timestamp.tv_sec;
	toSendData.time.nsec  = eventInfo.timestamp.tv_nsec;

	// create the actor string
	char * tempActor = buildString( daemonInfo.actorPath,
			eventInfo.actorIndex );

	// create the variable & value strings
	toSendData.variable = RTMemoryUtil::strdup( eventInfo.varName  );
	toSendData.value    = RTMemoryUtil::strdup( eventInfo.varValue );

	toSendData.actor    = tempActor;

	if( TO_encodeEvtVariable( &txBuffer, &toDaemonInfo, &toSendData )
			== ENC_SUCCESS )
	{
		sendTxBuffer();
	}

	delete [] toSendData.variable;
	delete [] toSendData.value;
	delete [] tempActor;

	return shouldHalt;
}
