/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTAsciiDecoding.h>
#include <RTDaemon.h>
#include <RTMemoryInBuffer.h>
#include <RTMemoryUtil.h>
#include <RTToolsetObserver.h>
#include <RTUnknownObject.h>

void RTToolsetObserver::handleInject( void )
{
	TO_injectMsg         * injMsg = static_cast<TO_injectMsg *>(structInfo.ptr);
	void                 * data   = nullptr;
	const RTObject_class * type   = nullptr;
	int                    dataLen;

	if( (dataLen = RTMemoryUtil::strlen( injMsg->data )) > 0 )
	{
		RTMemoryInBuffer source( injMsg->data, dataLen );
		RTAsciiDecoding  decoder( &source );

		if( decoder.get( &data, &type ) == 0 )
		{
			sendResultCode( RES_BAD_MESSAGE );
			return;
		}
	}

	if( type == &RTType_RTUnknownObject )
	{
		sendResultCode( RES_BAD_MESSAGE );
	}
	else if( daemon->inject( injMsg->inwards,
			injMsg->signal,
			injMsg->prio,
			data, type ) == 0 )
		sendResultCode( RES_BAD_MESSAGE );
	else
		sendResultCode( RES_OK );

	if( type != nullptr )
		type->destroy( data, RTObject_class::DestroyAndDeallocate );
}
