/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTToolsetObserver.h>

#include <RTDaemon.h>
#include <RTDaemonInfo.h>
#include <RTDebugger.h>

void RTToolsetObserver::handleVariable( void )
{
    TO_variableInfo * myVarInfo  = static_cast<TO_variableInfo *>(structInfo.ptr);
    RTDaemonInfo    * daemonInfo = new RTDaemonInfo;

    daemonInfo->setInfoVariable( myVarInfo->actor, myVarInfo->variable );

    switch (structInfo.func)
    {
        case CMD_ADD_VAR:
        {
            int probeNr = addInfo( daemonInfo );
            if( probeNr >= 0 )
            {
                sendResultCode( RES_OK );
                debugger->getPresence( probeNr,
                                       observerId,
                                       structInfo.daemon.number );
            }
            else
            {
                sendResultCode( RES_BAD_VAR );
                delete daemonInfo;
            }
        }
        break;
        case CMD_DEL_VAR:
        {
            if( delInfo( daemonInfo ) <= 0 ) // number of actors deleted
                sendResultCode( RES_BAD_VAR );
            else
                sendResultCode( RES_OK );

            delete daemonInfo;
        }
        break;
        case CMD_START_VAR_MON:
        case CMD_STOP_VAR_MON:
        {
            bool enable = structInfo.func == CMD_START_VAR_MON;
            if( daemon->monitorField( daemonInfo, enable ) )
                sendResultCode( RES_BAD_VAR );
            else
                sendResultCode( RES_OK );

            delete daemonInfo;
        }
    }
}
