/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTMemoryUtil.h>
#include <RTToolsetObserver.h>

int RTToolsetObserver::receiveData( void )
{
	enum { lengthDigits = 4 };
	int    charsNeeded  = lengthDigits;

	if( bufUsed < lengthDigits )
	{
#if PURIFY
		inBuffer[ lengthDigits ] = '\0'; // avoid UMR
#endif
	}
	else
	{
		char saved = inBuffer[ lengthDigits ];

		inBuffer[ lengthDigits ] = '\0';

		charsNeeded += RTMemoryUtil::atoi( inBuffer );

		inBuffer[ lengthDigits ] = saved;
	}

	if( bufSize <= charsNeeded )
	{
		bufSize  = charsNeeded + 1;

		char * newBuffer = new char [ static_cast<unsigned>(bufSize) ];

		RTMemoryUtil::memcpy( newBuffer, inBuffer, bufUsed ); //lint !e668 !e669

		delete [] inBuffer;

		inBuffer = newBuffer;
	}

	int numRead = readData( &inBuffer[ bufUsed ],
			charsNeeded - bufUsed ); //lint !e662

	if( numRead <= 0 )
		return numRead;

	bufUsed += numRead;

	if( bufUsed <= lengthDigits || bufUsed < charsNeeded )
		return 0; // entire msg not ready yet

	encoDeco( &inBuffer[ lengthDigits ], bufUsed - lengthDigits );

	bufUsed = 0;

	return 1;
}
