/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifdef PRAGMA
#pragma implementation "RTUnknownObject.h"
#endif

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTFieldDescriptor.h>
#include <RTFieldOffset.h>
#include <RTMemoryUtil.h>
#include <RTObject_class.h>
#include <RTTypeModifier.h>
#include <RTUnknownObject.h>

static void unknown_copy( const RTObject_class *,
		void                 * _target,
		const void           * _source )
{
	const RTUnknownObject * source = static_cast<const RTUnknownObject *>(_source);
	RTUnknownObject       * target = static_cast<RTUnknownObject *>(_target);

	target->signature = RTMemoryUtil::strdup( source->signature );
	target->length    = source->length;
	target->code      = static_cast<char *>(RTMemoryUtil::memdup( source->code,
			source->length ));
}

static void unknown_destroy( const RTObject_class *, void * target )
{
	RTUnknownObject * data = static_cast<RTUnknownObject *>(target);

	delete [] data->signature;
	delete [] data->code;

	data->signature = nullptr;
	data->length    = 0;
	data->code      = nullptr;
}

int RTUnknownObject_length( const RTTypeModifier *, const void * source )
{
	const RTUnknownObject * data = static_cast<const RTUnknownObject *>(source);

	return data->length;
}

static const RTTypeModifier coding_modifier =
{
		RTUnknownObject_length, 0, 1
};

static const RTFieldDescriptor _fields[] =
{
		{
				"signature",
				RTOffsetOf( RTUnknownObject, signature ),
				&RTType_RTpchar,nullptr
		},
		{
				"length",
				RTOffsetOf( RTUnknownObject, length ),
				&RTType_int,nullptr
		},
		{
				"code",
				RTOffsetOf( RTUnknownObject, code ),
				&RTType_char,
				&coding_modifier
		}
};

const RTObject_class RTUnknownObject::classData =
{
		nullptr,
		nullptr,
		"RTUnknownObject",
		0 /* RTVersionId */,
		sizeof( RTUnknownObject ),
		RTstruct_init,
		unknown_copy,
		nullptr, // Not movable
		RTstruct_decode,
#if OBJECT_ENCODE
		RTstruct_encode,
#endif
		unknown_destroy,
		3, _fields
};

RTTypeInstaller RTUnknownObject_installer( RTType_RTUnknownObject );
