/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDataObject.h>
#include <RTObject_class.h>
#include <RTVAsciiDecoding.h>
#include <RTUnknownObject.h>

int RTVAsciiDecoding::get( void ** data, RTObject_class const ** type )
{
	long version;
	char name[ 200 ];

	if( getName( name, sizeof( name ) ) == 0 )
		return 0;

	if( (*type = RTObject_class::lookup( name )) == nullptr )
	{
		RTUnknownObject * unknown = getUnknown( name );

		if( unknown == nullptr )
			return 0;

		*data = unknown;
		*type = &RTType_RTUnknownObject;

		return 1;
	}

	if( get_long( version ) == 0 )
		return 0;

	if( version != static_cast<long>((*type)->_version ))
		return 0;

	*data = (*type)->allocate();

	if( *data == nullptr )
		return 0; // no space?

	(*type)->init( *data );

	if( (*type)->decode( *data, this ) == 0 )
	{
		(*type)->destroy( *data, RTObject_class::DestroyAndDeallocate );
		return 0;
	}

	return 1;
}
