/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDecoding.h>
#include <RTEncoding.h>
#include <RTFieldDescriptor.h>
#include <RTWrapper.h>
#include <RTnew.h>

static void _init( const RTObject_class *, void * target )
{
	(void)new( target ) RTWrapper;
}

static void _copy( const RTObject_class *,
		void                 * target,
		const void           * source )
{
	(void)new( target ) RTWrapper( * static_cast<const RTWrapper *>(source) );
}

#if OBJECT_DECODE
static int _decode( const RTObject_class *,
		void                 * target,
		RTDecoding           * coding )
{
	void                 * data;
	const RTObject_class * type;

	if( coding->get( &data, &type ) == 0 )
		return 0;
	else
	{
		RTWrapper new_value( data, type );

		* static_cast<RTWrapper *>(target) = new_value;
	}

	return 1;
}
#endif

#if OBJECT_ENCODE
static int _encode( const RTObject_class *,
		const void           * source,
		RTEncoding           * coding )
{
	const RTWrapper * wrapper = static_cast<const RTWrapper *>(source);

	return wrapper->type()->encode( wrapper->data(), coding );
}
#endif

static void _destroy( const RTObject_class *, void * target )
{
	static_cast<RTWrapper *>(target)->~RTWrapper();
}

const RTObject_class RTWrapper::classData =
{
		nullptr,
		nullptr,
		"RTWrapper",
		0 /* RTVersionId */,
		sizeof( RTWrapper ),
		_init,
		_copy,
		nullptr, // Not movable
#if OBJECT_DECODE
		_decode,
#endif
#if OBJECT_ENCODE
		_encode,
#endif
		_destroy,
		0, nullptr
};

#if OBJECT_DECODE
RTTypeInstaller RTWrapper_installer( RTType_RTWrapper );
#endif
