/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTMemoryUtil.h>
#include <RTinet.h>
#include <RTtcp.h>

extern "C" int RTinet_accept( int listener,
		RTinet_address * address,
		RTinet_port    * port )
{
	int               sock;
	sockaddr_in       peer;
	SocketAddressSize peer_size = sizeof( peer );

	for(;;)
	{
		//lint -save -e740
		sock = INET_accept( listener,
				reinterpret_cast<SocketAddress *>(&peer),
				&peer_size );
		//lint -restore

		if( sock == INET_NO_SOCKET )
		{
#if defined( INET_EINTR ) || defined( INET_EAGAIN )
			int err = INET_error( listener );
#endif

#ifdef INET_EINTR
			if( err == INET_EINTR )
				continue;
#endif

#ifdef INET_EAGAIN
			if( err == INET_EAGAIN )
				continue;
#endif
		}
		else if( RTinet_async( sock ) == 0 || RTinet_nonblocking( sock ) == 0 )
		{
			RTinet_close( sock );
			sock = INET_NO_SOCKET;
		}
		else
		{
			RTMemoryUtil::memcpy( address, &peer.sin_addr,
					static_cast<int>(sizeof( *address ) ));
			*port = ntohs( peer.sin_port );
		}

		return sock;
	}
}
