/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTMemoryUtil.h>
#include <RTinet.h>
#include <RTtcp.h>

extern "C" int RTinet_connect( int sock,
		const RTinet_address * address,
		RTinet_port            port )
{
	sockaddr_in addr;

	RTMemoryUtil::memset( &addr, 0, static_cast<int>(sizeof( addr )) );
	addr.sin_family = AF_INET;
	addr.sin_port   = htons( port );
	RTMemoryUtil::memcpy( &addr.sin_addr, address, static_cast<int>(sizeof( *address )) );

	for(;;)
	{
		//lint -save -e740
		if( INET_connect( sock,
				(SocketAddress *)&addr,
				static_cast<int>(sizeof( addr )) ) == 0 )
			//lint -restore
		{
			return 1;
		}

		switch( INET_error( sock ) )
		{
#ifdef INET_EINTR
		case INET_EINTR:
			continue;
#endif

#ifdef INET_EALREADY
		case INET_EALREADY:
			return 0;
#endif

#ifdef INET_EINPROGRESS
		case INET_EINPROGRESS:
			return 0;
#endif

#ifdef INET_EWOULDBLOCK
		case INET_EWOULDBLOCK:
			return 0;
#endif

		case 0: // this is strange, but it is how things work on some targets
		case INET_EISCONN:
#ifdef INET_EMAYBECONN
		case INET_EMAYBECONN:
#endif
			return 1;

		default:
			return -1;
		}
	}
}
