/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTFieldOffset.h>
#include <RTMemoryUtil.h>
#include <RTinet.h>
#include <RTtcp.h>

extern "C" int RTinet_primary( int sock, RTinet_address * address )
{
	if( sock == INET_NO_SOCKET )
		return 0;

	struct ifconf ifc;
	struct ifreq  ifr;
	struct ifreq  buf[ 20 ];

	ifc.ifc_len = sizeof( buf );
	ifc.ifc_req = buf;

	// get list of configurations of all interfaces in the host
	if( INET_ioctl( sock, SIOCGIFCONF, &ifc ) != 0 ) //lint !e740
		return 0;

	int          avail;
	const char * cursor;
	size_t       len    = sizeof( ifr );

	for( cursor = ifc.ifc_buf, avail  = ifc.ifc_len;
			avail >= static_cast<int>(sizeof( ifr ));
			cursor += len,        avail -= len )
	{
		RTMemoryUtil::memcpy( &ifr, cursor, static_cast<int>(sizeof( ifr )) );

#if INET_HaveLength
		len = ifr.ifr_addr.sa_len + RTOffsetOf( struct ifreq, ifr_addr );
#endif

#if defined( IFF_LOOPBACK ) || defined( IFF_UP ) || defined( IFF_RUNNING )
		// Only issue the ioctl where these flags exist.
		// skip an interface if we can't get its flags
		if( INET_ioctl( sock, SIOCGIFFLAGS, &ifr ) != 0 )
			continue;

		// skip loopback interfaces
		if( ( ifr.ifr_flags & IFF_LOOPBACK ) != 0 )
			continue;

		// skip interfaces that are not UP and RUNNING
		if( ( ifr.ifr_flags & ( IFF_UP | IFF_RUNNING ) )
				!= ( IFF_UP | IFF_RUNNING ) )
			continue;
#endif

		// skip an interface if we can't get its address
		if( INET_ioctl( sock, SIOCGIFADDR, &ifr ) != 0 )
			continue;

		// we are only interested in internet interfaces
		if( ifr.ifr_addr.sa_family != AF_INET )
			continue;

		// this interface passes

		sockaddr_in * ip = reinterpret_cast<sockaddr_in *>(&ifr.ifr_addr);

		RTMemoryUtil::memcpy( address, &ip->sin_addr, static_cast<int>(sizeof( *address )) );

		return 1;
	}

	return 0;
}
