/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTMemoryUtil.h>
#include <RTinet.h>
#include <RTtcp.h>

extern "C" int RTinet_recvfrom( int              sock,
		char           * buffer,
		int              size,
		RTinet_address * address,
		RTinet_port    * port )
{
	int               count;
	sockaddr_in       source;
	SocketAddressSize source_len = static_cast<SocketAddressSize>(sizeof( source ));

	//lint -save -e740
	while( (count = INET_recvfrom( sock, buffer, size, 0,
			reinterpret_cast<SocketAddress *>(&source),
			&source_len )) < 0 )
	{
		//lint -restore

		int err = INET_error( sock );

#ifdef INET_EINTR
		if( err == INET_EINTR )
			continue;
#endif

#ifdef INET_EWOULDBLOCK
		if( err == INET_EWOULDBLOCK )
			return 0;
#endif

		return -1;
	}

	if( count > 0 )
	{
		RTMemoryUtil::memcpy( address, &source.sin_addr, static_cast<int>(sizeof( *address )) );
		*port = ntohs( source.sin_port );
	}

	return count;
}
