/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTMemoryUtil.h>
#include <RTinet.h>
#include <RTtcp.h>

extern "C" int RTinet_sendto( int                    sock,
		const char           * data,
		int                    size,
		const RTinet_address * address,
		RTinet_port            port )
{
	sockaddr_in addr;

	RTMemoryUtil::memset( &addr, 0, static_cast<int>(sizeof( addr )) );
	addr.sin_family = AF_INET;
	addr.sin_port   = htons( port );
	RTMemoryUtil::memcpy( &addr.sin_addr, address, static_cast<int>(sizeof( *address )) );

	int count;

	//lint -save -e740
	while( (count = INET_sendto( sock, data, size, 0,
			reinterpret_cast<SocketAddress *>(&addr),
			sizeof( addr ) )) < 0 )
	{
		//lint -restore

		int err = INET_error( sock );

#ifdef INET_EINTR
		if( err == INET_EINTR )
			continue;
#endif

#ifdef INET_EWOULDBLOCK
		if( err == INET_EWOULDBLOCK )
			return 0;
#endif

		return -1;
	}

	return count;
}
