/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTDaemon_h__
#define __RTDaemon_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTElasticArray_h__
#include <RTElasticArray.h>
#endif

#ifndef __RTEventInfo_h__
#include <RTEventInfo.h>
#endif

#include <RTMutex.h>

class RTDaemonInfo;
class RTDebugger;

class RTDaemon
{
public:
    enum SettingsType
    {
	SettHaltEnabled = RTEventInfo::EventLastEntry + 1,
	SettHaltSystem,
	SettDoReport,
	SettRecThreshold,

	SettLastEntry = SettRecThreshold
    };

private:
    RTDebugger     * debugger;
    RTElasticArray   actorInfo;
    RTMutex          lock;

    int              settings[ SettLastEntry + 1 ];

    int              posInfo_( RTDaemonInfo * );
    void             setInfo_( const RTEventInfo & );
    RTDaemonInfo   * eventMatches_( RTEventInfo & );
public:
    explicit         RTDaemon( RTDebugger * );
		     ~RTDaemon( void );

    void             updateSettings( const char * settingsString );

    int              addInfo( RTDaemonInfo * );
    int              delInfo( RTDaemonInfo * );
    int              posInfo( RTDaemonInfo * );

    RTDaemonInfo   * eventMatches( RTEventInfo & );

    void             setInfo( const RTEventInfo & );

    int              monitorField( RTDaemonInfo *, bool enable );
    void             fieldValue( const char * variable, const char * value );

    int              inject( int                    inwards,
			     const char           * signal,
			     int                    prio,
			     const void           * data,
			     const RTObject_class * type );

    int              setting( int settingNr ) const;
    void             refresh( int observerId, int daemonId );

    void             getMessages( RTDaemonInfo * );

    RTS_INLINE int   shouldHalt( void ) const;
    RTS_INLINE int   shouldReport( void ) const;

private:
    // unavailable methods
    RTDaemon( const RTDaemon & );
    RTDaemon & operator=( const RTDaemon & );
}; //lint !e1712

#if RTS_INLINES
#include <RTDaemon.inl>
#endif

#endif // __RTDaemon_h__
