/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTDiagStream_inl__
#define __RTDiagStream_inl__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTDiagStream_h__
#include <RTDiagStream.h>
#endif

#if USE_THREADS && ! defined( __RTMutex_h__ )
#include <RTMutex.h>
#endif

RTS_INLINE RTDiagStream & RTDiagStream::lock( void )
{
#if USE_THREADS
    _mutex->enter();
#endif
    return *this;
}

RTS_INLINE RTDiagStream & RTDiagStream::unlock( void )
{
#if USE_THREADS
    _mutex->leave();
#endif
    return *this;
}

RTS_INLINE RTDiagStream & operator<<( RTDiagStream & ds, const char * s )
{
    return ds.write( s );
}

RTS_INLINE RTDiagStream & operator<<( RTDiagStream & ds, char c )
{
    return ds << static_cast<long>(c);
}

RTS_INLINE RTDiagStream & operator<<( RTDiagStream & ds, unsigned char c )
{
    return ds << static_cast<unsigned long>(c);
}

RTS_INLINE RTDiagStream & operator<<( RTDiagStream & ds, short s )
{
    return ds << static_cast<long>(s);
}

RTS_INLINE RTDiagStream & operator<<( RTDiagStream & ds, unsigned short s )
{
    return ds << static_cast<unsigned long>(s);
}

RTS_INLINE RTDiagStream & operator<<( RTDiagStream & ds, int s )
{
    return ds << static_cast<long>(s);
}

RTS_INLINE RTDiagStream & operator<<( RTDiagStream & ds, unsigned s )
{
    return ds << static_cast<unsigned long>(s);
}

#endif // __RTDiagStream_inl__
