/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTImportList_h__
#define __RTImportList_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

class RTActor;
class RTActorRef;

// An array of these objects is allocated at the time an imported actor
// reference is initialized. For unused indices in the reference, the
// Unused variant is applied. As entries are reserved, the 'Used.reference'
// pointer is made to point at the containing reference. That pointer overlays
// the 'Unused.prev' pointer which can never point at a RTActorRef.
// The index into the reference is encoded as the difference between the
// pointer to a specific RTImportList and the beginning of the array in
// the reference. The 'Used.next' field is used to form the chain of
// contexts into which an actor has been imported.

union RTImportList
{
	struct Unused
	{
		RTImportList * next;
		RTImportList * prev;
	} unused;

	struct Used
	{
		RTImportList * next;
		RTActorRef   * reference;
		RTActor      * actor;
	} used;
};

#endif // __RTImportList_h__
