/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTLocalConnector_h__
#define __RTLocalConnector_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTDictionary_h__
#include <RTDictionary.h>
#endif

#ifndef __RTLayerConnector_h__
#include <RTLayerConnector.h>
#endif

class RTLocalConnector : public RTLayerConnector
{
public:
			 RTLocalConnector( void );
    virtual              ~RTLocalConnector( void );

    virtual RTLayerData * _registerSAP( RTController *,
					RTProtocol   *,
					const char   * ) override;

    virtual RTLayerData * _registerSPP( RTController *,
					RTProtocol   *,
					const char   * ) override;

    virtual void         _enumerate( void ) const override;

    // everything that follows should be considered private

    struct Request
    {
	Request( RTProtocol * _sap,
		 int          _start );

	Request    * prev;  // These two pointers are used to organize
	Request    * next;  // the requests in order of arrival in time.

	Request    * chain; // This pointer is used to organize all
			    // requests for a particular SAP.

	RTProtocol * sap;   // This is the SAP that would like to be bound.

	int          start; // This is the first replication index that
			    // would like to be bound.

	int          limit; // This is one more thant the last replication
			    // index that would like to be bound.
    }; //lint !e1712

    class SppNode;

    class SapNode : public RTLayerData
    {
    public:
			     SapNode( RTController *,
				      SppNode      *,
				      RTProtocol   * sap,
				      const char   * );
	virtual              ~SapNode( void );

	virtual const char * getRegisteredName( void ) const override;

	virtual void         resize( RTController *, int ) override;

	virtual void         deregister( RTController * ) override;

    public:
	Request              initial;
	SapNode            * prev;
	SapNode            * next;

    private:
	SppNode            * node;
	RTCachedString       name;
    }; //lint !e1712

    class SppNode : public RTLayerData
    {
    public:
	explicit             SppNode( RTLocalConnector * );

	virtual              ~SppNode( void );

	virtual const char * getRegisteredName( void ) const override;

	virtual void         resize( RTController *, int ) override;

	virtual void         deregister( RTController * ) override;

	RTLayerData         * publish( RTController *,
				       RTProtocol   *,
				       const char   * );

	int                  sapTotal( void ) const;
	int                  sppTotal( void ) const;

	void                 append( RTController *, Request * );
	void                 remove( RTController *, Request * );

	void                 append( SapNode * );
	void                 remove( RTController *, SapNode * );

    private:
	void                 connectAll( RTController * );

	RTProtocol         * spp;
	int                  spp_next;

	SapNode            * saps;

	Request            * requests;
	Request            * pending;

	RTLocalConnector   * connector;
	RTCachedString       name;
    }; //lint !e1712

private:
    RTDictionary /* of SppNode */ db;

    friend class SppNode;

    // unavailable methods
    RTLocalConnector( const RTLocalConnector & );
    RTLocalConnector & operator=( const RTLocalConnector & );
}; //lint !e1712

#endif // __RTLocalConnector_h__
