/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTMain.h>

#include <windows.h>
#if OBSERVABLE
#include <ctype.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#endif

int main( int argc, const char * const * argv )
{
#if OBSERVABLE
	if( argc <= 1 )
	{
		// Attempt to read program arguments from file. This occurs when
		// the toolset is running on NT, with TargetRTS selected, in VC50
		// debugging mode. Note: This is a hack to get around the fact that
		// the VC50 debugger cannot accept program arguments programatically.

		const long     ArgLimit = 32768L;
		const char   * ArgFile  = "TargetRTS.args";
		int            arg_fd   = _open( ArgFile, _O_RDONLY | _O_BINARY );
		struct _stat   info;
		char         * buffer   = nullptr;
		const char  ** argp     = nullptr;

		if( arg_fd == -1 )
		{
			// can't open file
		}
		else if( _fstat( arg_fd, &info ) == -1 )
		{
			// can't get info
		}
		else if( info.st_size > ArgLimit )
		{
			// too much data
		}
		else if( (buffer = new char [ static_cast<unsigned>(info.st_size) + 1U ]) == nullptr )
		{
			// can't allocate buffer
		}
		else if( info.st_size != _read( arg_fd, buffer, static_cast<unsigned>(info.st_size) ) )
		{
			// can't read data
		}
		else
		{
			buffer[ info.st_size ] = '\0';

			char * cursor = buffer;

			argc = 2;
			for(;;)
			{
				while( isspace( *cursor ) != 0 )
					++cursor;
				if( *cursor == '\0' )
					break;
				++argc;
				for( bool quoted = false; *cursor != '\0'; ++cursor )
				{
					if( *cursor == '"' )
						quoted = ! quoted;
					else if( ! quoted && isspace( *cursor ) != 0 )
						break;
				}
			}

			if( (argp = new const char * [ static_cast<unsigned>(argc) ]) == nullptr )
			{
				argc = 1; // not enough memory
				delete [] buffer;
				buffer = nullptr;
			}
			else
			{
				// we always have 1 arg, the app name
				argp[ 0 ] = argv[ 0 ];

				argc   = 0;
				cursor = buffer;
				for(;;)
				{
					while( isspace( *cursor ) != 0 )
						++cursor;
					if( *cursor == '\0' )
						break;
					argp[ ++argc ] = cursor;

					char * writec = cursor;
					for( bool quoted = false; *cursor != '\0'; ++cursor )
					{
						if( *cursor == '"' )
							quoted = ! quoted;
						else if( quoted || isspace( *cursor ) == 0 )
							*writec++ = *cursor;
						else
							break;
					}
					char separator = *cursor++;
					*writec = '\0';
					if( separator == '\0' )
						break;
				}
				argp[ ++argc ] = nullptr;

				argv = argp;
			}
		}

		if( arg_fd != -1 )
			(void)_close( arg_fd );

		int rc = RTMain::entryPoint( argc, argv );

		// cleanup

		delete [] argp;
		delete [] buffer;

		return rc;
	}
#endif // OBSERVABLE

	return RTMain::entryPoint( argc, argv );
}
