/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTDiag.h>
#include <RTJob.h>
#include <RTThread.h>
#include <RTThreadInfo.h>

static void thread_run( RTJob * job )
{
	job->mainLoop();
}

RTThread::RTThread( RTJob * job, int stack_size, int prio )
: _job ( job )
, _info( new RTThreadInfo )
{
	HANDLE handle;
	DWORD  tid;

	if( (handle = CreateThread( nullptr,
			static_cast<DWORD>(stack_size),
			reinterpret_cast<LPTHREAD_START_ROUTINE>(thread_run),
			job,
			CREATE_SUSPENDED, &tid )) == nullptr )
		RTDiag::panic( "RTThread(create)" );
	else if( SetThreadPriority( handle, THREAD_PRIORITY_NORMAL + prio ) == 0 )
		RTDiag::panic( "RTThread(priority)" );
	else if( ResumeThread( handle ) != static_cast<DWORD>(1) )
		RTDiag::panic( "RTThread(resume)" );
	else
		_info->handle = handle;
}

RTThreadInfo::RTThreadInfo( void )
: handle( nullptr )
{
}

RTThreadInfo::~RTThreadInfo( void )
{
	if( handle != nullptr )
	{
		if( WaitForSingleObject( handle, static_cast<DWORD>(INFINITE) ) == WAIT_FAILED )
			RTDiag::panic( "RTThreadInfo(wait)" );

		if( CloseHandle( handle ) == 0 )
			RTDiag::panic( "RTThreadInfo(close)" );

#ifdef _lint
		handle = nullptr;
#endif
	}
}
