/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#include <RTDiag.h>
#include <RTTimespec.h>

#include <windows.h>

void RTTimespec::getclock( RTTimespec & ts )
{
    SYSTEMTIME now_system;
    FILETIME   now_file;

    typedef unsigned __int64 u64;

    GetSystemTime( &now_system );
    if( SystemTimeToFileTime( &now_system, &now_file ) == 0 )
	RTDiag::panic( "SystemTimeToFileTime" );

    u64 ticks100ns;

    ticks100ns   = static_cast<u64>(now_file.dwHighDateTime);
    ticks100ns <<= 16;
    ticks100ns <<= 16;
    ticks100ns  += static_cast<u64>(now_file.dwLowDateTime);

    // There were 11644473600 seconds from January 1, 1601 to January 1, 1970.
    // Truncated to 32 bits that is -1240428288. We must subtract that bias
    // to produce a result based from the beginning of 1970.

    ts.tv_sec  = 1240428288L + static_cast<long>( ticks100ns / static_cast<u64>(10000000UL) );
    ts.tv_nsec =        100L * static_cast<long>( ticks100ns % static_cast<u64>(10000000UL) );
}
