/*
 * Licensed Materials - Property of HCL and/or IBM
 * Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
 * Copyright IBM Corporation 1999, 2016. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights -  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule.
 */

#ifndef __RTtcp_h__
#define __RTtcp_h__ included

#include <windows.h>
#include <winsock.h>

typedef struct sockaddr SocketAddress;
typedef int             SocketAddressSize;

#define INET_HaveLength 0

#define INET_accept( sock, addr, port ) \
			 (static_cast<int>( accept( static_cast<SOCKET>(sock), (addr), (port) ) ) )

#define INET_bind( sock, addr, size ) \
			 ( bind( static_cast<SOCKET>(sock), (addr), (size) ) )

#define INET_close( sock ) \
			 ( closesocket( static_cast<SOCKET>(sock) ) )

#define INET_connect( sock, addr, size ) \
			 ( connect( static_cast<SOCKET>(sock), (addr), (size) ) )

#define INET_error( sock ) \
			 WSAGetLastError()

#define INET_getsockname( sock, addr, size ) \
			 ( getsockname( static_cast<SOCKET>(sock), (addr), (size) ) )

#define INET_ioctl( sock, cmd, arg )                          \
			 ioctlsocket( static_cast<SOCKET>(sock), (cmd),  \
				      reinterpret_cast<unsigned long *>(arg) )
#define INET_listen( sock, backlog ) \
			 ( listen( static_cast<SOCKET>(sock), (backlog) ) )

#define INET_lookup      gethostbyname

#define INET_recv( sock, addr, size, flags ) \
			 ( recv( static_cast<SOCKET>(sock), (addr), (size), (flags) ) )

#define INET_recvfrom( sock, buf, size, flags, src, src_len ) \
			 ( recvfrom( static_cast<SOCKET>(sock), (buf), (size), (flags), \
				     (src), (src_len) ) )

#define INET_send( sock, addr, size, flags ) \
			 ( send( static_cast<SOCKET>(sock), (addr), (size), (flags) ) )

#define INET_sendto( sock, buf, size, flags, dst, dst_len ) \
			 ( sendto( static_cast<SOCKET>(sock), (buf), (size), (flags), \
				   (dst), (dst_len) ) )

#define INET_setsockopt( sock, level, cmd, value, size ) \
			 ( setsockopt( static_cast<SOCKET>(sock),   \
				       (level),          \
				       (cmd),            \
				       (value),          \
				       (size) ) )

#define INET_socket( domain, type, protocol ) \
			 ( static_cast<int>(socket( (domain), (type), (protocol) )) )

#define INET_EALREADY    WSAEALREADY
#define INET_EINPROGRESS WSAEINPROGRESS
#define INET_EINTR       WSAEINTR
#define INET_EISCONN     WSAEISCONN
#define INET_EWOULDBLOCK WSAEWOULDBLOCK

// Microsoft says this shouldn't happen when we specify WSA_FLAG_OVERLAPPED,
// but it does anyway!
#define INET_EMAYBECONN  WSAEINVAL

#endif // __RTtcp_h__
