#! perl -w

# Licensed Materials - Property of HCL and/or IBM
# Copyright HCL Technologies Ltd. 2016, 2021. All Rights Reserved.
# Copyright IBM Corporation 1999, 2016. All Rights Reserved.
#
# U.S. Government Users Restricted Rights -  Use, duplication or 
# disclosure restricted by GSA ADP Schedule.



sub fatal
{
    print STDERR "$_[0]\n";
    exit 1;
}

$#ARGV >= 0 || &fatal( "Usage: ${0} source-file ..." );

binmode( STDOUT );

@pragmas = ();

foreach $in ( @ARGV )
{
    open( IN, $in ) || &fatal( "$in: $!" );

    while( <IN> )
    {
	if( m{^#\s*pragma\b} )
	{
	    push @pragmas, $_;
	}
    }

    close( IN  ) || &fatal( "$in: $!" );
}

if( $#pragmas >= 0 )
{
    print "#ifdef PRAGMA\n", @pragmas, "#endif\n";
}

foreach $in ( @ARGV )
{
    open( IN, $in ) || &fatal( "$in: $!" );

    $level = 0;
    while( <IN> )
    {
	if( m{^#\s*ifdef\s+PRAGMA} )
	{
	    ++$level;
	}
	elsif( $level > 0 )
	{
	    --$level if( m{^#\s*endif} );
	}
	else
	{
	    s{^#\s*include\s*<(.*)>.*$}{#include "$1"};
	    print;
	}
    }

    close( IN  ) || &fatal( "$in: $!" );
}
