<?xml version="1.0" encoding="UTF-8"?>
<!--                                                                        -->
<!-- Licensed Materials - Property of IBM                                   -->
<!-- Copyright IBM Corporation 2007. All Rights Reserved.             	   	-->
<!--                                                                        -->
<!-- Note to U.S. Government Users Restricted Rights:  Use, duplication or  -->
<!-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      -->
<!--                                                                        -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:publish="http://www.ibm.com/Rational/XTools/Publish" >
    
 
    <!-- Localization support -->
    <xsl:include href="./l10n/trace_localize.xsl" />
    
    <!--Link Table generation 
    <xsl:include href="./trace_utility.xsl" />
    -->
    <!-- The file name for localization lookup, default  is English -->
    <xsl:param name="trace_stringTableFileName" select="'./trace_messages.xml'"/> 
    
   <!-- ################## MAIN TEMPLATE CALL #####################-->
    <!-- Notes
    *
    *  They main template needs to have a "mode" select param as well as a "choose"
    *  switch for 'link', 'details', and 'summary'
    *
    -->    
    <xsl:template name="createTraceRelationships">	
        <xsl:param name="mode" select="''" />
        <xsl:if test="count(publish:trace_relationship_refs) &gt; 0">
            <xsl:choose>
                <!-- TOP/BOTTOM PAGE LINK -->
                <xsl:when test="$mode = 'link'">
                    <publish:bookmarklink>
                        <xsl:attribute name="name">
                            <xsl:call-template name="trace_getLocalizedString">
                                <xsl:with-param name="trace_key" select="'Specifications'"/>
                            </xsl:call-template>
                        </xsl:attribute>
                        <xsl:attribute name="link">
                            <xsl:value-of
                                select="concat('#', 'specificationsSummary')" />
                        </xsl:attribute>
                        <xsl:attribute name="title">
                            <xsl:call-template name="trace_getLocalizedString">
                                <xsl:with-param name="trace_key" select="'Specifications'"/>
                            </xsl:call-template>
                        </xsl:attribute>
                    </publish:bookmarklink>
                </xsl:when>
                <xsl:when test="$mode = 'details'">
                </xsl:when>            
                <xsl:when test="$mode = 'summary'">				
                    <xsl:apply-templates select="publish:trace_relationship_refs"/>				
                </xsl:when>
                <xsl:when test="$mode = 'detailsTable'">
                </xsl:when>            
            </xsl:choose>
        </xsl:if>				
    </xsl:template>
    <!-- ################## END MAIN TEMPLATE CALL #####################-->
    
     
    <!-- ***************************************************-->        
    <!-- Build Specifications Summary Table                         -->
    <!-- ***************************************************-->        
    <xsl:template name="outputTraceRelationships" match="publish:trace_relationship_refs">
        <xsl:param name="indentCount" select="0"/>
            
        <!-- Localization variables -->        
        <xsl:variable name="specificationsTitle">
            <xsl:call-template name="trace_getLocalizedString">
                <xsl:with-param name="trace_key" select="'Specifications'"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="name">
            <xsl:call-template name="trace_getLocalizedString">
                <xsl:with-param name="trace_key" select="'Specification'"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="description">
            <xsl:call-template name="trace_getLocalizedString">
                <xsl:with-param name="trace_key" select="'TraceRelationship'"/>
            </xsl:call-template>
        </xsl:variable>
        
        <!--Specifications Table -->
        <p />
        <table style="width: 100%; height: 40px;" border="1" cellpadding="4" class="PropertiesTable" >
            <tr>
                <td class="SummaryTitle" colspan="2">
                    <a name="specificationsSummary">
                        <xsl:copy-of select="$specificationsTitle" />
                    </a>
                </td>
            </tr>
            <tr>  
                <td  class="PropertyName" style="width: 50%;"><b><xsl:value-of select="$name"/></b></td>
                <td  class="PropertyName" style="width: 50%;"><b><xsl:value-of select="$description"/></b></td>
            </tr>                
            <xsl:for-each select="publish:trace_relationship_ref">
                <!-- Loop and build a table of trace relationships -->
                <tr>  
                    <td  class="PropertyName" style="width: 50%;">
                        <xsl:for-each select="publish:specification_ref">
                            <xsl:call-template
                                name="outputImageAndTextLinkHTML">
                                <xsl:with-param name="link">
                                    <xsl:choose>
										<xsl:when test="@publish:specification_id != ''">
 											<xsl:value-of select="concat(@publish:specification_id, &apos;-content.html&apos;)"/>
										</xsl:when>
										<xsl:when test="@publish:specification_urllink != ''">
											<xsl:value-of select="@publish:specification_urllink"/>
										</xsl:when>
									</xsl:choose>
                                </xsl:with-param>
                                <xsl:with-param name="image">
                                    <xsl:if
                                        test="@publish:icon != ''">
                                        <xsl:call-template
                                            name="createElementIconFileName" />
                                    </xsl:if>
                                </xsl:with-param>
                                <xsl:with-param name="text"
                                    select="@publish:specification_name" />
                            </xsl:call-template>
                        </xsl:for-each>
                    </td>
                    <td  class="PropertyValue" style="width: 50%;">
                        <xsl:call-template
                            name="outputImageAndTextHTML">
                            <xsl:with-param name="image">
                                <xsl:if
                                    test="@publish:icon != ''">
                                    <xsl:call-template
                                        name="createElementIconFileName" />
                                </xsl:if>
                            </xsl:with-param>
                            <xsl:with-param name="text"
                                select="@publish:trace_relationship_name" />
                        </xsl:call-template>
                    </td>
                </tr>
            </xsl:for-each>
        </table>
    </xsl:template>
    
    <!-- ***************************************************-->        
    <!-- Generate the HTML for combined image and text      -->
    <!--                                                    -->
    <!-- Parameters:                                        -->
    <!--   image  - The image to display before the text    -->
    <!--   text   - The text to display next to the image   -->
    <!-- ***************************************************-->        
    <xsl:template name="outputImageAndTextHTML">
        <xsl:param name="image" select="''"/>
        <xsl:param name="text" select="@name"/>
        <xsl:param name="ttip" select="@publish:qualifiedname"/>
        <xsl:variable name="label">
            <xsl:if test="$text != ''">
                <xsl:value-of select="$text"/>
            </xsl:if>
            <xsl:if test="$text = ''">
                <xsl:text>Unnamed</xsl:text>
            </xsl:if>
        </xsl:variable>
        
        <!-- Create an image -->
        <xsl:if test="$image != ''">
            <img border="0">
                <xsl:attribute name="src">
                    <xsl:value-of select="$image"/>
                </xsl:attribute>
                <xsl:attribute name="alt">
                    <xsl:value-of select="$label"/>
                </xsl:attribute>
            </img>
            <xsl:text>&#160;</xsl:text>
        </xsl:if>
        
        <!-- Create the text -->
        <a>
            <xsl:if test="$ttip != ''">
                <xsl:attribute name="title">
                    <xsl:value-of select="$ttip"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:value-of select="$label"/>
        </a>
    </xsl:template>
   
</xsl:stylesheet>
